@@ -10,11 +10,12 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pdflib_pl.c,v 1.52.2.1 2006/10/11 14:36:29 rjs Exp $
+/* $Id: pdflib_pl.c,v 1.52.2.6 2010/03/31 21:38:00 rjs Exp $
  *
  * Wrapper code for the PDFlib Perl binding
  *
  */
+#undef PERL_IMPLICIT_SYS 
 
 /* some defines for SWIG {{{ */
 /*
@@ -39,8 +40,9 @@ extern "C" {
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
-#undef free
+/*#undef free
 #undef malloc
+*/
 #include <string.h>
 
 #ifdef __cplusplus
@@ -360,6 +362,7 @@ char *_SWIG_GetPtr(CPerlObj *pPerl, SV *sv, void **ptr, char *_t)
 #ifdef __cplusplus
 extern "C"
 #endif
+
 #ifndef PERL_OBJECT
 /* If we don't use the XS macro it doesn't work with all Perl configurations */
 SWIGEXPORT(/* */, XS(boot_pdflib_pl));
@@ -396,9 +399,7 @@ XS(_wrap_PDF_new) { /* {{{ */
 
 
     if (_result) {
-#if defined(ACTIVEPERL_VERSION)
-	sprintf(versionbuf, "ASperl %s", PERLFILEVERSION);
-#elif defined(PERL_REVISION) && defined(PERL_VERSION) && defined(PERL_SUBVERSION)
+#if defined(PERL_REVISION) && defined(PERL_VERSION) && defined(PERL_SUBVERSION)
 	sprintf(versionbuf, "Perl %d.%d.%d",
 	    PERL_REVISION, PERL_VERSION, PERL_SUBVERSION);
 #else
@@ -552,7 +553,7 @@ XS(_wrap_PDF_utf16_to_utf8) { /* {{{ */
     dXSARGS ;
 
     if (items != 2)
-        croak("Usage: _wrap_PDF_utf16_to_utf8(p, utf16string);");
+        croak("Usage: PDF_utf16_to_utf8(p, utf16string);");
     if (SWIG_GetPtr(ST(0),(void *) &p,"PDFPtr")) {
         croak("Type error in argument 1 of PDF_utf16_to_utf8. Expected PDFPtr.");
         XSRETURN(1);
@@ -576,7 +577,7 @@ XS(_wrap_PDF_utf8_to_utf16) { /* {{{ */
     dXSARGS ;
 
     if (items != 3)
-        croak("Usage: _wrap_PDF_utf8_to_utf16(p, utf8string, format);");
+        croak("Usage: PDF_utf8_to_utf16(p, utf8string, ordering);");
     if (SWIG_GetPtr(ST(0),(void *) &p,"PDFPtr")) {
         croak("Type error in argument 1 of PDF_utf8_to_utf16. Expected PDFPtr.");
         XSRETURN(1);
@@ -593,6 +594,76 @@ XS(_wrap_PDF_utf8_to_utf16) { /* {{{ */
 } /* }}} */
 
 
+XS(_wrap_PDF_utf32_to_utf16) { /* {{{ */
+    PDF * p;
+    const char * utf32string;
+    size_t len;
+    const char * ordering;
+    int size;
+    const char *_result = NULL;
+    dXSARGS;
+
+    if (items != 3) {
+        croak("Usage: PDF_utf32_to_utf16(p, utf32string, ordering);");
+    }
+    if (SWIG_GetPtr(ST(0),(void *) &p, "PDFPtr")) {
+croak("Type error in argument 1 of PDF_utf32_to_utf16. Expected PDPDFr.");
+        XSRETURN(1);
+    }
+
+    utf32string = SvPV(ST(1), len);
+    ordering = SvPV(ST(2), PL_na);
+
+    try {
+        _result = PDF_utf32_to_utf16(p, utf32string, (int)len, ordering, &size);
+    } catch;
+
+    ST(0) = sv_newmortal();
+    sv_setpvn((SV*)ST(0), (char *) _result, size);
+    XSRETURN(1);
+} /* }}} */
+
+
+
+XS(_wrap_PDF_show_boxed) { /* {{{ */
+    PDF * p;
+    const char * text;
+    size_t text_len;
+    double left;
+    double top;
+    double width;
+    double height;
+    const char * hmode;
+    const char * feature;
+    int _result = -1;
+    dXSARGS;
+
+    if (items != 8) {
+	croak("Usage: PDF_show_boxed(p, text, left, top, width, height, hmode, feature);");
+    }
+    if (SWIG_GetPtr(ST(0),(void *) &p, "PDFPtr")) {
+	croak("Type error in argument 1 of PDF_show_boxed. Expected PDFPtr.");
+	XSRETURN(1);
+    }
+
+    text = SvPV(ST(1), text_len);
+    left = SvNV(ST(2));
+    top = SvNV(ST(3));
+    width = SvNV(ST(4));
+    height = SvNV(ST(5));
+    hmode = SvPV(ST(6), PL_na);
+    feature = SvPV(ST(7), PL_na);
+
+    try {
+        _result = PDF_show_boxed(p, text, left, top, width, height, hmode, feature);
+    } catch;
+    
+    ST(0) = sv_newmortal();
+    sv_setiv(ST(0),(IV) _result);
+    XSRETURN(1);
+} /* }}} */
+
+    
 /* BEGIN created with wrap.pl */
 #define _WRAP_CODE	1
 #include "pl_wrapped.c"
@@ -622,7 +693,9 @@ XS(boot_pdflib_pl) { /* {{{ */
     newXS("pdflibc::PDF_setpolydash", _wrap_PDF_setpolydash, file);
     newXS("pdflibc::PDF_open_pdi", _wrap_PDF_open_pdi, file);
     newXS("pdflibc::PDF_open_image", _wrap_PDF_open_image, file);
+    newXS("pdflibc::PDF_show_boxed", _wrap_PDF_show_boxed, file);
     newXS("pdflibc::PDF_utf16_to_utf8", _wrap_PDF_utf16_to_utf8, file);
+    newXS("pdflibc::PDF_utf32_to_utf16", _wrap_PDF_utf32_to_utf16, file);
     newXS("pdflibc::PDF_utf8_to_utf16", _wrap_PDF_utf8_to_utf16, file);
 
     /* BEGIN created with wrap.pl */
