@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: hello.py,v 1.20 2006/10/02 21:17:37 tm Exp $
+# $Id: hello.py,v 1.20.2.4 2009/03/02 14:14:07 rjs Exp $
 #
 # PDFlib client: hello example in Python
 #
@@ -16,7 +16,7 @@ try:
     PDF_set_parameter(p, "errorpolicy", "return")
 
     if PDF_begin_document(p, "hello.pdf", "") == -1:
-	raise Exception("Error: " + PDF_get_errmsg(p))
+        raise Exception("Error: " + PDF_get_errmsg(p))
 
     PDF_set_info(p, "Author", "Thomas Merz")
     PDF_set_info(p, "Creator", "hello.py")
@@ -26,7 +26,7 @@ try:
 
     font = PDF_load_font(p, "Helvetica-Bold", "unicode", "")
     if font == -1:
-	raise PDFlibException("Error: " + PDF_get_errmsg(p))
+        raise PDFlibException("Error: " + PDF_get_errmsg(p))
 
     PDF_setfont(p, font, 24)
     PDF_set_text_pos(p, 50, 700)
@@ -34,17 +34,14 @@ try:
     PDF_continue_text(p, "(says Python)")
     PDF_end_page_ext(p, "")
 
-    PDF_end_document(p, "")	
+    PDF_end_document(p, "")     
 
   except PDFlibException:
-    print "PDFlib Exception occurred:"
-    print "[" + repr(PDF_get_errnum(p)) + "] " + PDF_get_apiname(p) + ": " \
-	    + PDF_get_errmsg(p)
+    print("PDFlib exception occurred:\n[%d] %s: %s" %
+	((PDF_get_errnum(p)), PDF_get_apiname(p),  PDF_get_errmsg(p)))
 
-  except Exception, e:
-    print "Exception occurred:"
-    print e
+  except Exception:
+    print("Exception occurred: %s" % (exc_info()[0]))
 
 finally:
     PDF_delete(p)
-
