@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: image.py,v 1.17 2006/10/02 21:17:37 tm Exp $
+# $Id: image.py,v 1.17.2.3 2009/03/02 14:14:07 rjs Exp $
 #
 # PDFlib client: image example in Python
 #
@@ -23,7 +23,7 @@ try:
     PDF_set_parameter(p, "SearchPath", searchpath)
 
     if PDF_begin_document(p, "image.pdf", "") == -1:
-	raise PDFlibException("Error: " + PDF_get_errmsg(p))
+        raise PDFlibException("Error: " + PDF_get_errmsg(p))
 
     PDF_set_info(p, "Creator", "image.py")
     PDF_set_info(p, "Author", "Thomas Merz")
@@ -31,7 +31,7 @@ try:
 
     image = PDF_load_image(p, "auto", imagefile, "")
     if image == -1:
-	raise PDFlibException("Error: " + PDF_get_errmsg(p))
+        raise PDFlibException("Error: " + PDF_get_errmsg(p))
 
     # dummy page size, will be adjusted by PDF_fit_image()
     PDF_begin_page_ext(p, 10, 10, "")
@@ -42,14 +42,11 @@ try:
     PDF_end_document(p, "")
 
   except PDFlibException:
-    print "PDFlib Exception occurred:"
-    print "[" + repr(PDF_get_errnum(p)) + "] " + PDF_get_apiname(p) + ": " \
-	    + PDF_get_errmsg(p)
+    print("PDFlib exception occurred:\n[%d] %s: %s" %
+	((PDF_get_errnum(p)), PDF_get_apiname(p),  PDF_get_errmsg(p)))
 
-  except Exception, e:
-    print "Exception occurred:"
-    print e
+  except Exception:
+    print("Exception occurred: %s" % (exc_info()[0]))
 
 finally:
     PDF_delete(p)
-
