@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-# $Id: hello.rb,v 1.6.2.3 2008/02/26 11:09:27 rjs Exp $
+# $Id: hello.rb,v 1.6.2.4 2008/02/28 14:49:59 rjs Exp $
 #
 # PDFlib client: hello example in Ruby
 #
@@ -37,7 +37,6 @@ begin
   p.end_page_ext("")
 
   p.end_document("")
-  p.delete()
 
 rescue  PDFlibException => pe
   print "PDFlib exception occurred in hello sample:\n"
@@ -45,6 +44,8 @@ rescue  PDFlibException => pe
 		  ": " + pe.get_errmsg + "\n"
 rescue  Exception => e
   print e.backtrace.join("\n") + "\n" + e.to_s
+ensure
+  p.delete() if p
 end
 
 # vim: sw=2
