@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-#  $Id: image.rb,v 1.3.2.2 2008/02/26 11:09:27 rjs Exp $
+#  $Id: image.rb,v 1.3.2.3 2008/02/28 14:49:59 rjs Exp $
 #
 #  PDFlib client: image example in Ruby
 # 
@@ -39,7 +39,6 @@ begin
   p.end_page_ext("")
 
   p.end_document("")
-  p.delete()
 
 rescue  PDFlibException => pe
   print "PDFlib exception occurred in image sample:\n"
@@ -47,6 +46,8 @@ rescue  PDFlibException => pe
 		  ": " + pe.get_errmsg + "\n"
 rescue Exception => e
   print e.backtrace.join("\n") + "\n" + e.to_s
+ensure
+  p.delete() if p
 end
 
 # vim: sw=2
