@@ -1592,21 +1592,6 @@ show(VALUE self, VALUE text)
 }
 
     
-/* Deprecated, use PDF_fit_textline() or PDF_fit_textflow().. */
-static VALUE
-show_boxed(VALUE self, VALUE text, VALUE left, VALUE top, VALUE width, VALUE height, VALUE hmode, VALUE feature)
-{
-    PDF *pdf = get_pdf(self);
-    int ret = 0;
-
-    pdf_try {
-	ret = PDF_show_boxed2(pdf, CHAR(text), LEN(text), NUM2DBL(left), NUM2DBL(top), NUM2DBL(width), NUM2DBL(height), CHAR(hmode), CHAR(feature));
-    } pdf_catch;
-    
-    return INT2NUM(ret);
-}
-
-    
 /* Same as PDF_show_xy() but with explicit string length.. */
 static VALUE
 show_xy(VALUE self, VALUE text, VALUE x, VALUE y)
@@ -1802,7 +1787,6 @@ translate(VALUE self, VALUE tx, VALUE ty)
     rb_define_method(cPDFlib, "shading_pattern", shading_pattern, 2);
     rb_define_method(cPDFlib, "shfill", shfill, 1);
     rb_define_method(cPDFlib, "show", show, 1);
-    rb_define_method(cPDFlib, "show_boxed", show_boxed, 7);
     rb_define_method(cPDFlib, "show_xy", show_xy, 3);
     rb_define_method(cPDFlib, "skew", skew, 2);
     rb_define_method(cPDFlib, "stringwidth", stringwidth, 3);
