@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pdflib_tcl.c,v 1.56.2.1 2008/01/09 15:55:31 kurt Exp $
+/* $Id: pdflib_tcl.c,v 1.56.2.3 2009/04/21 10:35:23 rjs Exp $
  *
  * in sync with pdflib.h 1.151.2.22
  *
@@ -742,6 +742,93 @@ _wrap_PDF_setpolydash(ClientData clientData, Tcl_Interp *interp, int objc,
 }
 
 
+static int
+_wrap_PDF_show_boxed(ClientData clientData, Tcl_Interp *interp,
+    int objc, Tcl_Obj *CONST objv[])
+{
+    PDF *p;
+    char *pdf;
+    const char * text = NULL;
+    int text_len;
+    double left;
+    double top;
+    double width;
+    double height;
+    const char * hmode = NULL;
+    int hmode_len;
+    const char * feature = NULL;
+    int feature_len;
+    int volatile _retval = -1;
+
+    if (objc != 9)
+    {
+	PDF_WrongCommand(interp,
+                "PDF_show_boxed p text left top width height hmode feature");
+	return TCL_ERROR;
+    }
+
+    if ((pdf = Tcl_GetStringFromObj(objv[1], NULL)) == NULL)
+    {
+	PDF_NoPDFHandle(interp, "PDF_show_boxed");
+        return TCL_ERROR;
+    }
+
+    if (SWIG_GetPtr(pdf, (void **) &p, "_PDF_p"))
+    {
+	PDF_WrongPDFHandle(interp, "PDF_show_boxed");
+        Tcl_AppendResult(interp, pdf, (char *) NULL);
+	return TCL_ERROR;
+    }
+    if ((text = PDF_GetStringFromObj(p, interp, objv[2], PDF_UTF16, &text_len))
+                        == NULL)
+    {
+	PDF_WrongParameter(interp, "text in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetDoubleFromObj(interp, objv[3], &left) != TCL_OK)
+    {
+	PDF_WrongParameter(interp, "left in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetDoubleFromObj(interp, objv[4], &top) != TCL_OK)
+    {
+	PDF_WrongParameter(interp, "top in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetDoubleFromObj(interp, objv[5], &width) != TCL_OK)
+    {
+	PDF_WrongParameter(interp, "width in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if (Tcl_GetDoubleFromObj(interp, objv[6], &height) != TCL_OK)
+    {
+	PDF_WrongParameter(interp, "height in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if ((hmode = PDF_GetStringFromObj(p, interp, objv[7], PDF_BYTES,
+                        &hmode_len)) == NULL)
+    {
+	PDF_WrongParameter(interp, "hmode in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+    if ((feature = PDF_GetStringFromObj(p, interp, objv[8], PDF_BYTES,
+                        &feature_len)) == NULL)
+    {
+	PDF_WrongParameter(interp, "feature in PDF_show_boxed");
+	return TCL_ERROR;
+    }
+
+    try {
+	_retval = PDF_show_boxed2(p, text, text_len, left, top, width, height,
+                                hmode, feature);
+    } catch;
+
+    sprintf(interp->result, "%d", _retval);
+
+    return TCL_OK;
+}
+
+
 
 #define _WRAP_CODE
 #include "tcl_wrapped.c"
@@ -780,6 +867,11 @@ SWIGEXPORT(int,Pdf_tcl_SafeInit)(Tcl_Interp *interp)
 
 SWIGEXPORT(int,Pdflib_Init)(Tcl_Interp *interp)
 {
+    char short_version[6];
+
+    strncpy(short_version, PDFLIB_VERSIONSTRING, 5);
+    short_version[5] = 0;
+
     if (interp == 0)
 	return TCL_ERROR;
 
@@ -794,12 +886,15 @@ SWIGEXPORT(int,Pdflib_Init)(Tcl_Interp *interp)
 #endif
 
     /* Tell Tcl which package we are going to define */
-    Tcl_PkgProvide(interp, "pdflib", PDFLIB_VERSIONSTRING);
+    Tcl_PkgProvide(interp, "pdflib", short_version);
 
     Tcl_CreateObjCommand(interp, "PDF_open_pdi",
                 (Tcl_ObjCmdProc*) _wrap_PDF_open_pdi,
                 (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 
+    Tcl_CreateObjCommand(interp, "PDF_show_boxed", (Tcl_ObjCmdProc*)
+	_wrap_PDF_show_boxed, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+
 #define _WRAP_METHODS
 #include "tcl_wrapped.c"
 #undef _WRAP_METHODS
