@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: ft_truetype.c,v 1.131.2.47 2008/02/21 10:27:50 kurt Exp $
+/* $Id: ft_truetype.c,v 1.131.2.59 2009/02/16 12:30:39 kurt Exp $
  *
  * FONT TrueType handling routines
  *
@@ -471,6 +471,8 @@ tt_get_tab_cmap(tt_file *ttf)
                             pdc_malloc(pdc, sizeof (tt_cmap0_6), fn);
 		tp->mac->format = 0;
                 tt_get_cmap0(ttf, tp->mac);
+
+                offset_mac = offsetEncTab;
 	    }
 	    else if (tableFormat == 6 && tp->mac == (tt_cmap0_6 *) 0)
 	    {
@@ -478,25 +480,38 @@ tt_get_tab_cmap(tt_file *ttf)
                             pdc_malloc(pdc, sizeof (tt_cmap0_6), fn);
 		tp->mac->format = 6;
                 tt_get_cmap6(ttf, tp->mac);
+
+                offset_mac = offsetEncTab;
 	    }
-            offset_mac = offsetEncTab;
+            else if (numEncTabs == 1 && tableFormat == 4)
+            {
+                /* simulating Windows */
+                tp->win = (tt_cmap4 *) pdc_malloc(pdc, sizeof (tt_cmap4), fn);
+                tp->win->format = tableFormat;
+                tp->win->encodingID = encodingID;
+                tp->win = tt_get_cmap4(ttf, tp->win);
+
+                /* cmap picking not necessary */
+            }
 	}
 
         /*
-         * platformID: 0  encodingID: 3    tableFormat: 4  (old mac)
-         * platformID: 3  encodingID: 0/1  tableFormat: 4  preferred!
+         * platformID: 0  encodingID: 3     tableFormat: 4  (old mac)
+         * platformID: 3  encodingID: 0/1/4 tableFormat: 4  preferred!
          */
         else if (tableFormat == 4 &&
                  ((platformID == tt_pfid_win &&
                    (encodingID == tt_wenc_symbol ||
-                    encodingID == tt_wenc_text)) ||
+                    encodingID == tt_wenc_text ||
+                    encodingID == tt_wenc_big5)) ||
                   (platformID == tt_pfid_uni &&
                    encodingID == tt_wenc_mtext)))
         {
             if (tp->win == (tt_cmap4 *) 0 ||
                 (tp->win != (tt_cmap4 *) 0 &&
-                 tp->win->encodingID == tt_wenc_mtext &&
-                 encodingID < tt_wenc_mtext))
+                 ((tp->win->encodingID == tt_wenc_mtext ||
+                   tp->win->encodingID == tt_wenc_big5) &&
+                  encodingID < tt_wenc_mtext)))
             {
                 if (tp->win != (tt_cmap4 *) 0)
                     tt_cleanup_cmap4(ttf, tp->win);
@@ -525,7 +540,7 @@ tt_get_tab_cmap(tt_file *ttf)
     ttf->issymbol = (tp->win && tp->win->encodingID == tt_wenc_symbol) ?
                     pdc_true : pdc_false;
 
-    /* has Unicode cmap */
+    /* has Unicode/CMap cmap */
     ttf->haswinuni = (!ttf->issymbol && (tp->win || tp->ucs4)) ?
                      pdc_true : pdc_false;
 
@@ -560,6 +575,7 @@ tt_get_tab_cmap(tt_file *ttf)
         }
         else
         {
+            ttf->hasbig5cmap = tp->win->encodingID == tt_wenc_big5;
             tp->encoding = tt_wenc_text;
             tp->format = tp->win->format;
             tp->offset = offset_win;
@@ -573,12 +589,16 @@ tt_get_tab_cmap(tt_file *ttf)
         tp->platform, tp->encoding, tp->format,
         ttf->issymbol ? "symbol" : "text");
 
+    if (ttf->hasbig5cmap)
+        pdc_logg_cond(ttf->pdc, 1, trc_font, "\t\tCID font with Big5 cmap\n");
+
     /* for subsetting and symbolic font:
      * tp->platform = tt_pfid_mac according PDF specification
      * otherwise GS will emit an error message
      */
     if (ttf->issymbol && offset_mac > 0)
     {
+        ttf->forcesubset = !ttf->hasonlymac;
         tp->platform = tt_pfid_mac;
         tp->encoding = tt_wenc_symbol;
         tp->format = tp->mac->format;
@@ -1044,6 +1064,7 @@ void
 tt_get_tab_OS_2(tt_file *ttf)
 {
     pdc_bool logg3 = pdc_logg_is_enabled(ttf->pdc, 3, trc_font);
+    pdc_bool logg5 = pdc_logg_is_enabled(ttf->pdc, 5, trc_font);
     int i, j;
 
     tt_tab_OS_2 *tp = NULL;
@@ -1103,7 +1124,8 @@ tt_get_tab_OS_2(tt_file *ttf)
     for (i = 0; i < PDC_NUMCHARCOLL; i++)
     {
         j = i + 17;
-        if (tp->ulCodePageRange1 & (1<<j))
+        if (tp->ulCodePageRange1 & (1<<j) || (ttf->hasbig5cmap &&
+            tt_cpflag2charcoll[i] == cc_traditional_chinese))
             tp->charcolls[i] = tt_cpflag2charcoll[i];
         else
             tp->charcolls[i] = cc_none;
@@ -1126,6 +1148,15 @@ tt_get_tab_OS_2(tt_file *ttf)
 	tp->usMaxContext = 0;
     }
 
+    if (logg5)
+    {
+        pdc_logg(ttf->pdc, "\t\t\tusFirstCharIndex=0x%04X\n",
+                 ttf->tab_OS_2->usFirstCharIndex);
+        if (ttf->tab_cmap && ttf->tab_cmap->win)
+            pdc_logg(ttf->pdc, "\t\t\tstartCount[0]=0x%04X\n",
+                     ttf->tab_cmap->win->startCount[0]);
+    }
+
     /* there are fonts with inconsistent usFirstCharIndex */
     if (ttf->tab_cmap && ttf->tab_cmap->win &&
         tp->usFirstCharIndex != ttf->tab_cmap->win->startCount[0])
@@ -1217,7 +1248,7 @@ tt_get_tab_post(tt_file *ttf)
     tp->maxMemType42 = tt_get_ulong(ttf);
     tp->minMemType1 = tt_get_ulong(ttf);
     tp->maxMemType1 = tt_get_ulong(ttf);
-
+    tp->numberOfGlyphs = ttf->numGlyphs;
 
     /* there are subset fonts with different number of glyphs
      * see bug #1418
@@ -1419,22 +1450,43 @@ tt_unicode2gidx(tt_file *ttf, int usv, pdc_bool logg)
     pdc_core *pdc = ttf->pdc;
     tt_cmap4 *cm4 = ttf->tab_cmap->win;
     pdc_ushort uv;
-    int segs;
-    int gidx = 0;
-    int i;
+    int lo, hi, segs;
+    int gidx = 0, i = 0;
 
     uv = (pdc_ushort) usv;
-    if (logg) pdc_logg(pdc, "\t\t\tU+%04X: ", uv);
+    if (logg) pdc_logg(pdc, "\t\t\tUCS2: %04X: ", uv);
     segs = cm4->segCountX2 / 2;
 
-    for (i = 0; i < segs; ++i)
-        if (uv <= cm4->endCount[i])
-            break;
+    lo = 0;
+    hi = segs;
+    while (lo < hi)
+    {
+        i = (lo + hi) / 2;
 
-    if (logg) pdc_logg(pdc, "i=%d start=U+%04X  ", i, cm4->startCount[i]);
+        if (uv <= cm4->endCount[i])
+        {
+            if (uv >= cm4->startCount[i])
+            {
+                break;
+            }
+            else if (!i || uv > cm4->endCount[i - 1])
+            {
+                i = -1;
+                break;
+            }
+        }
+
+        if (uv < cm4->startCount[i])
+            hi = i;
+        else
+            lo = i + 1;
+    }
+
+
+    if (logg) pdc_logg(pdc, "i=%d start=UCS2: %04X  ", i, cm4->startCount[i]);
 
     TT_IOCHECK(ttf, i != segs);
-    if (uv < cm4->startCount[i] || uv == 0xFFFF)
+    if (i == -1 || uv == 0xFFFF)
     {
         if (logg) pdc_logg(pdc, "==> gidx=0\n");
         return 0;
@@ -1510,12 +1562,16 @@ tt_gidx2width(tt_file *ttf, int gidx)
 void
 fnt_set_tt_fontvalues(tt_file *ttf)
 {
+    pdc_bool logg3 = pdc_logg_is_enabled(ttf->pdc, 3, trc_font);
     fnt_font *font = ttf->font;
     fnt_font_metric *ftm = &font->m;
 
     if (ttf->onlyCFF)
         return;
 
+    if (logg3)
+        pdc_logg(ttf->pdc, "\tUnits per EM: %d\n", ttf->tab_head->unitsPerEm);
+
     if (ttf->tab_head)
     {
         ftm->llx = FNT_TT2PDF(ttf->tab_head->xMin);
@@ -1633,6 +1689,9 @@ fnt_set_tt_fontarrays(tt_file *ttf, int flags)
             ** like that; perhaps we have to apply similar tricks then...
             */
             uvoffset = (ttf->tab_OS_2->usFirstCharIndex & 0xFF00);
+
+            if (logg5)
+                pdc_logg(pdc, "\t\tuvoffset=0x%04X\n", uvoffset);
         }
         else
         {
@@ -2173,6 +2232,7 @@ fnt_check_tt_font(pdc_core *pdc, const char *filename, const char *fontname,
  * ttf->kerning = pdc_true;
  * ttf->vertical = pdc_true;
  * ttf->ignorename = pdc_true;
+ * ttf->cmst = ~NULL;
  * ttf->fortet = pdc_true;
  *
  * ttf->check = pdc_true;
@@ -2234,6 +2294,9 @@ fnt_new_tt(pdc_core *pdc, fnt_font *font)
     ttf->issymbol = pdc_false;
     ttf->haswinuni = pdc_false;
     ttf->hasonlymac = pdc_false;
+    ttf->hasbig5cmap = pdc_false;
+    ttf->forcesubset = pdc_false;
+    ttf->gidunequcid = pdc_false;
 
 
     ttf->utf16fontname = (char *) 0;
