@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_core.c,v 1.124.2.23 2008/02/21 16:57:48 kurt Exp $
+/* $Id: pc_core.c,v 1.124.2.30 2009/10/01 17:14:18 kurt Exp $
  *
  * PDFlib core services
  *
@@ -375,7 +375,7 @@ pdc_new_core(
     pdc->uniqueno = 0;
 
 
-#ifdef  PDC_DEBUG
+#ifdef PDC_DEBUG
     pdc->pr->hexdump = pdc_true;
 #endif
 
@@ -441,6 +441,7 @@ pdc_new_core(
     */
     pdc->pr->in_error = pdc_false;
 
+
     return pdc;
 }
 
@@ -862,6 +863,7 @@ pdc_push_errmsg(
     const pdc_error_info *ei = get_error_info(pdc, errnum);
 
     pdc_pop_errmsg(pdc);
+    pdc->pr->errnum = 0;
 
     make_errmsg(pdc, ei, parm1, parm2, parm3, parm4, pdc_false);
 
@@ -878,13 +880,17 @@ pdc_set_errmsg(
     const char *parm3,
     const char *parm4)
 {
-    const pdc_error_info *ei = get_error_info(pdc, errnum);
+    if (errnum != 0)
+    {
+        const pdc_error_info *ei = get_error_info(pdc, errnum);
 
-    make_errmsg(pdc, ei, parm1, parm2, parm3, parm4, pdc_false);
+        make_errmsg(pdc, ei, parm1, parm2, parm3, parm4, pdc_false);
+    }
 
     pdc->pr->errnum = errnum;
 
-    pdc_logg_cond(pdc, 2, trc_warning,
+    if (errnum)
+        pdc_logg_cond(pdc, 2, trc_warning,
                 "[Reason for error message %d: \"%s\"]\n",
                 pdc->pr->errnum, pdc->pr->errbuf);
 
@@ -928,7 +934,7 @@ pdc_error(
     const char *parm3,
     const char *parm4)
 {
-    const char *logmsg;
+    const char *logmsg = NULL;
 
     /* avoid recursive errors, but allow rethrow.
     */
@@ -948,18 +954,22 @@ pdc_error(
 
     if (pdc->pr->x_sp > pdc->pr->x_sp0)
     {
-	logmsg = "\n[/// Exception %d in %s ]";
+        if (pdc_logg_is_enabled(pdc, 2, trc_warning))
+	    logmsg = "[Nested exception %d in %s]";
     }
     else
     {
-	logmsg = "\n[+++ Exception %d in %s ]";
+	logmsg = "\n[Last exception %d in %s]";
     }
 
-    pdc_logg(pdc, logmsg, pdc->pr->errnum,
-	(pdc->pr->errnum == 0 || !pdc->pr->apiname) ? "" : pdc->pr->apiname,
-	pdc->pr->x_sp0 + 1, pdc->pr->x_sp - pdc->pr->x_sp0);
+    if (logmsg != NULL)
+    {
+        pdc_logg(pdc, logmsg, pdc->pr->errnum,
+	    (pdc->pr->errnum == 0 || !pdc->pr->apiname) ? "" : pdc->pr->apiname,
+	    pdc->pr->x_sp0 + 1, pdc->pr->x_sp - pdc->pr->x_sp0);
 
-    pdc_logg(pdc, "[\"%s\"]\n\n", pdc->pr->errbuf);
+        pdc_logg(pdc, "[\"%s\"]\n\n", pdc->pr->errbuf);
+    }
 
     if (pdc->pr->x_sp == -1)
     {
@@ -994,6 +1004,9 @@ pdc_jbuf(pdc_core *pdc)
 {
     static const char fn[] = "pdc_jbuf";
 
+    pdc_logg_cond(pdc, 3, trc_api,
+                  "[TRY to level %d]\n", pdc->pr->x_sp + 1);
+
     if (++pdc->pr->x_sp == pdc->pr->x_ssize)
     {
 	pdc_xframe *aux;
@@ -1053,6 +1066,9 @@ pdc_jbuf(pdc_core *pdc)
 void
 pdc_exit_try(pdc_core *pdc)
 {
+    pdc_logg_cond(pdc, 3, trc_api,
+                  "[EXIT_TRY at level %d]\n", pdc->pr->x_sp);
+
     if (pdc->pr->x_sp == -1)
     {
         strcpy(pdc->pr->errbuf, "exception stack underflow");
@@ -1069,6 +1085,9 @@ pdc_catch_intern(pdc_core *pdc)
 {
     pdc_bool result;
 
+    pdc_logg_cond(pdc, 3, trc_api,
+                  "[CATCH intern at level %d]\n", pdc->pr->x_sp);
+
     if (pdc->pr->x_sp == -1)
     {
         strcpy(pdc->pr->errbuf, "exception stack underflow");
@@ -1091,6 +1110,9 @@ pdc_catch_extern(pdc_core *pdc)
 {
     pdc_bool result;
 
+    pdc_logg_cond(pdc, 3, trc_api,
+                  "[CATCH at level %d]\n", pdc->pr->x_sp);
+
     if (pdc->pr->x_sp == -1)
     {
         strcpy(pdc->pr->errbuf, "exception stack underflow");
@@ -1149,14 +1171,6 @@ pdc_get_errpref(pdc_core *pdc)
     return pdc->pr->premsg;
 }
 
-/* ----------- service function to get PDF version string  -------------- */
-
-const char *
-pdc_get_pdfversion(pdc_core *pdc, int compatibility)
-{
-    return pdc_errprintf(pdc, "%d.%d", compatibility / 10, compatibility % 10);
-}
-
 
 #ifdef	PDC_DEBUG
 
