@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_file.c,v 1.123.2.40 2008/01/31 17:27:33 kurt Exp $
+/* $Id: pc_file.c,v 1.123.2.50 2009/10/09 09:35:27 kurt Exp $
  *
  * Various file routines
  *
@@ -30,7 +30,9 @@
 #include <windows.h>
 #include <process.h>
 #include <io.h>
+#define PDC_FILENO _fileno
 #else
+#define PDC_FILENO fileno
 #if defined(MAC)
 #include <MacErrors.h>
 #else
@@ -51,6 +53,37 @@
 #include <ssdef.h>
 #endif
 
+#if defined(WIN32) && !defined(__BORLANDC__)
+
+#define PDC_MAXFILEHANDLES 2048
+
+/* set a new maximum for the number of simultaneously open files.
+ * if return = -1 error occured
+ */
+int
+pdc_set_maxfilehandles(pdc_core *pdc, int maxfps)
+{
+    const char *stemp = pdc_errprintf(pdc, "%d", maxfps);
+
+    if (maxfps < _IOB_ENTRIES)
+        pdc_error(pdc, PDC_E_IO_TOOFEW_REQFILEHDLS, stemp,
+                  pdc_errprintf(pdc, "%d", _IOB_ENTRIES), 0, 0);
+
+    if (maxfps > PDC_MAXFILEHANDLES)
+        pdc_error(pdc, PDC_E_IO_TOOMANY_REQFILEHDLS, stemp,
+                  pdc_errprintf(pdc, "%d", PDC_MAXFILEHANDLES), 0, 0);
+
+    return _setmaxstdio(maxfps);
+}
+
+int
+pdc_get_maxfilehandles(void)
+{
+    return _getmaxstdio();
+}
+
+#endif
+
 /* platform independent wrapper functions for 64-bit file handling.
 */
 int
@@ -76,10 +109,10 @@ pdc__fseek(FILE *fp, pdc_off_t offset, int whence)
 	    {
 		pdc_off_t pos, len;
 
-		pos = _telli64(fileno(fp));
-		_lseeki64(fileno(fp), 0, SEEK_END);
-		len = _telli64(fileno(fp));
-		_lseeki64(fileno(fp), pos, SEEK_SET);
+		pos = _telli64(PDC_FILENO(fp));
+		_lseeki64(PDC_FILENO(fp), 0, SEEK_END);
+		len = _telli64(PDC_FILENO(fp));
+		_lseeki64(PDC_FILENO(fp), pos, SEEK_SET);
 
 		len += offset;
 		return fsetpos(fp, &len);
@@ -467,6 +500,31 @@ pdc_set_fopen_errmsg(pdc_core *pdc, int errnum, const char *qualifier,
     pdc_set_errmsg(pdc, errnum, qualifier, filename, stemp1, stemp2);
 }
 
+void
+pdc_set_fwrite_errmsg(pdc_core *pdc, const char *filename)
+{
+    const char *stemp1 = NULL;
+    const char *stemp2 = NULL;
+    int errno1 = errno;
+    int errnum = PDC_E_IO_WRITE;
+
+#if defined(EMACOSERR)
+#if defined(MAC) && defined(PDF_ALLOW_MAC_DEPR_FUNCS)
+    errno1 = (int) __MacOSErrNo;
+#endif
+#endif
+
+    stemp1 = pdc_errprintf(pdc, "%d", errno1);
+
+#ifdef PDC_HAS_STRERROR
+    stemp2 = strerror(errno1);
+    if (stemp2 != NULL)
+        errnum = PDC_E_IO_WRITE_CODETEXT;
+#endif
+
+    pdc_set_errmsg(pdc, errnum, filename, stemp1, stemp2, 0);
+}
+
 pdc_bool
 pdc_check_fopen_errmsg(pdc_core *pdc, pdc_bool requested)
 {
@@ -493,7 +551,7 @@ pdc_logg_openclose(pdc_core *pdc, FILE *fp, pdc_bool opened)
             pdc_logg(pdc, " closed");
 #if PDC_FILENO_EXISTS
         if (fp != NULL && opened)
-            pdc_logg(pdc, ", fileno=%d", fileno(fp));
+            pdc_logg(pdc, ", fileno=%d", PDC_FILENO(fp));
 #endif
         pdc_logg(pdc, ", errno=%d", errno1);
         if (errno2 != 0)
@@ -550,6 +608,7 @@ filelen = len;
  * will be deleted and a new char pointer to the Latin-1 string will
  * be returned. Otherwise an execption will be thrown.
  *
+ * Returned string is temporary allocated.
  */
 char *
 pdc_check_filename(pdc_core *pdc, char *filename)
@@ -557,7 +616,7 @@ pdc_check_filename(pdc_core *pdc, char *filename)
 #if !defined(PDC_UNICODE_FILENAME)
     char *ffname = pdc_utf8_to_hostbytes(pdc, pdc->honorlang, filename);
 
-    pdc_free(pdc, filename);
+    pdc_free_tmp(pdc, filename);
     if (ffname == NULL)
         pdc_error(pdc, PDC_E_IO_UNSUPP_UNINAME, 0, 0, 0, 0);
     filename = (char *) ffname;
@@ -566,6 +625,9 @@ pdc_check_filename(pdc_core *pdc, char *filename)
     return filename;
 }
 
+/*
+ * Returned string is temporary allocated.
+ */
 char *
 pdc_get_filename(pdc_core *pdc, char *filename)
 {
@@ -574,7 +636,7 @@ pdc_get_filename(pdc_core *pdc, char *filename)
 #if defined(PDC_UNICODE_FILENAME)
     static const char fn[] = "pdc_get_filename";
 
-    ffname = pdc_strdup_ext(pdc, filename, 0, fn);
+    ffname = pdc_strdup_ext(pdc, filename, PDC_CONV_TMPALLOC, fn);
 #else
     ffname = pdc_hostbytes_to_utf8(pdc, pdc->honorlang, filename);
 #endif
@@ -584,11 +646,13 @@ pdc_get_filename(pdc_core *pdc, char *filename)
 
 /*
  * pdc_convert_filename_ext converts a file name as string of name type
- * (see function pdc_convert_name) to a [EBCDIC-]UTF-8 string with or
+ * (see function pdc_convert_name_ext) to a [EBCDIC-]UTF-8 string with or
  * without a BOM. If the compiler doesn't allow Unicode filenames
  * (see define PDC_UNICODE_FILENAME) the filename is Latin-1 encoded
  * if possible or an exception will be thrown.
  *
+ * Returned string is temporary allocated.
+ *
  */
 const char *
 pdc_convert_filename_ext(pdc_core *pdc, const char *filename, int len,
@@ -602,6 +666,9 @@ pdc_convert_filename_ext(pdc_core *pdc, const char *filename, int len,
     if (filename == NULL)
         pdc_error(pdc, PDC_E_ILLARG_EMPTY, paramname, 0, 0, 0);
 
+    /* temporary allocation will be enforced */
+    flags |= PDC_CONV_TMPALLOC;
+
     fname = pdc_convert_name_ext(pdc, filename, len, enc, codepage, flags);
 
     if (fname == NULL || *fname == '\0')
@@ -616,12 +683,13 @@ pdc_convert_filename_ext(pdc_core *pdc, const char *filename, int len,
 #endif
     }
 
-    outfilename = pdc_errprintf(pdc, "%s", &fname[i]);
-    pdc_free(pdc, fname);
-
+    outfilename = &fname[i];
     return outfilename;
 }
 
+/*
+ * Returned string is temporary allocated.
+ */
 const char *
 pdc_convert_filename(pdc_core *pdc, const char *filename, int len,
                      const char *paramname, pdc_bool withbom)
@@ -644,6 +712,7 @@ FILE *
 pdc_fopen_logg(pdc_core *pdc, const char *filename, const char *mode)
 {
     FILE *fp = NULL;
+    int i = 0;
 
 
 #if defined(PDC_UNICODE_FILENAME)
@@ -651,10 +720,13 @@ pdc_fopen_logg(pdc_core *pdc, const char *filename, const char *mode)
     pdc_byte *outfilename = NULL;
     pdc_text_format nameformat = PDC_UTF8;
     pdc_text_format targetnameformat = pdc_utf16;
-    int len = (int) strlen(filename);
+    int len = (int) pdc_strlen(filename);
     int outlen = 0;
 
-    /* convert filename from UTF-8 to UTF-16 or Latin-1 */
+    if (pdc_is_utf16be_unicode(filename))
+        nameformat = pdc_utf16be;
+
+    /* convert filename from UTF-8 / UTF-16BE to UTF-16 or Latin-1 */
     pdc_convert_string(pdc, nameformat, 0, NULL, (pdc_byte *) filename, len,
                        &targetnameformat, NULL, &outfilename, &outlen,
                        PDC_CONV_TRYBYTES | PDC_CONV_NOBOM, pdc_true);
@@ -666,7 +738,6 @@ pdc_fopen_logg(pdc_core *pdc, const char *filename, const char *mode)
     else
     {
         wchar_t wmode[8];
-        int i;
 
         len = (int) strlen(mode);
         for (i = 0; i < len; i++)
@@ -681,7 +752,11 @@ pdc_fopen_logg(pdc_core *pdc, const char *filename, const char *mode)
 #else
     (void) pdc;
 
-    fp = fopen(filename, mode);
+    /* due to honorlang, codeset of LANG: UTF-8 */
+    if (pdc_is_utf8_bytecode(filename))
+        i = 3;
+
+    fp = fopen(&filename[i], mode);
 #endif
 
     pdc_logg_openclose(pdc, fp, pdc_true);
@@ -698,6 +773,9 @@ pdc_fopen(pdc_core *pdc, const char *filename, const char *qualifier,
     static const char fn[] = "pdc_fopen";
     pdc_file *sfp;
 
+    /* reset error number */
+    pdc_set_errmsg(pdc, 0, 0, 0, 0, 0);
+
     sfp = (pdc_file *) pdc_calloc(pdc, sizeof(pdc_file), fn);
 
     /* initialize */
@@ -991,7 +1069,17 @@ pdc_fwrite(const void *ptr, size_t size, size_t nmemb, pdc_file *sfp)
         size_t poslen, nbytes = 0;
 
         if (sfp->fp)
-            return pdc__fwrite(ptr, size, nmemb, sfp->fp);
+        {
+            size_t total = pdc__fwrite(ptr, size, nmemb, sfp->fp);
+
+            if (total < size * nmemb)
+            {
+                pdc_set_fwrite_errmsg(sfp->pdc, sfp->filename);
+                PDC_RETHROW(sfp->pdc);
+            }
+
+            return total;
+        }
 
         nbytes = size * nmemb;
         if (sfp->pos + nbytes > sfp->limit)
@@ -1112,49 +1200,63 @@ pdc_file_fullname(pdc_core *pdc, const char *dirname, const char *basename,
                   char *fullname)
 {
     const char *pathsep = PDC_PATHSEP;
+    const char *stemp = NULL;
+    pdc_bool dirhasbom = (dirname != NULL && pdc_is_utf8_bytecode(dirname));
+    pdc_bool basehasbom = (basename != NULL && pdc_is_utf8_bytecode(basename));
+    const char *sdirname = dirname;
+    const char *sbasename = basename;
     size_t len = 0;
 
+    fullname[0] = 0;
+    if (dirhasbom || basehasbom)
+    {
+        len = 3;
+        strcat(fullname, PDC_UTF8_STRING);
+        if (dirhasbom)
+            sdirname = &dirname[3];
+        if (basehasbom)
+            sbasename = &basename[3];
+    }
+
 #ifdef MVS
     pdc_bool lastterm = pdc_false;
 #endif
 
-    if (!dirname || !dirname[0])
+    if (sdirname == NULL || !sdirname[0])
     {
-        len += strlen(basename);
+        len += strlen(sbasename);
         if (len >= PDC_FILENAMELEN)
             goto PDC_FILE_ERROR;
 
-        strcpy(fullname, basename);
+        strcat(fullname, sbasename);
     }
     else
     {
-        fullname[0] = 0;
-
 #ifdef MVS
-        if (strncmp(dirname, PDC_FILEQUOT, 1))
+        if (strncmp(sdirname, PDC_FILEQUOT, 1))
             strcat(fullname, PDC_FILEQUOT);
 #endif
 
-        len += strlen(dirname);
+        len += strlen(sdirname);
         if (len >= PDC_FILENAMELEN)
             goto PDC_FILE_ERROR;
 
-        strcat(fullname, dirname);
+        strcat(fullname, sdirname);
 
 #ifdef VMS
         /* look for logical name or whose result */
-        if(getenv(dirname))
+        if(getenv(sdirname))
             pathsep = PDC_PATHSEP_LOG;
         else if (fullname[strlen(fullname)-1] == ']')
             pathsep = "";
 #endif
 
-        len += strlen(pathsep) + strlen(basename);
+        len += strlen(pathsep) + strlen(sbasename);
         if (len >= PDC_FILENAMELEN)
             goto PDC_FILE_ERROR;
 
         strcat(fullname, pathsep);
-        strcat(fullname, basename);
+        strcat(fullname, sbasename);
 
 #ifdef MVS
         lastterm = pdc_true;
@@ -1192,7 +1294,12 @@ pdc_file_fullname(pdc_core *pdc, const char *dirname, const char *basename,
     return;
 
     PDC_FILE_ERROR:
-    pdc_error(pdc, PDC_E_IO_TOOLONG_FULLNAME, fullname, 0, 0, 0);
+
+    if (sdirname == NULL || !sdirname[0])
+        stemp = pdc_errprintf(pdc, "%s", basename);
+    else
+        stemp = pdc_errprintf(pdc, "%s%s%s", dirname, pathsep, basename);
+    pdc_error(pdc, PDC_E_IO_TOOLONG_FULLNAME, stemp, 0, 0, 0);
 }
 
 #define EXTRA_SPACE     32     /* extra space for separators, FILEQUOT etc. */
@@ -1540,7 +1647,7 @@ pdc_fwrite_ascii(pdc_core *pdc, const char *str, size_t len, FILE *fp)
 {
 
     (void) pdc;
-    len = fwrite(str, 1, len, fp);
+    len = pdc__fwrite(str, 1, len, fp);
 
 
     return len;
@@ -1565,11 +1672,12 @@ pdc_write_file(
     if (sfp != NULL)
     {
         wlen = pdc_fwrite_ascii(pdc, content, len, sfp->fp);
-        pdc_fclose(sfp);
-        if (wlen != len)
+        if (wlen < len)
         {
-            pdc_set_errmsg(pdc, PDC_E_IO_WRITE, filename, 0, 0, 0);
+            pdc_set_fwrite_errmsg(pdc, filename);
+            PDC_RETHROW(pdc);
         }
+        pdc_fclose(sfp);
     }
 
 
@@ -1578,37 +1686,4 @@ pdc_write_file(
 
 
 
-#if defined(MAC) || defined(MACOSX)
-
-#ifdef PDF_TARGET_API_MAC_CARBON
-
-/* Construct an FSSpec from a Posix path name. Only required for
- * Carbon (host font support and file type/creator).
- */
-
-OSStatus
-FSPathMakeFSSpec(const UInt8 *path, FSSpec *spec)
-{
-    OSStatus        result;
-    FSRef           ref;
-
-    /* convert the POSIX path to an FSRef */
-    result = FSPathMakeRef(path, &ref, NULL);
-
-    if (result != noErr)
-            return result;
-
-    /* and then convert the FSRef to an FSSpec */
-    result = FSGetCatalogInfo(&ref, kFSCatInfoNone, NULL, NULL, spec, NULL);
-
-    return result;
-}
-
-
-#else
-
-
-#endif /* !PDF_TARGET_API_MAC_CARBON */
-
-#endif /* (defined(MAC) || defined(MACOSX)) */
 
