@@ -10,13 +10,14 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_optparse.c,v 1.118.2.23 2007/12/07 16:25:12 kurt Exp $
+/* $Id: pc_optparse.c,v 1.118.2.25 2009/03/16 14:01:12 kurt Exp $
  *
  * Parser options routines
  *
  */
 
 #include "pc_util.h"
+#include "pc_file.h"
 #include "pc_geom.h"
 #include "pc_ctype.h"
 
@@ -454,7 +455,7 @@ pdc_parse_optionlist(pdc_core *pdc, const char *optlist,
                             boolval = pdc_false;
                             break;
                         }
-                        else
+                        else if (isequal)
                         {
                             boolval = pdc_true;
                             break;
@@ -1448,6 +1449,26 @@ pdc_get_opt_utf8strings(pdc_core *pdc, const char *keyword, pdc_resopt *resopt,
     return ns;
 }
 
+const char *
+pdc_get_opt_filename(pdc_core *pdc, const char *keyword, pdc_resopt *resopts)
+{
+    const char *filename = NULL;
+    char **strlist;
+
+    if (pdc_get_optvalues(keyword, resopts, NULL, &strlist))
+    {
+        pdc_bool isutf8 = pdc_is_lastopt_utf8(resopts);
+        int flags = PDC_CONV_WITHBOM;
+
+        if (isutf8)
+            flags |= PDC_CONV_ISUTF8;
+
+        filename = pdc_convert_filename(pdc, strlist[0], 0, keyword, flags);
+    }
+
+    return filename;
+}
+
 void
 pdc_cleanup_optionlist(pdc_core *pdc, pdc_resopt *resopt)
 {
