@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_output.c,v 1.144.2.18 2008/02/12 13:53:31 kurt Exp $
+/* $Id: pc_output.c,v 1.144.2.29 2009/10/23 11:28:02 stm Exp $
  *
  * PDFlib output routines
  *
@@ -33,7 +33,7 @@
 #include <winbase.h>
 #endif
 
-#if defined(MAC) || defined (MACOSX)
+#if !defined(PDF_MAC_NOCORESERVICES) && (defined(MAC) || defined (MACOSX))
 
 /*
  * Setting the file type requires either Carbon or InterfaceLib/Classic.
@@ -47,7 +47,7 @@
 #include <Files.h>
 #endif
 
-#endif /* MAC || MACOSX */
+#endif /* !PDF_MAC_NOCORESERVICES && (MAC || MACOSX) */
 
 #ifdef HAVE_LIBZ
 #include "zlib.h"
@@ -101,8 +101,19 @@ struct pdc_output_s {
     MD5_CTX	md5;			/* MD5 digest context for file ID */
     unsigned char id[2][MD5_DIGEST_LENGTH];
     void	*opaque;		/* this will be used to store PDF *p */
+
+
+    /* ------------- extended output control ------------- */
 };
 
+/* ----------- service function to get PDF version string  -------------- */
+
+const char *
+pdc_get_pdfversion(pdc_core *pdc, int compatibility)
+{
+    return pdc_errprintf(pdc, "%d.%d", compatibility / 10, compatibility % 10);
+}
+
 /* --------------------- PDFlib stream handling ----------------------- */
 
 void *
@@ -228,11 +239,11 @@ pdc_init_stream(
         {
 	    out->fp = stdout;
 #if !defined(__MWERKS__) && (defined(WIN32) || defined(OS2))
-#if defined WINCE
-	    _setmode(fileno(stdout), _O_BINARY);
+#if !defined(__BORLANDC__) && !defined(OS2)
+	    _setmode(_fileno(stdout), _O_BINARY);
 #else
 	    setmode(fileno(stdout), O_BINARY);
-#endif /* !WINCE */
+#endif /* !__BORLANDC__ && !OS2 */
 #endif
 	}
         else
@@ -563,6 +574,8 @@ pdc_init_outctl(pdc_outctl *oc)
     oc->fopenparams     = 0;
     oc->recordsize      = 0;
 #endif
+
+
 } /* pdc_init_outctl */
 
 /*
@@ -586,6 +599,7 @@ pdc_init_output(
     pdc_cleanup_output(out, pdc_false);
 
     out->opaque		= opaque;
+
     out->lastobj	= 0;
 #if defined(MVS) || defined(MVS_TEST)
     out->fopenparams    = oc->fopenparams;
@@ -611,6 +625,8 @@ pdc_init_output(
 
     if (!pdc_init_stream(pdc, out, oc->filename, oc->fp, oc->writeproc))
 	return pdc_false;
+
+
     {
 	/* Write the document header */
 	pdc_printf(out, "%%PDF-%s\n", pdc_get_pdfversion(pdc, compatibility));
@@ -712,6 +728,7 @@ pdc_begin_obj(pdc_output *out, pdc_id obj_id)
     out->file_offset[obj_id] = pdc_tell_out(out);
     pdc_printf(out, "%ld 0 obj\n", obj_id);
 
+
     return obj_id;
 }
 
@@ -719,6 +736,7 @@ pdc_id
 pdc_alloc_id(pdc_output *out)
 {
 
+
     out->lastobj++;
 
     if (out->lastobj > PDF_MAXINDOBJS)
@@ -788,38 +806,73 @@ void
 pdc_put_pdffilename(pdc_output *out, const char *text, int len)
 {
     static const char *fn = "pdc_put_pdffilename";
+    pdc_byte *btext = (pdc_byte *) text;
+    pdc_bool isuni = pdc_is_utf16be_unicode(btext);
     char *ttext;
-    int i, j = 0;
+    pdc_byte c, cp, cpp;
+    int i, ia = 0, j = 0;
 
-    ttext = (char *) pdc_malloc(out->pdc, (size_t) (len + 4), fn);
+    ttext = (char *) pdc_calloc(out->pdc, (size_t) (len + 4), fn);
+
+    if (isuni)
+    {
+        ttext[0] = PDF_BOM0;
+        ttext[1] = PDF_BOM1;
+        ia = 2;
+        j = 2;
+    }
 
     /* absolute path name:
      * r:\pdfdocs\spec.pdf -> /r/pdfdocs/spec.pdf
      * pclib/eng:\pdfdocs\spec.pdf -> /pclib/eng/pdfdocs/spec.pdf
      */
-    if (strchr(text, PDF_COLON))
+    cp = 0x7F;
+    for (i = ia; i < len; i++)
     {
-        ttext[j] = PDF_SLASH;
-        j++;
-    }
-
-    for (i = 0; i < len; i++)
-    {
-        if (text[i] == PDF_BACKSLASH || text[i] == PDF_SLASH ||
-            text[i] == PDF_COLON)
+        c = btext[i];
+        if (c == PDF_COLON && (!isuni || cp == 0))
         {
-            /* convert to slash, but avoid multiple slashes */
-            if (!j || ttext[j - 1] != PDF_SLASH)
+            if (isuni)
             {
-                ttext[j] = PDF_SLASH;
+                ttext[j] = 0;
                 j++;
             }
-        }
-        else
-        {
-            ttext[j] = text[i];
+            ttext[j] = PDF_SLASH;
             j++;
+
+            break;
         }
+        cp = c;
+    }
+
+    cp = 0x7F;
+    cpp = 0x7F;
+    for (i = ia; i < len; i++)
+    {
+        c = btext[i];
+
+        if ((c == PDF_BACKSLASH || c == PDF_SLASH || c == PDF_COLON) &&
+            (!isuni || cp == 0))
+        {
+            /* convert to slash, but avoid multiple slashes */
+            if (cpp != PDF_SLASH)
+            {
+                c = PDF_SLASH;
+            }
+            else
+            {
+                if (isuni)
+                    j--;
+                continue;
+            }
+        }
+
+        ttext[j] = c;
+        j++;
+
+        cp = c;
+        if (c)
+            cpp = c;
     }
 
     len = j;
@@ -934,6 +987,8 @@ pdc_write_xref(pdc_output *out)
     pdc_id	free_id;
     pdc_core *	pdc = out->pdc;
 
+
+
     /* Don't write any object after this check! */
 
     for (i = start; i <= out->lastobj; i++) {
@@ -1068,8 +1123,6 @@ pdc_write_trailer(
 
 /* ---------------------- High-level output functions ---------------------- */
 
-#define PDC_LINEBUFLEN  4048             /* len of line output buffer */
-
 /*
  * Write a native encoded string to the output.
  */
@@ -1106,7 +1159,7 @@ pdc_putc(pdc_output *out, const char c)
 void
 pdc_printf(pdc_output *out, const char *fmt, ...)
 {
-    char buf[PDC_LINEBUFLEN];
+    char buf[PDC_GEN_BUFSIZE];
     va_list ap;
 
     va_start(ap, fmt);
