@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2006 Thomas Merz and PDFlib GmbH. All rights reserved. |
+ | Copyright (c) 1997-2008 Thomas Merz and PDFlib GmbH. All rights reserved. |
  +---------------------------------------------------------------------------+
  |                                                                           |
  |    This software is subject to the PDFlib license. It is NOT in the       |
@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_resource.c,v 1.61.2.34 2008/02/22 15:30:00 kurt Exp $
+/* $Id: pc_resource.c,v 1.61.2.55 2010/04/22 14:37:52 kurt Exp $
  *
  * Resource routines
  *
@@ -28,6 +28,8 @@
 #if defined(WIN32)
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#else
+#include <unistd.h>
 #endif
 
 
@@ -219,14 +221,17 @@ pdc_get_resourcefile(pdc_core *pdc)
 static void
 pdc_read_resourcefile(pdc_core *pdc, const char *filename)
 {
+    pdc_bool logg1 = pdc_logg_is_enabled(pdc, 1, trc_resource);
     pdc_reslist *resl = pdc_get_reslist(pdc);
     pdc_file *fp = NULL;
     char **linelist;
     char *line;
     char *category = NULL;
-    char *uprfilename = NULL;
+    const char *uprfilename = NULL;
     char tmpname[PDC_FILENAMELEN];
-#if defined(AS400) || defined(WIN32)
+    char prodname[32];
+    char prodversion[32];
+    char *c;
 #define BUFSIZE 2048
     char buffer[BUFSIZE];
 #ifdef WIN32
@@ -234,11 +239,35 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
     HKEY hKey = NULL;
     DWORD size, lType;
 #endif
-#endif
-    int il, nlines = 0, nextcat, begin;
+    int il, ip, nlines = 0, nextcat, begin;
 
-    pdc_logg_cond(pdc, 1, trc_resource,
-                      "\n\tSearching for resource file...\n");
+    if (logg1)
+        pdc_logg(pdc, "\n\tSearching for resource file...\n");
+
+    /* product name */
+    strcpy(prodname, pdc->prodname);
+
+    /* product version: <major>.<minor> */
+    strcpy(prodversion, pdc->version);
+    if (strlen(pdc->version))
+    {
+        c = strchr(prodversion, '.');
+        if (c != NULL)
+        {
+            c++;
+            if (*c)
+            {
+                c++;
+                if (pdc_isdigit(*c))
+                    c++;
+                *c = 0;
+            }
+        }
+    }
+
+    if (logg1)
+        pdc_logg(pdc, "\tProduct name=%s, version=%s\n",
+                 prodname, prodversion);
 
 #ifdef WIN32
 
@@ -251,24 +280,28 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
 #endif  /* WIN32 */
 
 #ifdef AS400
-    pdc_logg_cond(pdc, 1, trc_resource,
-                      "\tSet AS400 default resources\n");
-    strcpy (buffer, "/pdflib/");
-    strcat (buffer, pdc->version);
-    il = (int) strlen(buffer);
-    strcat (buffer, "/fonts");
+    if (logg1)
+        pdc_logg(pdc, "\tSet AS400 default SearchPath entries\n");
+
+    sprintf(buffer, "/%s/%s/fonts", prodname, pdc->version);
     pdc_add_resource(pdc, "SearchPath", buffer, "");
-    strcpy(&buffer[il], "/bind/data");
+
+    sprintf(buffer, "/%s/%s/bind/data", prodname, pdc->version);
     pdc_add_resource(pdc, "SearchPath", buffer, "");
 #endif  /* AS400 */
 
+#ifdef MVS
+    (void) buffer;
+    (void) ip;
+#endif
+
 #ifdef WIN32
     /* process registry entries */
     if (RegOpenKeyExA(HKEY_LOCAL_MACHINE, regkey, 0L,
         (REGSAM) KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
     {
-        pdc_logg_cond(pdc, 1, trc_resource,
-                           "\tRead registry key \"%s\":\n", regkey);
+        if (logg1)
+            pdc_logg(pdc, "\tRead registry key \"%s\":\n", regkey);
 
         size = BUFSIZE - 2;
         if (RegQueryValueExA(hKey, "searchpath", (LPDWORD) NULL,
@@ -276,14 +309,17 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
             == ERROR_SUCCESS && *buffer)
         {
             char **pathlist;
-            int ip, np;
+            int np;
 
-            pdc_logg_cond(pdc, 1, trc_resource,
-                          "\tsearchpath entry: \"%s\"\n", buffer);
+            if (logg1)
+                pdc_logg(pdc, "\tsearchpath entry: \"%s\"\n", buffer);
 
             np = pdc_split_stringlist(pdc, buffer, ";", 0, &pathlist);
             for (ip = 0; ip < np; ip++)
+            {
+                /* TODO: we should only accessible directories */
                 pdc_add_resource(pdc, "SearchPath", pathlist[ip], "");
+            }
             pdc_cleanup_stringlist(pdc, pathlist);
         }
 
@@ -291,14 +327,40 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
     }
 #endif  /* WIN32 */
 
+#if !defined(WIN32) && !defined(AS400) && !defined(MVS)
+    if (logg1)
+        pdc_logg(pdc, "\tSet UNIX default SearchPath entries\n");
+
+    for (il = 0; rootdirectories[il] != NULL; il++)
+    {
+        const char *home = pdc_getenv_filename(pdc, "HOME");
+
+        if (home != NULL)
+             sprintf(tmpname, rootdirectories[il], home);
+        else
+             strcpy(tmpname, rootdirectories[il]);
+
+        /* we allow only accessible root directories */
+        if (access(tmpname, X_OK) != -1)
+        {
+            for (ip = 0; defsearchpathlist[ip] != NULL; ip++)
+            {
+                sprintf(buffer, defsearchpathlist[ip],
+                        tmpname, prodname, prodversion);
+                pdc_add_resource(pdc, "SearchPath", buffer, "");
+            }
+        }
+    }
+#endif /* !WIN32 && !AS400 && !MVS */
+
     /* searching for name of upr file */
-    uprfilename = (char *)filename;
+    uprfilename = filename;
     if (uprfilename == NULL || *uprfilename == '\0')
     {
         /* upr file name via environment variable */
         sprintf(tmpname, RESOURCEFILEENVNAME, pdc->prodname);
         pdc_strtoupper(tmpname);
-        uprfilename = pdc_getenv(tmpname);
+        uprfilename = pdc_getenv(pdc, tmpname);
 
 #ifdef WIN32
         /* registry upr file name */
@@ -312,8 +374,8 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
                                      &lType, (LPBYTE) buffer, &size)
                     == ERROR_SUCCESS && *buffer)
                 {
-                    pdc_logg_cond(pdc, 1, trc_resource,
-                                  "\tresourcefile entry: \"%s\"\n", buffer);
+                    if (logg1)
+                        pdc_logg(pdc, "\tresourcefile entry: \"%s\"\n", buffer);
 
                     uprfilename = buffer;
                 }
@@ -330,7 +392,7 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
             uprfilename = pdc_strtolower(tmpname);
 
             /* user-supplied upr file */
-            fp = pdc_fsearch_fopen(pdc, uprfilename, NULL, NULL, 0);
+            fp = pdc_fsearch_fopen(pdc, uprfilename, NULL, "UPR ", 0);
             if (fp == NULL)
             {
                 uprfilename = NULL;
@@ -342,8 +404,8 @@ pdc_read_resourcefile(pdc_core *pdc, const char *filename)
     {
         char *resfilename = resl->filename;
 
-        pdc_logg_cond(pdc, 1, trc_resource,
-                           "\tRead resource file \"%s\":\n", uprfilename);
+        if (logg1)
+            pdc_logg(pdc, "\tRead resource file \"%s\":\n", uprfilename);
 
         resl->filename = pdc_strdup(pdc, uprfilename);
         if (resfilename)
@@ -417,23 +479,26 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
                      const char *resvalue, pdc_encoding enc, int codepage)
 {
     static const char fn[] = "pdc_add_resource";
+    pdc_bool logg1 = pdc_logg_is_enabled(pdc, 1, trc_resource);
     pdc_reslist *resl = pdc_get_reslist(pdc);
     pdc_rescategory rescat;
     pdc_category *cat = NULL, *lastcat = NULL;
     pdc_res *res = NULL, *lastres = NULL;
     char *resnamutf8 = NULL;
     char *resvalutf8 = NULL;
-    int resnamflags = PDC_CONV_EBCDIC;
-    int resvalflags = PDC_CONV_EBCDIC;
+    int resnamflags = PDC_CONV_EBCDIC | PDC_CONV_TMPALLOC;
+    int resvalflags = PDC_CONV_EBCDIC | PDC_CONV_TMPALLOC;
     int k;
 
-    if (!resvalue || !strlen(resvalue))
-        pdc_logg_cond(pdc, 1, trc_resource,
-               "\tAdd \"%s\" to resource category \"%s\"\n", resname, category);
-    else
-        pdc_logg_cond(pdc, 1, trc_resource,
-               "\tAdd \"%s=%s\" to resource category \"%s\"\n",
-               resname, resvalue, category);
+    if (logg1)
+    {
+        if (!resvalue || !strlen(resvalue))
+            pdc_logg(pdc, "\tAdd \"%s\" to resource category \"%s\"\n",
+                     resname, category);
+        else
+            pdc_logg(pdc, "\tAdd \"%s=%s\" to resource category \"%s\"\n",
+                     resname, resvalue, category);
+    }
 
     /* We no longer raise an error but silently ignore unknown categories */
     k = pdc_get_keycode_ci(category, pdc_rescategories);
@@ -508,7 +573,7 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
         }
         else
         {
-            resvalutf8 = pdc_strdup(pdc, "");
+            resvalutf8 = pdc_strdup_ext(pdc, "", PDC_CONV_TMPALLOC, fn);
         }
         pdc_cleanup_stringlist(pdc, strlist);
     }
@@ -537,8 +602,6 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
         {
             if (!strlen(resnamutf8) || !strlen(resvalutf8))
             {
-                pdc_free(pdc, resnamutf8);
-                pdc_free(pdc, resvalutf8);
                 if (resvalue == NULL)
                     pdc_error(pdc, PDC_E_RES_BADRES, resname, category, 0, 0);
                 else
@@ -555,15 +618,12 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
         {
             if (strlen(resvalutf8))
             {
-                if (resnamutf8 != NULL)
-                    pdc_free(pdc, resnamutf8);
-                pdc_free(pdc, resvalutf8);
                 pdc_error(pdc, PDC_E_RES_BADRES, resname, category, 0, 0);
             }
 
             if (resvalutf8 != NULL)
             {
-                pdc_free(pdc, resvalutf8);
+                pdc_free_tmp(pdc, resvalutf8);
                 resvalutf8 = NULL;
             }
 
@@ -576,11 +636,12 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
             {
                 /* delete all entries */
                 if (resnamutf8 != NULL)
-                    pdc_free(pdc, resnamutf8);
+                    pdc_free_tmp(pdc, resnamutf8);
                 pdc_delete_rescategory(pdc, lastcat, cat, pdc_true);
 
-                pdc_logg_cond(pdc, 1, trc_resource,
-                        "\tResource category \"%s\" removed\n", category);
+                if (logg1)
+                    pdc_logg(pdc, "\tResource category \"%s\" removed\n",
+                             category);
 
                 return;
             }
@@ -609,26 +670,27 @@ pdc_add_resource_ext(pdc_core *pdc, const char *category, const char *resname,
         else
             cat->kids = res;
         res->prev = lastres;
-        res->name = resnamutf8;
+        res->name = pdc_strdup(pdc, resnamutf8);
     }
     else
     {
-        pdc_free(pdc, resnamutf8);
+        pdc_free_tmp(pdc, resnamutf8);
     }
 
     /* New value */
     if (res->value)
         pdc_free(pdc, res->value);
-    res->value = resvalutf8;
+    res->value = pdc_strdup(pdc, resvalutf8);
 
-    if (res->value && strlen(res->value))
-        pdc_logg_cond(pdc, 1, trc_resource,
-            "\tNew category.resource: \"%s.%s = %s\"\n",
-            category, res->name, res->value);
-    else
-        pdc_logg_cond(pdc, 1, trc_resource,
-            "\tNew category.resource: \"%s.%s\"\n",
-            category, res->name);
+    if (logg1)
+    {
+        if (res->value && strlen(res->value))
+            pdc_logg(pdc, "\tNew category.resource: \"%s.%s = %s\"\n",
+                     category, res->name, res->value);
+        else
+            pdc_logg(pdc, "\tNew category.resource: \"%s.%s\"\n",
+                     category, res->name);
+    }
 }
 
 void
@@ -743,7 +805,7 @@ pdc_find_resource_nr(pdc_core *pdc, const char *category, int nr)
                         pdc_errprintf(pdc, "%s%s%s", resname, separ, resval);
 
                     if (tobefree)
-                        pdc_free(pdc, resval);
+                        pdc_free_tmp(pdc, resval);
 
                     return retval;
                 }
@@ -804,13 +866,17 @@ pdc__create_pvf(pdc_core *pdc, const char *filename,
     static const char fn[] = "pdc__create_pvf";
     pdc_bool iscopy = pdc_false;
     pdc_virtfile *vfile, *lastvfile = NULL;
+    const char *stemp = NULL;
     pdc_resopt *results;
 
-    if (!data)
-        pdc_error(pdc, PDC_E_ILLARG_EMPTY, "data", 0, 0, 0);
+    if (data == NULL)
+        stemp = "data = NULL";
 
     if (!size)
-        pdc_error(pdc, PDC_E_ILLARG_EMPTY, "size", 0, 0, 0);
+        stemp = "size = 0";
+
+    if (stemp != NULL)
+        pdc_error(pdc, PDC_E_PAR_NODATA, stemp, 0, 0, 0);
 
     /* Parse optlist */
     results = pdc_parse_optionlist(pdc, optlist, pdc_create_pvf_options,
@@ -1112,8 +1178,8 @@ static pdc_loggdef *
 pdc_new_logg(pdc_core *pdc)
 {
     static const char fn[] = "pdc_new_logg";
+    const char *filename;
     char envname[32];
-    const char *envval = NULL;
 
     pdc_loggdef *logg = (pdc_loggdef *)
                               pdc_malloc(pdc, sizeof(pdc_loggdef), fn);
@@ -1137,11 +1203,11 @@ pdc_new_logg(pdc_core *pdc)
     /* logging file name defined by environment variable */
     sprintf(envname, LOGFILEENVNAME, pdc->prodname);
     pdc_strtoupper(envname);
-    envval = pdc_getenv(envname);
-    if (envval != NULL)
+    filename = pdc_getenv(pdc, envname);
+    if (filename != NULL)
     {
-        logg->filename = pdc_strdup(pdc, envval);
-        logg->fromenviron = pdc_true;
+        logg->filename = pdc_strdup(pdc, filename);
+        logg->fromenviron = pdc_undef; /* not yet initialized */
     }
 
     return logg;
@@ -1261,16 +1327,20 @@ static const pdc_defopt pdc_logg_options[] =
     PDC_OPT_TERMINATE
 };
 
+#define PDC_SEPARSTR_LEN 80
+
 static const char *separstr =
-    "[ --------------------------------------------------------- ]\n";
+    "[---------------------------------------"
+    "---------------------------------------]\n";
 
 void
 pdc_set_logg_options(pdc_core *pdc, const char *optlist)
 {
     pdc_loggdef *logg = pdc_get_logg(pdc);
     pdc_resopt *resopts = NULL;
-    char **strlist, *keyword;
+    char **strlist;
     char filename[PDC_FILENAMELEN+1];
+    const char *keyword;
     pdc_bool sare = pdc_false;
     pdc_bool enable = pdc_true;
     pdc_bool remfile = pdc_false;
@@ -1310,9 +1380,12 @@ pdc_set_logg_options(pdc_core *pdc, const char *optlist)
 
         pdc_get_optvalues("remove", resopts, &remfile, NULL);
 
-        if (!logg->fromenviron)
+        if (logg->fromenviron == pdc_false)
         {
-            pdc_get_optvalues("filename", resopts, filename, NULL);
+            const char *fname = pdc_get_opt_filename(pdc, "filename", resopts);
+
+            if (fname != NULL)
+                strcpy(filename, fname);
 
         }
 
@@ -1389,16 +1462,22 @@ pdc_set_logg_options(pdc_core *pdc, const char *optlist)
         else
         {
             strcpy(filename, logg->filename);
+            if (logg->fromenviron == pdc_undef)
+            {
+                logg->fromenviron = pdc_true;
+                pdc_free(pdc, logg->filename);
+                logg->filename = NULL;
+            }
         }
     }
 
     /* new logging file */
-    if (logg->filename == NULL || strcmp(logg->filename, filename))
+    if (pdc_strcmp(logg->filename, filename))
     {
         pdc_time ltime;
 
         /* close file */
-        if (logg->fp != stdout && logg->fp != stderr && logg->filename)
+        if (logg->fp != stdout && logg->fp != stderr && logg->filename != NULL)
         {
             pdc_localtime(&ltime);
             pdc_logg(pdc, "[%04d-%02d-%02d %02d:%02d:%02d]\n",
@@ -1421,12 +1500,15 @@ pdc_set_logg_options(pdc_core *pdc, const char *optlist)
         /* open file */
         if (!logg->flush)
         {
+            /* due to honorlang, codeset of LANG: UTF-8 */
+            i = pdc_is_utf8_bytecode(logg->filename) ? 3 : 0;
+
             if (!strcmp(logg->filename, "stdout"))
                 logg->fp = stdout;
             else if (!strcmp(logg->filename, "stderr"))
                 logg->fp = stderr;
             else
-                logg->fp = fopen(logg->filename, APPENDMODE);
+                logg->fp = fopen(&logg->filename[i], APPENDMODE);
             if (logg->fp == NULL)
             {
                 pdc_error(pdc, PDC_E_IO_WROPEN, "log ", logg->filename,
@@ -1443,7 +1525,9 @@ pdc_set_logg_options(pdc_core *pdc, const char *optlist)
         logg->enabled = enable;
         if (logg->enabled && logg->header && pdc->prodname != NULL)
         {
-            char binding[64];
+            char binding[64], buf[256], *bp;
+
+            pdc_logg(pdc, separstr);
 
             pdc_localtime(&ltime);
             binding[0] = 0;
@@ -1452,17 +1536,38 @@ pdc_set_logg_options(pdc_core *pdc, const char *optlist)
                 strcat(binding, pdc->binding);
                 strcat(binding, " binding ");
             }
-            pdc_logg(pdc, separstr);
-            pdc_logg(pdc, "[ %s %s  %son %s (%s) ",
-                      pdc->prodname, pdc->version, binding,
-                      PDF_PLATFORM, PDC_ISBIGENDIAN ? "be" : "le");
-            pdc_logg(pdc, "%04d-%02d-%02d %02d:%02d:%02d ]\n",
-                ltime.year + 1900, ltime.month + 1, ltime.mday,
-                ltime.hour, ltime.minute, ltime.second);
+
+            sprintf(buf, "[ %s %s  %son %s (%s) %04d-%02d-%02d %02d:%02d:%02d",
+                    pdc->prodname, pdc->version, binding,
+                    PDF_PLATFORM, PDC_ISBIGENDIAN ? "be" : "le",
+                    ltime.year + 1900, ltime.month + 1, ltime.mday,
+                    ltime.hour, ltime.minute, ltime.second);
+
+            i = MAX(PDC_SEPARSTR_LEN - (int) strlen(buf) - 1, 1);
+            pdc_logg(pdc, "%s%*s]\n", buf, i, " ");
+
+            bp = buf;
+            bp += sprintf(buf, "[ Classes:");
+            for (pclass = 0; pclass < trc_numclasses; pclass++)
+            {
+                level = logg->classlist[logg->sri][pclass];
+                if (level)
+                {
+                    keyword = pdc_get_keyword(pclass, pdf_protoclass_keylist);
+                    bp += sprintf(bp, " %s=%d", keyword, level);
+                }
+            }
+            i = MAX(PDC_SEPARSTR_LEN - (int) strlen(buf) - 1, 1);
+            pdc_logg(pdc, "%s%*s]\n", buf, i, " ");
 
             if (logg->classapi)
-                pdc_logg(pdc, "[ Use  %%s/\\[[^]]*\\]//g  and  %%s/)$/);"
-                               "/  in vi to compile it ]\n");
+            {
+                strcpy(buf, "[ Use  %s/\\[[^]]*\\]//g  and  %s/)$/);"
+                             "/  in vi to compile it");
+                i = MAX(PDC_SEPARSTR_LEN - (int) strlen(buf) - 1, 1);
+                pdc_logg(pdc, "%s%*s]\n", buf, i, " ");
+            }
+
             pdc_logg(pdc, separstr);
         }
     }
@@ -1502,12 +1607,15 @@ pdc_logg_output(pdc_core *pdc, const char *fmt, va_list ap)
     {
         FILE *fp = NULL;
 
+        /* due to honorlang, codeset of LANG: UTF-8 */
+        int i = pdc_is_utf8_bytecode(logg->filename) ? 3 : 0;
+
         if (!strcmp(logg->filename, "stdout"))
             fp = stdout;
         else if (!strcmp(logg->filename, "stderr"))
             fp = stderr;
         else
-            fp = fopen(logg->filename, APPENDMODE);
+            fp = fopen(&logg->filename[i], APPENDMODE);
 
         if (fp == NULL)
         {
@@ -1551,7 +1659,7 @@ pdc_enter_api_logg(pdc_core *pdc, const char *funame, pdc_bool enter_api,
             pdc->loggenv = pdc_true;
             sprintf(envname, LOGGINGENVNAME, pdc->prodname);
             pdc_strtoupper(envname);
-            envval = pdc_getenv(envname);
+            envval = pdc_getenv(pdc, envname);
             if (envval != NULL)
             {
                 pdc_set_logg_options(pdc, envval);
@@ -1645,8 +1753,10 @@ pdc_logg_exit_api(pdc_core *pdc, pdc_bool cleanup, const char *fmt, ...)
 void
 pdc_logg_enable(pdc_core *pdc, pdc_bool enable)
 {
-    if (pdc != NULL && pdc->logg != NULL)
+    if (pdc != NULL && pdc->logg != NULL && pdc->logg->filename != NULL)
+    {
         pdc->logg->enabled = enable;
+    }
 }
 
 /*
