@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pc_util.c,v 1.209.2.42 2008/02/26 11:43:03 york Exp $
+/* $Id: pc_util.c,v 1.209.2.69 2010/04/21 18:58:46 kurt Exp $
  *
  * PDFlib various utility routines
  *
@@ -34,11 +34,15 @@
 #define PDC_ISFINITE isfinite
 #else /* isfinite */
 
-#ifdef _WIN32
+#if defined(WIN32)
+#if !defined(_UNICODE)
+#define UNICODE
+#endif
 #include <windows.h>
 #include <float.h>
+#include <wchar.h>
 #define PDC_ISFINITE _finite
-#else /* _WIN32 */
+#else /* WIN32 */
 
 #ifdef OS_ZOS_SASC
 #define PDC_ISFINITE isfinite
@@ -50,7 +54,7 @@
 #endif
 
 
-/* ------------------- Floating-point number check ----------------------- */
+/* ---------------------- finite() workarounds -------------------------- */
 
 
 
@@ -146,22 +150,22 @@ void
 pdc_ascii_error(pdc_core *pdc, int errnum, int flags, const char *parm1,
                 const char *parm2, const char *parm3, const char *parm4)
 {
-    if (flags & 1<<1)
+    if (flags & (1<<0))
     {
         parm1 = pdc_errprintf(pdc, "%a", parm1);
     }
 
-    if (flags & 1<<2)
+    if (flags & (1<<1))
     {
         parm2 = pdc_errprintf(pdc, "%a", parm2);
     }
 
-    if (flags & 1<<3)
+    if (flags & (1<<2))
     {
         parm3 = pdc_errprintf(pdc, "%a", parm3);
     }
 
-    if (flags & 1<<4)
+    if (flags & (1<<3))
     {
         parm4 = pdc_errprintf(pdc, "%a", parm4);
     }
@@ -315,17 +319,73 @@ pdc_get_timestr(char *str, pdc_bool ktoascii)
 /* -------------------------- Environment ------------------------------ */
 
 char *
-pdc_getenv(const char *name)
+pdc_getenv(pdc_core *pdc, const char *envname)
 {
-#ifdef HAVE_ENVVARS
-    return getenv(name);
-#else
-    (void) name;
+    char *envvalue = NULL;
 
-    return (char *) 0;
-#endif
+    (void) pdc;
+    (void) envname;
+
+#ifdef HAVE_ENVVARS
+    envvalue = getenv(envname);
+    if (envvalue != NULL)
+    {
+        pdc_logg_cond(pdc, 1, trc_filesearch,
+            "\tEnvironment variable \"%s=%s\"\n", envname, envvalue);
+    }
+#endif /* HAVE_ENVVARS */
+
+    return envvalue;
 }
 
+char *
+pdc_getenv_filename(pdc_core *pdc, const char *envname)
+{
+    static const char fn[] = "pdc_getenv_filename";
+    char *envvalue = NULL;
+    int flags = PDC_CONV_TMPALLOC;
+
+#if defined(WIN32)
+
+    size_t len = strlen(envname), wlen;
+    const wchar_t *wenvvalue;
+    wchar_t *wenvname;
+
+    DWORD nSize = PDC_FILENAMELEN;
+
+    wlen = 2 * (len + 1);
+    wenvname = (wchar_t *) pdc_calloc(pdc, wlen, fn);
+    pdc_inflate_ascii(envname, (int) len, (char *) wenvname, pdc_utf16);
+
+    wenvvalue = _wgetenv(wenvname);
+    pdc_free(pdc, wenvname);
+
+    if (wenvvalue != NULL && pdc_wstrlen((char *) wenvvalue))
+    {
+        wlen = 2 * wcslen(wenvvalue);
+        pdc_logg_cond(pdc, 1, trc_filesearch,
+            "\tEnvironment variable \"%s=%T\"\n", envname, wenvvalue, wlen);
+
+        if (pdc_logg_is_enabled(pdc, 3, trc_filesearch))
+            flags |= PDC_CONV_LOGGING;
+
+        envvalue = pdc_convert_name(pdc, (char *) wenvvalue, wlen, flags);
+    }
+
+#else /* WIN32 */
+
+    envvalue = pdc_getenv(pdc, envname);
+    if (envvalue != NULL && strlen(envvalue))
+    {
+        envvalue = pdc_strdup_ext(pdc, envvalue, flags, fn);
+    }
+
+#endif /* !WIN32 */
+
+    return envvalue;
+}
+
+
 
 /* ------------------------ Language Code ------------------------------ */
 
@@ -393,12 +453,20 @@ pdc_check_lang_code(pdc_core *pdc, const char* lang_code)
 
 /* -------------------------- Bit arryas ------------------------------ */
 
+/* set bit right to left within a byte */
 void
 pdc_setbit(char *bitarr, int bit)
 {
     bitarr[bit/8] |= (char) (1<<(bit%8));
 }
 
+/* set bit left to right within a byte */
+void
+pdc_setbit_l2r(char *bitarr, int bit)
+{
+    bitarr[bit/8] |= (char) (0x80>>(bit%8));
+}
+
 pdc_bool
 pdc_getbit(const char *bitarr, int bit)
 {
@@ -529,11 +597,12 @@ pdc_strlen(const char *str)
  * must be terminated by a single null byte.
  * The caller is responsible for freeing the buffer.
  *
- * The special functions pdc_strdup and pdc_strdup_tmp
+ * The special functions pdc_strdup, pdc_strdup_tmp and pdc_strdup_withbom
  * should be replaced by the more sophisticated function pdc_strdup_ext.
  * There: flags (see pc_unicode.h):
  *
- * PDC_CONV_TMPALLOC, PDC_CONV_EBCDIC
+ * PDC_CONV_TMPALLOC, PDC_CONV_EBCDIC, PDC_CONV_ASCII,
+ * PDC_CONV_WITHBOM, PDC_CONV_NOBOM, PDC_CONV_MAXSTRLEN
  *
  */
 char *
@@ -544,19 +613,55 @@ pdc_strdup_ext(pdc_core *pdc, const char *text, int flags, const char *fn)
     if (text != NULL)
     {
         size_t len = pdc_strlen(text) + 1;
+        size_t is = 0, it = 0;
 
+        if ((flags & PDC_CONV_MAXSTRLEN) && len > PDC_ERR_MAXSTRLEN)
+            len = PDC_ERR_MAXSTRLEN;
+
+        if ((flags & PDC_CONV_NOBOM) && pdc_is_utf8_bytecode(text))
+            is = 3;
+
+        if ((flags & PDC_CONV_WITHBOM) && !pdc_is_utf8_bytecode(text))
+            it = 3;
+
+        len += it - is;
         if (flags & PDC_CONV_TMPALLOC)
             buf = (char *) pdc_malloc_tmp(pdc, len + 1, fn, NULL, NULL);
         else
             buf = (char *) pdc_malloc(pdc, len + 1, fn);
-        memcpy(buf, text, len);
+
+        memcpy(&buf[it], &text[is], len - it);
         buf[len] = 0;
 
+
+        if (it == 3)
+            pdc_copy_utf8_bom(buf);
+
     }
 
     return buf;
 }
 
+/* Convenience functions
+*/
+char *
+pdc_strdup_tmp(pdc_core *pdc, const char *text)
+{
+    static const char fn[] = "pdc_strdup_tmp";
+
+    return pdc_strdup_ext(pdc, text, PDC_CONV_TMPALLOC, fn);
+}
+
+char *
+pdc_strdup_withbom(pdc_core *pdc, const char *text)
+{
+    static const char fn[] = "pdc_strdup_withbom";
+
+    return pdc_strdup_ext(pdc, text, PDC_CONV_WITHBOM, fn);
+}
+
+/* Rapid function
+*/
 char *
 pdc_strdup(pdc_core *pdc, const char *text)
 {
@@ -591,53 +696,18 @@ pdc_strdup2(pdc_core *pdc, const char *text, size_t len)
     return buf;
 }
 
-char *
-pdc_strdup_tmp(pdc_core *pdc, const char *text)
-{
-    char *buf = NULL;
-    static const char fn[] = "pdc_strdup_tmp";
-
-    if (text != NULL)
-    {
-        size_t len = pdc_strlen(text) + 1;
-
-        buf = (char *) pdc_malloc_tmp(pdc, len + 1, fn, NULL, NULL);
-        memcpy(buf, text, len);
-        buf[len] = 0;
-    }
-
-    return buf;
-}
-
-/* Allocate a local buffer and copy a locale UTF-8 string
- * provided with an UTF-8 BOM.
- * The caller is responsible for freeing the buffer.
+/* Convert Pascal string to a null terminated C string.
+ * Size of C string: at least 256 bytes
  */
-char *
-pdc_strdup_withbom(pdc_core *pdc, const char *text)
+int
+pdc_convert_pascal_str(const char *pstr, char *cstr)
 {
-    char *buf = NULL;
-    static const char fn[] = "pdc_strdup_withbom";
+    int len = (int) *((pdc_byte *) pstr);
 
-    if (text != NULL)
-    {
-        size_t len;
+    memcpy(cstr, pstr + 1, (size_t) len);
+    cstr[len] = 0;
 
-        if (pdc_is_utf8_bytecode(text))
-        {
-            buf = pdc_strdup(pdc, text);
-        }
-        else
-        {
-            len = strlen(text);
-            buf = (char *) pdc_malloc(pdc, len + 4, fn);
-
-            pdc_copy_utf8_bom(buf);
-            strcpy(&buf[3], text);
-        }
-    }
-
-    return buf;
+    return len;
 }
 
 char *
@@ -820,11 +890,16 @@ pdc_strprint(pdc_core *pdc, const char *str, int leni, int maxchar,
     return (char *) pdc_calloc_tmp(pdc, 1, fn, NULL, NULL);
 }
 
+/*
+ * Returned string is temporary allocated.
+ */
 const char *
 pdc_utf8strprint(pdc_core *pdc, const char *str)
 {
-    int i = pdc_is_utf8_bytecode(str) ? 3 : 0;
-    return pdc_errprintf(pdc, "%.*s", PDC_ERR_MAXSTRLEN, &str[i]);
+    static const char fn[] = "pdc_utf8strprint";
+
+    return pdc_strdup_ext(pdc, str,
+                  PDC_CONV_TMPALLOC | PDC_CONV_NOBOM | PDC_CONV_MAXSTRLEN, fn);
 }
 
 /*
@@ -852,8 +927,7 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
     static const char fn[] = "pdc_split_stringlist";
     const char *separstr = " \f\n\r\t\v";
     const char *oldtext;
-    char **strlist = NULL, *newtext;
-    pdc_bool isoptlist = (flags & PDC_SPLIT_ISOPTLIST);
+    char **strlist = NULL, *newtext = NULL;
     int it, len, jt = 0, jtb = 0, maxk = 0, count = 0, inside = 0;
     int ns, nbs = 0, nbss;
 
@@ -869,7 +943,8 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
     ns = (int) strspn(text, separstr);
     oldtext = &text[ns];
     len = (int) strlen(oldtext);
-    if (!len) return 0;
+    if (!len)
+        return 0;
 
     /* check for UTF-8-BOM */
     if (pdc_is_utf8_bytecode(oldtext))
@@ -879,11 +954,13 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
         ns = (int) strspn(oldtext, separstr);
         oldtext = &oldtext[ns];
         len -= ns;
-        if (!len) return 0;
+        if (!len)
+            return 0;
     }
 
     /* new string */
-    newtext = (char *) pdc_malloc(pdc, (size_t) (len + 1), fn);
+    if (stringlist != NULL)
+        newtext = (char *) pdc_malloc(pdc, (size_t) (len + 1), fn);
     for (it = 0; it <= len; it++)
     {
         /* check for separators */
@@ -897,16 +974,17 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
         /* close text part */
         if (ns)
         {
-            newtext[jt] = 0;
-            if (count == maxk)
+            if (stringlist != NULL)
             {
-                maxk += 16;
-                strlist = (strlist == NULL) ?
-                    (char **) pdc_malloc(pdc, maxk * sizeof(char *), fn):
-                    (char **) pdc_realloc(pdc, strlist, maxk *
-                                          sizeof(char *), fn);
+                newtext[jt] = 0;
+                if (count == maxk)
+                {
+                    maxk += 16;
+                    strlist = (char **) pdc_realloc(pdc, strlist,
+                                                    maxk * sizeof(char *), fn);
+                }
+                strlist[count] = &newtext[jtb];
             }
-            strlist[count] = &newtext[jtb];
             count++;
 
             /* Exit */
@@ -919,7 +997,7 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
         }
 
         /* option list */
-        if (isoptlist)
+        if (flags & PDC_SPLIT_ISOPTLIST)
         {
             /* save backslash counter */
             nbss = nbs;
@@ -965,12 +1043,48 @@ pdc_split_stringlist(pdc_core *pdc, const char *text, const char *i_separstr,
             }
         }
 
+        /* argument list */
+        else if (flags & PDC_SPLIT_ISARGLIST)
+        {
+            /* save backslash counter */
+            nbss = nbs;
+
+            /* backslash */
+            if (oldtext[it] == '\\')
+            {
+                nbs++;
+                if (!(nbs % 2))
+                    continue;
+            }
+            else
+            {
+                nbs = 0;
+            }
+
+            /* open and close quotation mark */
+            if (oldtext[it] == '"')
+            {
+                if (!(nbss % 2))
+                {
+                    inside = 1 - inside;
+                    continue;
+                }
+                else
+                {
+                    jt--;
+                }
+            }
+        }
+
         /* save character */
-        newtext[jt] = oldtext[it];
-        jt++;
+        if (stringlist != NULL)
+        {
+            newtext[jt] = oldtext[it];
+            jt++;
+        }
     }
 
-    if (stringlist)
+    if (stringlist != NULL)
         *stringlist = strlist;
 
     return inside ? -count : count;
@@ -1275,35 +1389,51 @@ pdc_strtolower(char *str)
 }
 
 void
-pdc_swap_bytes(char *instring, int inlen, char *outstring)
+pdc_swap_bytes2(const char *instring, int inlen, char *outstring)
 {
-    char c;
-    int i,j;
+    pdc_ushort *inp, *outp;
+    int i;
 
     if (instring == NULL)
         return;
 
     if (outstring == NULL)
-        outstring = instring;
+        outstring = (char *) instring;
 
-    inlen = 2 * inlen / 2;
+    inp = (pdc_ushort *) instring;
+    outp = (pdc_ushort *) outstring;
+
+    inlen /= sizeof(pdc_ushort);
     for (i = 0; i < inlen; i++)
     {
-        j = i;
-        i++;
-        c = instring[j];
-        outstring[j] = instring[i];
-        outstring[i] = c;
+        outp[i] = (pdc_ushort) (((inp[i] & (pdc_ushort)0x00FFu) << 8) |
+                                ((inp[i] & (pdc_ushort)0xFF00u) >> 8));
     }
 }
 
 void
-pdc_swap_unicodes(char *instring)
+pdc_swap_bytes4(const char *instring, int inlen, char *outstring)
 {
-    if (instring &&
-        ((pdc_is_utf16be_unicode(instring) && !PDC_ISBIGENDIAN) ||
-         (pdc_is_utf16le_unicode(instring) &&  PDC_ISBIGENDIAN)))
-        pdc_swap_bytes(&instring[2], (int) (pdc_wstrlen(instring) - 2), NULL);
+    pdc_uint32 *inp, *outp;
+    int i;
+
+    if (instring == NULL)
+        return;
+
+    if (outstring == NULL)
+        outstring = (char *) instring;
+
+    inp = (pdc_uint32 *) instring;
+    outp = (pdc_uint32 *) outstring;
+
+    inlen /= sizeof(pdc_uint32);
+    for (i = 0; i < inlen; i++)
+    {
+        outp[i] = (pdc_uint32) (((inp[i] & (pdc_uint32)0x000000FFu) << 24) |
+                                ((inp[i] & (pdc_uint32)0x0000FF00u) <<  8) |
+                                ((inp[i] & (pdc_uint32)0x00FF0000u) >>  8) |
+                                ((inp[i] & (pdc_uint32)0xFF000000u) >> 24));
+    }
 }
 
 void
@@ -1409,6 +1539,8 @@ pdc_get_string_value(pdc_byte *str, int i, int charlen)
     return retval;
 }
 
+/* return value: length of new string. -1: error
+*/
 int
 pdc_subst_backslash(pdc_core *pdc, pdc_byte *str, int len,
                     pdc_encodingvector *ev, pdc_text_format textformat,
@@ -1542,7 +1674,7 @@ pdc_subst_backslash(pdc_core *pdc, pdc_byte *str, int len,
                 if (verbose)
                     pdc_error(pdc, -1, 0, 0, 0, 0);
 
-                return 0;
+                return -1;
             }
         }
 
@@ -1569,7 +1701,7 @@ pdc_subst_backslash(pdc_core *pdc, pdc_byte *str, int len,
     if (verbose)
         pdc_error(pdc, -1, 0, 0, 0, 0);
 
-    return 0;
+    return -1;
 }
 
 
@@ -1840,6 +1972,23 @@ pdc_str2integer(const char *string, int flags, void *o_iz)
     return pdc_true;
 }
 
+
+pdc_bool
+pdc_str2integer_ext(pdc_core *pdc, const char *string, int len,
+                    int dupflags, int flags, void *o_iz)
+{
+    static const char fn[] = "pdc_str2integer_ext";
+    char *dupstr;
+    pdc_bool retval;
+
+    dupstr = pdc_strdup_ext(pdc, string, dupflags, fn);
+    dupstr[len] = 0;
+    retval = pdc_str2integer(dupstr, flags, o_iz);
+    pdc_free(pdc, dupstr);
+    return retval;
+}
+
+
 static const char digits[] = "0123456789ABCDEF";
 
 static char *
@@ -1894,9 +2043,9 @@ pdc_ltoa(char *buf, long n, int width, char pad, int base)
 
 
 static char *
-pdc_ulong2a(
+pdc_uoff_t2a(
     char *	buf,
-    pdc_ulong	n,
+    pdc_uoff_t	n,
     int		width,
     char	pad,
     int		base,
@@ -1906,9 +2055,6 @@ pdc_ulong2a(
     int         k, i = sizeof aux;
     char *      dest = buf;
 
-    aux[--i] = digits[n % base];
-    n = n / base;
-
     while (0 < n)
     {
         aux[--i] = digits[n % base];
@@ -1933,7 +2079,7 @@ pdc_ulong2a(
     }
 
     return dest;
-} /* pdc_ulong2a */
+} /* pdc_uoff_t2a */
 
 
 static char *
@@ -1942,7 +2088,8 @@ pdc_off_t2a(
     pdc_off_t	n,
     int		width,
     char	pad,
-    pdc_bool	left_justify)
+    pdc_bool	left_justify,
+    pdc_bool	pos_sign)
 {
     char        aux[100];
     int         k, i = sizeof aux;
@@ -1958,6 +2105,9 @@ pdc_off_t2a(
     }
     else
     {
+	if (pos_sign)
+	    --width;
+
         sign = pdc_false;
         aux[--i] = digits[n % 10];
         n = n / 10;
@@ -1978,7 +2128,13 @@ pdc_off_t2a(
     }
 
     if (sign)
+    {
         *(dest++) = '-';
+    }
+    else if (pos_sign)
+    {
+        *(dest++) = '+';
+    }
 
     memcpy(dest, &aux[i], sizeof aux - i);
     dest += sizeof aux - i;
@@ -2082,6 +2238,17 @@ pdc_ftoa_pdfconf(pdc_core *pdc, char *buf, double x)
     ifd = pdc->floatdigits;
     powd = pow10[ifd];
 
+    /* number <= 1/powd will be mappepd to 1/powd */
+    if (x <= 1 / powd)
+    {
+        *(dest++) = '0';
+        *(dest++) = '.';
+        while (--ifd)
+            *(dest++) = '0';
+        *(dest++) = '1';
+        return dest;
+    }
+
     fract = modf(x, &integ);
     f = (long) (fract * powd + 0.5);
 
@@ -2126,12 +2293,23 @@ pdc_ftoa_pdfconf(pdc_core *pdc, char *buf, double x)
 } /* pdc_ftoa_pdfconf */
 
 
+/* flags for formatting function pdc_vxprintf()
+*/
+typedef enum
+{
+    pdc_form_nolimit,   /* no buffer limit supplied, no overflow check */
+    pdc_form_fixlimit,  /* fix buffer limit, buffer overflow causes exception */
+    pdc_form_varlimit   /* buffer overflow causes string truncation */
+}
+pdc_limitkind;
+
 /* write to string or file
 */
 static char *
 write_sf(
     pdc_core *pdc,
     FILE *fp,
+    pdc_limitkind ltd,
     char *dst,
     char *limit,
     const char *src,
@@ -2141,26 +2319,40 @@ write_sf(
     {
 	pdc_fwrite_ascii(pdc, src, (size_t) n, fp);
     }
-    else if (dst < limit)
+    else
     {
-	int avail = (int) (limit - dst);
+        if (ltd != pdc_form_nolimit)
+        {
+            int avail = (int) (limit - dst);
 
-	if (avail < n)
-	    n = avail;
+            if (avail < n)
+            {
+                if (ltd == pdc_form_fixlimit)
+                {
+                    pdc_error(pdc, PDC_E_INT_FORMOVERFLOW, 0, 0, 0, 0);
+                }
+                else
+                {
+                    n = MAX(avail, 0);
+                }
+            }
+        }
 
-	memcpy(dst, src, (size_t) n);
-	dst += n;
+        if (n > 0)
+        {
+            memcpy(dst, src, (size_t) n);
+            dst += n;
+        }
     }
 
     return dst;
 } /* write2buf */
 
-#define BIG_STR_SIZE	100000
-
 static int
 pdc_vxprintf(
     pdc_core *pdc,
     pdc_bool pdfconf,
+    pdc_limitkind ltd,
     char *cp,
     size_t size,
     FILE *fp,
@@ -2174,7 +2366,7 @@ pdc_vxprintf(
     int result = 0;
     char *limit = (char *) 0;
 
-    if (cp != (char *) 0)
+    if (cp != (char *) 0 && ltd != pdc_form_nolimit)
 	limit = cp + (int) (size - 1);
 
     for (/* */ ; /* */ ; /* */)
@@ -2183,6 +2375,7 @@ pdc_vxprintf(
         int             prec = -1;	/* = no precision specified	*/
         char            pad = ' ';
         pdc_bool        left_justify = pdc_false;
+        pdc_bool        pos_sign = pdc_false;
 
 	char		fbuf[100];	/* format buffer for %f and %g	*/
 	char *		fscan = fbuf;
@@ -2196,7 +2389,7 @@ pdc_vxprintf(
 	{
 	    int inbuf = (int) (dest - buf);
 
-	    cp = write_sf(pdc, fp, cp, limit, buf, inbuf);
+	    cp = write_sf(pdc, fp, ltd, cp, limit, buf, inbuf);
 	    result += inbuf;
 	    dest = buf;
 	}
@@ -2220,6 +2413,9 @@ pdc_vxprintf(
 		case '-':	left_justify = pdc_true;
 				break;
 
+		case '+':	pos_sign = pdc_true;
+				break;
+
 		case '0':	pad = '0';
 				break;
 
@@ -2297,8 +2493,8 @@ pdc_vxprintf(
         {
             case 'x':
             case 'X':
-                dest = pdc_ulong2a(
-                        dest, (pdc_ulong) va_arg(args, pdc_ulong),
+                dest = pdc_uoff_t2a(
+                        dest, (pdc_uoff_t) va_arg(args, pdc_uint),
                         width, pad, 16, left_justify);
                 break;
 
@@ -2308,12 +2504,12 @@ pdc_vxprintf(
 
             case 'd':
                 dest = pdc_off_t2a(dest, (pdc_off_t) va_arg(args, int),
-                                   width, pad, left_justify);
+                                   width, pad, left_justify, pos_sign);
                 break;
 
             case 'u':
-                dest = pdc_ulong2a(
-                        dest, (pdc_ulong) va_arg(args, pdc_ulong),
+                dest = pdc_uoff_t2a(
+                        dest, (pdc_uoff_t) va_arg(args, pdc_uint),
                         width, pad, 10, left_justify);
                 break;
 
@@ -2332,7 +2528,7 @@ pdc_vxprintf(
             case 'l':
             {
                 pdc_off_t	n = 0;
-		pdc_ulong	u = 0;
+                pdc_uoff_t	u = 0;
 		pdc_bool	ll = pdc_false;
 
                 if (*(++format) == 'l')
@@ -2343,7 +2539,10 @@ pdc_vxprintf(
 
 		if (strchr("xXu", *format))
 		{
-		    u = va_arg(args, pdc_ulong);
+		    if (ll)
+			u = va_arg(args, pdc_uoff_t);
+		    else
+			u = va_arg(args, pdc_ulong);
 		}
 		else if (*format == 'd')
 		{
@@ -2365,17 +2564,17 @@ pdc_vxprintf(
                 {
                     case 'x':
                     case 'X':
-                        dest = pdc_ulong2a(
+                        dest = pdc_uoff_t2a(
 				    dest, u, width, pad, 16, left_justify);
                         break;
 
                     case 'd':
-                        dest = pdc_off_t2a(
-				    dest, n, width, pad, left_justify);
+                        dest = pdc_off_t2a(dest, n, width, pad,
+					    left_justify, pos_sign);
                         break;
 
                     case 'u':
-                        dest = pdc_ulong2a(
+                        dest = pdc_uoff_t2a(
 				    dest, u, width, pad, 10, left_justify);
                         break;
 
@@ -2408,34 +2607,41 @@ pdc_vxprintf(
                 const char *cstr = str;
                 pdc_bool tobefree = pdc_false;
                 size_t len;
+                int llen;
 
-                if (str == 0)
-                    cstr = "(NULL)";
-                len = strlen(cstr);
-
-                if (*format == 'T')
+                if (str != 0)
                 {
-                    int l = va_arg(args, int);
-
-                    if (str != 0)
+                    if (*format == 'T')
                     {
-                        cstr = pdc_print_loggstring(pdc, str, l);
-                        len = strlen(cstr);
+                        llen = va_arg(args, int);
+                        cstr = pdc_print_loggstring(pdc, str, llen);
+                    }
+                    else if (*format == 'a')
+                    {
+                        cstr = pdc_strdup_ext(pdc, str, PDC_CONV_EBCDIC, fn);
+                        tobefree = pdc_true;
                     }
                 }
-
-                if (*format == 'a' && str != 0)
+                else
                 {
-                    cstr = pdc_strdup_ext(pdc, str, PDC_CONV_EBCDIC, fn);
-                    tobefree = pdc_true;
+                    cstr = "(NULL)";
+                    if (*format == 'T')
+                        llen = va_arg(args, int);
                 }
 
+		len = strlen(cstr);
+
+                if (prec != -1 && prec < (int) len)
+		{
+		    len = prec;
+		}
+
                 if (!left_justify && len < (size_t) width)
                 {
 		    int inbuf = (int) (width - len);
 
                     memset(buf, pad, (size_t) inbuf);
-		    cp = write_sf(pdc, fp, cp, limit, buf, inbuf);
+                    cp = write_sf(pdc, fp, ltd, cp, limit, buf, inbuf);
 		    result += inbuf;
                 }
 
@@ -2447,19 +2653,20 @@ pdc_vxprintf(
                     {
                         pdc_fwrite_ascii(pdc, cstr, len, fp);
                     }
-		    else if (cp < limit)
-		    {
-			if (result >= (int) size)
-			{
-			    memcpy(cp, cstr, (size_t) (limit - cp));
-			    cp = limit;
-			}
-			else
-			{
-			    memcpy(cp, cstr, len);
-			    cp += (int) len;
-			}
-		    }
+                    else if (ltd == pdc_form_nolimit || result < (int) size)
+                    {
+                        memcpy(cp, cstr, len);
+                        cp += (int) len;
+                    }
+                    else if (ltd == pdc_form_fixlimit)
+                    {
+                        pdc_error(pdc, PDC_E_INT_FORMOVERFLOW, 0, 0, 0, 0);
+                    }
+                    else if (cp < limit)
+                    {
+                        memcpy(cp, cstr, (size_t) (limit - cp));
+                        cp = limit;
+                    }
 
                     if (tobefree)
                         pdc_free(pdc, (char *) cstr);
@@ -2470,7 +2677,7 @@ pdc_vxprintf(
 		    int inbuf = (int) (width - len);
 
                     memset(buf, pad, (size_t) inbuf);
-		    cp = write_sf(pdc, fp, cp, limit, buf, inbuf);
+                    cp = write_sf(pdc, fp, ltd, cp, limit, buf, inbuf);
 		    result += inbuf;
                 }
 
@@ -2503,7 +2710,8 @@ int
 pdc_vfprintf(pdc_core *pdc, pdc_bool pdfconf, FILE *fp,
              const char *format, va_list args)
 {
-    return pdc_vxprintf(pdc, pdfconf, NULL, 0, fp, format, args);
+    return pdc_vxprintf(pdc, pdfconf, pdc_form_nolimit,
+                        NULL, 0, fp, format, args);
 } /* pdc_vfprintf */
 
 int
@@ -2514,7 +2722,8 @@ pdc_fprintf(pdc_core *pdc, pdc_bool pdfconf, FILE *fp,
     va_list ap;
 
     va_start(ap, format);
-    result = pdc_vxprintf(pdc, pdfconf, NULL, 0, fp, format, ap);
+    result = pdc_vxprintf(pdc, pdfconf, pdc_form_nolimit,
+                          NULL, 0, fp, format, ap);
     va_end(ap);
 
     return result;
@@ -2528,7 +2737,8 @@ int
 pdc_vsprintf(pdc_core *pdc, pdc_bool pdfconf, char *buf,
              const char *format, va_list args)
 {
-    return pdc_vxprintf(pdc, pdfconf, buf, BIG_STR_SIZE, NULL, format, args);
+    return pdc_vxprintf(pdc, pdfconf, pdc_form_fixlimit,
+                        buf, PDC_GEN_BUFSIZE, NULL, format, args);
 } /* pdc_vsprintf */
 
 int
@@ -2539,7 +2749,8 @@ pdc_sprintf(pdc_core *pdc, pdc_bool pdfconf, char *buf,
     va_list ap;
 
     va_start(ap, format);
-    result = pdc_vxprintf(pdc, pdfconf, buf, BIG_STR_SIZE, NULL, format, ap);
+    result = pdc_vxprintf(pdc, pdfconf, pdc_form_nolimit,
+                          buf, 0, NULL, format, ap);
     va_end(ap);
 
     return result;
@@ -2549,7 +2760,8 @@ int
 pdc_vsnprintf(pdc_core *pdc, char *buf, size_t size,
               const char *format, va_list args)
 {
-    return pdc_vxprintf(pdc, pdc_false, buf, size, NULL, format, args);
+    return pdc_vxprintf(pdc, pdc_false, pdc_form_varlimit,
+                        buf, size, NULL, format, args);
 } /* pdc_vsnprintf */
 
 
@@ -2608,7 +2820,9 @@ pdc_create_treebranch(pdc_core *pdc, pdc_branch *root, const char *pathname,
 
             name = namelist[i];
             if (name_p)
-                *name_p = pdc_errprintf(pdc, "%.*s", PDC_ERR_MAXSTRLEN, name);
+            {
+                *name_p = pdc_utf8strprint(pdc, name);
+            }
 
             nkids = parent->nkids;
             for (j = 0; j < nkids; j++)
@@ -2642,8 +2856,8 @@ pdc_create_treebranch(pdc_core *pdc, pdc_branch *root, const char *pathname,
             if (!strcmp(kid->name, name))
             {
                 if (errcode) *errcode = tree_nameexists;
-                if (name_p) *name_p =
-                    pdc_errprintf(pdc, "%.*s", PDC_ERR_MAXSTRLEN, name);
+                if (name_p) *name_p = pdc_utf8strprint(pdc, name);
+
                 pdc_free(pdc, name);
                 return NULL;
             }
@@ -2975,3 +3189,4 @@ pdc_srand(pdc_core *pdc, pdc_uint seed)
 {
     pdc->last_rand = seed;
 } /* pdc_srand */
+
