@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_actions.c,v 1.91.2.17 2007/12/21 11:32:22 kurt Exp $
+/* $Id: p_actions.c,v 1.91.2.21 2009/02/10 16:46:54 kurt Exp $
  *
  * PDFlib actions handling routines
  *
@@ -244,6 +244,7 @@ typedef struct pdf_action_s
     pdf_dest *dest;
     pdc_encoding hypertextencoding;
     char *filename;
+    char *nativefilename;
     char *parameters;
     char *operation;
     char *defaultdir;
@@ -274,6 +275,7 @@ pdf_reclaim_action(void *item)
     action->dest = NULL;
     action->hypertextencoding = pdc_invalidenc;
     action->filename = NULL;
+    action->nativefilename = NULL;
     action->parameters = NULL;
     action->operation = NULL;
     action->defaultdir = NULL;
@@ -308,6 +310,12 @@ pdf_release_action(void *context, void *item)
         action->filename = NULL;
     }
 
+    if (action->nativefilename)
+    {
+        pdc_free(p->pdc, action->nativefilename);
+        action->nativefilename = NULL;
+    }
+
     if (action->parameters)
     {
         pdc_free(p->pdc, action->parameters);
@@ -483,20 +491,26 @@ pdf__create_action(PDF *p, const char *type, const char *optlist)
     for (i = 0; ; i++)
     {
         keyword = pdf_filename_keylist[i].word;
-        if (keyword)
-        {
-            if (pdf_get_opt_textlist(p, keyword, resopts, htenc, htcp, pdc_true,
-                                     NULL, &action->filename, NULL))
-            {
-                if (!pdf_opt_effectless(p, keyword, atype,
-                                 (pdf_actiontype) pdf_filename_keylist[i].code))
-                    pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
-                else
-                    action->filename = NULL;
-            }
-        }
-        else
+        if (keyword == NULL)
             break;
+
+        if (!pdc_get_optvalues(keyword, resopts, NULL, NULL) ||
+            pdf_opt_effectless(p, keyword, atype,
+                               (pdf_actiontype) pdf_filename_keylist[i].code))
+            continue;
+
+        /* DON'T change order */
+
+        /* native filename */
+        if (!i)
+            action->nativefilename = pdf_get_opt_filename(p, keyword, resopts,
+                                                          htenc, htcp);
+
+        /* Unicode filename */
+        pdf_get_opt_textlist(p, keyword, resopts, htenc, htcp, pdc_true,
+                             NULL, &action->filename, NULL);
+
+        pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
     }
 
     keyword = "parameters";
@@ -675,12 +689,12 @@ pdf_write_action(PDF *p, pdf_action *action, pdc_id next_id)
             pdc_puts(p->out, "/Win");
             pdc_begin_dict(p->out);                     /* Win dict */
             pdc_puts(p->out, "/F");
-            pdf_put_pdffilename(p, action->filename);
+            pdf_put_pdffilename(p, action->nativefilename);
             pdc_puts(p->out, "\n");
             if (p->compatibility >= PDC_1_7)
             {
                 pdc_puts(p->out, "/UF");
-                pdf_put_hypertext(p, action->filename);
+                pdf_put_pdfunifilename(p, action->filename);
                 pdc_puts(p->out, "\n");
             }
             if (action->parameters)
@@ -714,12 +728,12 @@ pdf_write_action(PDF *p, pdf_action *action, pdc_id next_id)
             pdc_begin_dict(p->out);                     /* F dict */
             pdc_puts(p->out, "/Type/Filespec\n");
             pdc_puts(p->out, "/F");
-            pdf_put_pdffilename(p, action->filename);
+            pdf_put_pdffilename(p, action->nativefilename);
             pdc_puts(p->out, "\n");
             if (p->compatibility >= PDC_1_7)
             {
                 pdc_puts(p->out, "/UF");
-                pdf_put_hypertext(p, action->filename);
+                pdf_put_pdfunifilename(p, action->filename);
                 pdc_puts(p->out, "\n");
             }
             pdc_end_dict(p->out);                       /* F dict */
