@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_annots.c,v 1.198.2.22 2008/01/23 13:35:01 kurt Exp $
+/* $Id: p_annots.c,v 1.198.2.30 2010/04/16 12:32:27 kurt Exp $
  *
  * PDFlib routines for annnotations
  *
@@ -516,6 +516,7 @@ typedef struct pdf_annot_s
     char *subject;
     char *contents;
     char *filename;
+    char *nativefilename;
     char *mimetype;
     const char *iconname;
     pdc_off_t filesize;
@@ -580,6 +581,7 @@ pdf_reclaim_annot(void *item)
     ann->subject = NULL;
     ann->contents = NULL;
     ann->filename = NULL;
+    ann->nativefilename = NULL;
     ann->mimetype = NULL;
     ann->iconname = NULL;
     ann->filesize = 0;
@@ -650,6 +652,11 @@ pdf_release_annot(void *context, void *item)
             pdc_free(p->pdc, ann->filename);
             ann->filename = NULL;
         }
+        if (ann->nativefilename)
+        {
+            pdc_free(p->pdc, ann->nativefilename);
+            ann->nativefilename = NULL;
+        }
         if (ann->mimetype)
         {
             pdc_free(p->pdc, ann->mimetype);
@@ -688,10 +695,10 @@ static pdc_vtr_parms pdf_annot_parms =
 };
 
 static pdf_annot *
-pdf_new_annot(PDF *p)
+pdf_new_annot(PDF *p, pdf_annottype atype)
 {
     pdc_vtr *annots = pdf_get_annots_list(p);
-    pdf_annot *result;
+    pdf_annot *ann;
 
     if (annots == NULL)
     {
@@ -699,14 +706,16 @@ pdf_new_annot(PDF *p)
         pdf_set_annots_list(p, annots);
     }
 
-    result = pdc_vtr_incr(annots, pdf_annot);
-    result->usercoordinates = p->usercoordinates;
-    result->hypertextencoding = p->hypertextencoding;
-    result->hypertextcodepage = p->hypertextcodepage;
-    pdf_init_coloropt(p, &result->fillcolor);
+    ann = pdc_vtr_incr(annots, pdf_annot);
+    ann->atype = atype;
 
 
-    return result;
+    ann->usercoordinates = p->usercoordinates;
+    ann->hypertextencoding = p->hypertextencoding;
+    ann->hypertextcodepage = p->hypertextcodepage;
+    pdf_init_coloropt(p, &ann->fillcolor);
+
+    return ann;
 }
 
 static void
@@ -925,7 +934,7 @@ pdf__create_annotation(PDF *p,
     char **strlist = NULL;
     pdc_scalar *line;
     int i, j, k, ns, nss[2];
-    pdf_colorspacetype maxcs = color_rgb;
+    pdf_colortype maxcs = color_rgb;
     pdf_ppt *ppt = p->curr_ppt;
     pdc_matrix *ctm = &ppt->gstate[ppt->sl].ctm;
 
@@ -960,8 +969,7 @@ pdf__create_annotation(PDF *p,
                                    &cdata, pdc_true);
 
     /* Initializing */
-    ann = pdf_new_annot(p);
-    ann->atype = atype;
+    ann = pdf_new_annot(p, atype);
 
     keyword = "usercoordinates";
     pdc_get_optvalues(keyword, resopts, &ann->usercoordinates, NULL);
@@ -1149,21 +1157,24 @@ pdf__create_annotation(PDF *p,
     }
 
     keyword = "filename";
-    if (pdf_get_opt_textlist(p, keyword, resopts, ann->hypertextencoding,
-                             ann->hypertextcodepage, pdc_true,
-                             NULL, &ann->filename, NULL))
+    if (pdc_get_optvalues(keyword, resopts, NULL, NULL) &&
+        !pdf_opt_effectless(p, keyword, atype,
+                           (pdf_annottype) (ann_fileattachment | ann_movie)))
     {
-        if (!pdf_opt_effectless(p, keyword, atype,
-                            (pdf_annottype) (ann_fileattachment | ann_movie)))
-        {
-            pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
-            if (atype == ann_fileattachment)
-                ann->filesize = pdf_check_file(p, ann->filename, pdc_true);
-        }
-        else
-        {
-            ann->filename = NULL;
-        }
+        /* DON'T change order */
+
+        /* native filename */
+        ann->nativefilename = pdf_get_opt_filename(p, keyword, resopts,
+                          ann->hypertextencoding, ann->hypertextcodepage);
+
+        pdf_get_opt_textlist(p, keyword, resopts, ann->hypertextencoding,
+                             ann->hypertextcodepage,
+             /* bug #2344 */ atype == ann_fileattachment ? pdc_undef : pdc_true,
+                             NULL, &ann->filename, NULL);
+        pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
+
+        if (atype == ann_fileattachment)
+            ann->filesize = pdf_check_file(p, ann->filename, pdc_true);
     }
 
     keyword = "mimetype";
@@ -1281,7 +1292,7 @@ pdf__create_annotation(PDF *p,
                         0, 0, 0);
             }
         }
-        
+
         if (ann->dest)
         {
             pdf_cleanup_destination(p, ann->dest);
@@ -1423,8 +1434,7 @@ pdf__create_annotation(PDF *p,
                     if (ir > irect)
                     {
                         /* create copy */
-                        ann = pdf_new_annot(p);
-                        ann->atype = atype;
+                        ann = pdf_new_annot(p, atype);
                         memcpy(ann, ann_first, sizeof(pdf_annot));
                         ann->obj_id = PDC_BAD_ID;
                         ann->iscopy = pdc_true;
@@ -1868,7 +1878,7 @@ pdf_write_page_annots(PDF *p, pdc_vtr *annots)
             case ann_fileattachment:
             {
                 /* see bug #1439 */
-                const char *basename = pdc_file_strip_dirs(ann->filename);
+                const char *basename = pdc_file_strip_dirs(ann->nativefilename);
 
                 pdc_puts(p->out, "/FS");
                 pdc_begin_dict(p->out);                 /* FS dict */
@@ -1879,7 +1889,7 @@ pdf_write_page_annots(PDF *p, pdc_vtr *annots)
                 if (p->compatibility >= PDC_1_7)
                 {
                     pdc_puts(p->out, "/UF");
-                    pdf_put_hypertext(p, basename);
+                    pdf_put_pdfunifilename(p, basename);
                     pdc_puts(p->out, "\n");
                 }
 
@@ -1904,12 +1914,12 @@ pdf_write_page_annots(PDF *p, pdc_vtr *annots)
                 pdc_begin_dict(p->out);                 /* F dict */
                 pdc_puts(p->out, "/Type/Filespec\n");
                 pdc_puts(p->out, "/F");
-                pdf_put_pdffilename(p, ann->filename);
+                pdf_put_pdffilename(p, ann->nativefilename);
                 pdc_puts(p->out, "\n");
                 if (p->compatibility >= PDC_1_7)
                 {
                     pdc_puts(p->out, "/UF");
-                    pdf_put_hypertext(p, ann->filename);
+                    pdf_put_pdfunifilename(p, ann->filename);
                     pdc_puts(p->out, "\n");
                 }
                 pdc_end_dict(p->out);                   /* F dict */
@@ -2157,11 +2167,13 @@ pdf__attach_file(
     pdc_fclose(attfile);
 
     /* fill up annotation struct */
-    ann = pdf_new_annot(p);
-    ann->atype = ann_fileattachment;
+    ann = pdf_new_annot(p, ann_fileattachment);
+    ann->zoom = pdc_false;
+    ann->rotate = pdc_false;
     pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
     pdf_insert_annot_params(p, ann);
     ann->filename = pdc_strdup(p->pdc, filename);
+    ann->nativefilename = pdc_strdup(p->pdc, filename);
     ann->filesize = pdf_check_file(p, ann->filename, pdc_true);
     ann->contents = pdf_convert_hypertext_depr(p, description, len_descr);
     ann->title = pdf_convert_hypertext_depr(p, author, len_auth);
@@ -2170,8 +2182,6 @@ pdf__attach_file(
     if (icontype != icon_attach_pushpin)
         ann->iconname =
             pdc_get_keyword(icontype, pdf_attach_iconnames_pdfkeylist);
-    ann->zoom = pdc_false;
-    ann->rotate = pdc_false;
 }
 
 void
@@ -2200,8 +2210,7 @@ pdf__add_note(
     }
 
     /* fill up annotation struct */
-    ann = pdf_new_annot(p);
-    ann->atype = ann_text;
+    ann = pdf_new_annot(p, ann_text);
     pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
     pdf_insert_annot_params(p, ann);
     ann->contents = pdf_convert_hypertext_depr(p, contents, len_cont);
@@ -2249,8 +2258,7 @@ pdf__add_pdflink(
     /* fill up annotation struct */
     if (acthdl > -1)
     {
-        ann = pdf_new_annot(p);
-        ann->atype = ann_link;
+        ann = pdf_new_annot(p, ann_link);
         pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
         pdf_insert_annot_params(p, ann);
         if (p->pdc->hastobepos) acthdl++;
@@ -2318,8 +2326,7 @@ pdf__add_launchlink(
     /* fill up annotation struct */
     if (acthdl > -1)
     {
-        ann = pdf_new_annot(p);
-        ann->atype = ann_link;
+        ann = pdf_new_annot(p, ann_link);
         pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
         pdf_insert_annot_params(p, ann);
         if (p->pdc->hastobepos) acthdl++;
@@ -2344,8 +2351,7 @@ pdf__add_locallink(
     pdf_annot *ann;
 
     /* fill up annotation struct */
-    ann = pdf_new_annot(p);
-    ann->atype = ann_link;
+    ann = pdf_new_annot(p, ann_link);
     pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
     pdf_insert_annot_params(p, ann);
     ann->dest = pdf_parse_destination_optlist(p, optlist, page, pdf_locallink);
@@ -2377,8 +2383,7 @@ pdf__add_weblink(
     /* fill up annotation struct */
     if (acthdl > -1)
     {
-        ann = pdf_new_annot(p);
-        ann->atype = ann_link;
+        ann = pdf_new_annot(p, ann_link);
         pdf_init_rectangle(p, ann, llx, lly, urx, ury, NULL);
         pdf_insert_annot_params(p, ann);
         if (p->pdc->hastobepos) acthdl++;
