@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_color.c,v 1.217.2.9 2008/02/27 19:48:11 kurt Exp $
+/* $Id: p_color.c,v 1.217.2.12 2009/06/04 11:22:04 york Exp $
  *
  * PDFlib color routines
  *
@@ -162,10 +162,9 @@ pdf_color_components(PDF *p, int slot)
 static void
 pdf_write_color_values(PDF *p, pdf_color *color, pdf_drawmode drawmode)
 {
-    static const char *fn = "pdf_write_color_values";
+    static const char	fn[] = "pdf_write_color_values";
+
     pdf_colorspace *	cs = &p->colorspaces[color->cs];
-    int			cs_bias = p->curr_ppt->cs_bias;
-    int			pt_bias = p->curr_ppt->pt_bias;
 
     switch (cs->type)
     {
@@ -221,11 +220,11 @@ pdf_write_color_values(PDF *p, pdf_color *color, pdf_drawmode drawmode)
 		{
                     pdf_color *fillcolor = pdf_get_cstate(p, pdf_fill);
 
-		    pdc_printf(p->out, "/C%d cs ", cs_bias + color->cs);
+		    pdc_printf(p->out, "/C%d cs ", color->cs);
                     pdf_write_color_values(p, fillcolor, pdf_none);
 		}
 
-		pdc_printf(p->out, "/P%d scn\n", pt_bias + color->val.pattern);
+		pdc_printf(p->out, "/P%d scn\n", color->val.pattern);
 	    }
 	    else if (drawmode == pdf_stroke)
 	    {
@@ -237,11 +236,11 @@ pdf_write_color_values(PDF *p, pdf_color *color, pdf_drawmode drawmode)
 		{
                     pdf_color *strokecolor = pdf_get_cstate(p, pdf_stroke);
 
-		    pdc_printf(p->out, "/C%d CS ", cs_bias + color->cs);
+		    pdc_printf(p->out, "/C%d CS ", color->cs);
                     pdf_write_color_values(p, strokecolor, pdf_none);
 		}
 
-		pdc_printf(p->out, "/P%d SCN\n", pt_bias + color->val.pattern);
+		pdc_printf(p->out, "/P%d SCN\n", color->val.pattern);
 	    }
 
 	    p->pattern[color->val.pattern].used_on_current_page = pdc_true;
@@ -509,13 +508,12 @@ void
 pdf_write_page_colorspaces(PDF *p)
 {
     int i, total = 0;
-    int bias = p->curr_ppt->cs_bias;
 
     for (i = 0; i < p->colorspaces_number; i++)
 	if (p->colorspaces[i].used_on_current_page)
 	    total++;
 
-    if (total > 0 || bias
+    if (total > 0
        )
     {
 	pdc_puts(p->out, "/ColorSpace");
@@ -534,7 +532,7 @@ pdf_write_page_colorspaces(PDF *p)
 		    /* don't write simple color spaces as resource */
 		    if (!PDF_SIMPLE_COLORSPACE(cs))
 		    {
-			pdc_printf(p->out, "/C%d", bias + i);
+			pdc_printf(p->out, "/C%d", i);
 			pdc_objref(p->out, "", cs->obj_id);
 		    }
 		}
@@ -542,7 +540,6 @@ pdf_write_page_colorspaces(PDF *p)
 	}
 
 
-	if (!bias)
 	    pdc_end_dict(p->out);		/* color space names */
     }
 } /* pdf_write_page_colorspaces */
