@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_document.c,v 1.174.2.43 2008/02/27 19:48:11 kurt Exp $
+/* $Id: p_document.c,v 1.174.2.60 2009/10/21 14:28:09 stm Exp $
  *
  * PDFlib document related routines
  *
@@ -31,6 +31,8 @@
 
 
 
+#include "pc_strconst.h"
+
 
 #if (defined(WIN32) || defined(OS2)) && !defined(WINCE)
 #include <fcntl.h>
@@ -233,7 +235,7 @@ static const pdc_keyconn pdc_permissions_keylist[] =
     {"pdfx", pdc_keywordlist, PDF_ICC_FLAG, 1, 1, \
       0.0, 0.0, pdf_pdfx_keylist}, \
 \
-    {"compatibility", pdc_keywordlist, PDC_OPT_IGNOREIF1, 1, 1, \
+    {"compatibility", pdc_keywordlist, PDC_OPT_NONE, 1, 1, \
       0.0, 0.0, pdf_compatibility_keylist}, \
 \
     {"flush", pdc_keywordlist, PDC_OPT_NONE, 1, 1, \
@@ -351,6 +353,11 @@ struct pdf_document_s
 
 
 
+#if defined(MVS) || defined(MVS_TEST)
+    char *fopenparams;               /* additional fopen() parameter string */
+    char **tempfilenames;            /* 2 temporary file names */
+#endif
+
     pdc_bool moddate;                /* modified date will be created */
     char lang[PDF_MAX_LANGCODE + 1]; /* default natural language */
     char *action;                    /* document actions */
@@ -394,6 +401,11 @@ pdf_init_get_document(PDF *p)
 
 
 
+#if defined(MVS) || defined(MVS_TEST)
+        doc->fopenparams = NULL;
+        doc->tempfilenames = NULL;
+#endif
+
         doc->moddate = pdc_false;
         doc->lang[0] = 0;
         doc->action = NULL;
@@ -452,6 +464,20 @@ pdf_cleanup_document_internal(PDF *p)
 
 
 
+#if defined(MVS) || defined(MVS_TEST)
+        if (doc->fopenparams)
+        {
+            pdc_free(p->pdc, doc->fopenparams);
+            doc->fopenparams = NULL;
+        }
+
+        if (doc->tempfilenames)
+        {
+            pdc_cleanup_optstringlist(p->pdc, doc->tempfilenames, 2);
+            doc->tempfilenames = NULL;
+        }
+#endif
+
 
         if (doc->searchindexname)
         {
@@ -492,6 +518,12 @@ pdf_fix_openmode(PDF *p)
 
 
 
+pdc_bool
+pdf_get_plainmetadata(PDF *p)
+{
+    (void) p;
+    return pdc_false;
+}
 
 
 /* ------------------------- viewerpreferences ----------------------- */
@@ -766,7 +798,7 @@ pdf_write_search_indexes(PDF *p)
         if (p->compatibility >= PDC_1_7)
         {
             pdc_printf(p->out, "/UF");
-            pdf_put_hypertext(p, doc->searchindexname);
+            pdf_put_pdfunifilename(p, doc->searchindexname);
         }
         pdc_end_dict(p->out);                           /* Index */
         pdc_end_dict(p->out);                           /* Indexes */
@@ -863,7 +895,7 @@ pdf_parse_attachments_optlist(PDF *p, char **optlists, int ns,
                             pdf_attachments_options, &cdata, pdc_true);
 
         if (pdf_get_opt_textlist(p, "filename", resopts, htenc, htcp,
-                                 pdc_true, NULL, &fat->filename, NULL))
+                 /* bug #2344 */ pdc_undef, NULL, &fat->filename, NULL))
             pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
 
         if (pdf_get_opt_textlist(p, "description", resopts, htenc, htcp,
@@ -914,7 +946,7 @@ pdf_write_attachments(PDF *p)
             if (p->compatibility >= PDC_1_7)
             {
                 pdc_printf(p->out, "/UF");
-                pdf_put_hypertext(p, basename);
+                pdf_put_pdfunifilename(p, basename);
                 pdc_puts(p->out, "\n");
             }
 
@@ -1129,9 +1161,7 @@ pdf_get_document_common_options(PDF *p, pdc_resopt *resopts, int fcode)
 
     pdc_get_optvalues("moddate", resopts, &doc->moddate, NULL);
 
-
-
-    ns = pdc_get_optvalues("attachments", resopts, NULL, &strlist);
+    ns = pdc_get_opt_utf8strings(p->pdc, "attachments", resopts, 0, &strlist);
     if (ns)
         pdf_parse_attachments_optlist(p, strlist, ns, htenc, htcp);
 }
@@ -1210,6 +1240,14 @@ pdf_begin_document_internal(PDF *p, const char *optlist, pdc_bool callback)
 
 
 
+#if defined(MVS) || defined(MVS_TEST)
+        if (pdc_get_optvalues("filemode", resopts, NULL, NULL))
+        {
+            doc->fopenparams =
+                (char *) pdc_save_lastopt(resopts, PDC_OPT_SAVE1ELEM);
+        }
+#endif
+
 	n_groups = pdc_get_optvalues("groups", resopts, NULL, &groups);
     }
 
@@ -1233,23 +1271,6 @@ pdf_begin_document_internal(PDF *p, const char *optlist, pdc_bool callback)
     pdf_get_document_common_options(p, resopts, PDF_FC_BEGIN_DOCUMENT);
 
 
-    /* deprecated */
-    p->bookmark_dest = pdf_init_destination(p);
-
-    pdf_init_images(p);
-    pdf_init_xobjects(p);
-    pdf_init_fonts(p);
-    pdf_init_outlines(p);
-    pdf_init_annot_params(p);
-    pdf_init_colorspaces(p);
-    pdf_init_pattern(p);
-    pdf_init_shadings(p);
-    pdf_init_extgstates(p);
-
-
-
-
-
     /* create document digest */
     pdc_init_digest(p->out);
 
@@ -1296,22 +1317,54 @@ pdf_begin_document_internal(PDF *p, const char *optlist, pdc_bool callback)
 #endif
 
 
-    if (!pdc_init_output((void *) p, p->out, doc->compatibility, &oc))
+    PDC_TRY(p->pdc)
     {
-        if (oc.filename && *oc.filename)
+        if (!pdc_init_output((void *) p, p->out, doc->compatibility, &oc))
         {
-            pdc_set_fopen_errmsg(p->pdc,
-                pdc_get_fopen_errnum(p->pdc, PDC_E_IO_WROPEN), "PDF ",
-                pdc_errprintf(p->pdc, "%.*s", PDC_ERR_MAXSTRLEN, oc.filename));
+            if (oc.filename && *oc.filename)
+            {
+                pdc_set_fopen_errmsg(p->pdc,
+                    pdc_get_fopen_errnum(p->pdc, PDC_E_IO_WROPEN), "PDF ",
+                    pdc_errprintf(p->pdc, "%.*s", PDC_ERR_MAXSTRLEN,
+                                  oc.filename));
+                if (verbose)
+                {
+                    pdf_cleanup_document_internal(p);
+                    PDC_RETHROW(p->pdc);
+                }
+            }
 
-            if (verbose)
-                PDC_RETHROW(p->pdc);
+            pdf_cleanup_document_internal(p);
+            PDC_EXIT_TRY(p->pdc);
+            return -1;
         }
-
+    }
+    PDC_CATCH(p->pdc)
+    {
         pdf_cleanup_document_internal(p);
+        if (verbose)
+            PDC_RETHROW(p->pdc);
+
         return -1;
     }
 
+    /* deprecated */
+    p->bookmark_dest = pdf_init_destination(p);
+
+    pdf_init_images(p);
+    pdf_init_xobjects(p);
+    pdf_init_fonts(p);
+    pdf_init_outlines(p);
+    pdf_init_annot_params(p);
+    pdf_init_colorspaces(p);
+    pdf_init_pattern(p);
+    pdf_init_shadings(p);
+    pdf_init_extgstates(p);
+
+
+
+
+
     /* Write the constant /ProcSet array once at the beginning */
     p->procset_id = pdc_begin_obj(p->out, PDC_NEW_ID);
     pdc_puts(p->out, "[/PDF/ImageB/ImageC/ImageI/Text]\n");
@@ -1350,8 +1403,8 @@ pdf__begin_document(PDF *p, const char *filename, int len, const char *optlist)
          * The Intel version of the Metrowerks compiler doesn't have setmode().
          */
 #if !defined(__MWERKS__) && (defined(WIN32) || defined(OS2))
-#if defined WINCE
-        _setmode(fileno(fp), _O_BINARY);
+#if !defined(__BORLANDC__) && !defined(OS2)
+        _setmode(_fileno(fp), _O_BINARY);
 #else
         setmode(fileno(fp), O_BINARY);
 #endif
@@ -1653,7 +1706,8 @@ pdf_write_pages_and_catalog(PDF *p, pdc_id orig_root_id)
         for (i = 0; i < p->names_number; i++)
         {
             inlen = strlen(p->names[i].name);
-            name = pdf_convert_pdfstring(p, p->names[i].name, inlen, &outlen);
+            name = pdf_convert_pdfstring(p, p->names[i].name, inlen,
+                                PDC_CONV_WITHBOM | PDC_CONV_TRYBYTES, &outlen);
 
             if (name != p->names[i].name)
                 pdc_free(p->pdc, p->names[i].name);
@@ -2043,3 +2097,4 @@ pdf_set_viewerpreference(PDF *p, const char *viewerpreference)
 
 
 
+
