@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_font.c,v 1.461.2.53 2008/02/18 15:00:55 kurt Exp $
+/* $Id: p_font.c,v 1.461.2.80 2011/05/02 10:42:22 kurt Exp $
  *
  * PDFlib font handling routines
  *
@@ -437,6 +437,7 @@ static pdc_bool
 pdf_get_metrics_core(PDF *p, pdf_font *font, const char *fontname,
              const char *outfilename, pdc_encoding enc, pdc_bool checktimes)
 {
+    static const char fn[] = "pdf_get_metrics_core";
     const char *fname = NULL;
     const fnt_font_metric *ftm;
 
@@ -445,6 +446,12 @@ pdf_get_metrics_core(PDF *p, pdf_font *font, const char *fontname,
     {
         fontname = fname;
         font->opt.fontstyle = fnt_Normal;
+
+        if (font->apiname != NULL)
+        {
+            pdc_free(p->pdc, font->apiname);
+            font->apiname = pdc_strdup_ext(p->pdc, fontname, 0, fn);
+        }
     }
 
     ftm = fnt_get_core_metric(fontname);
@@ -823,6 +830,7 @@ pdf_init_font_options(PDF *p, pdf_font_options *fo)
     fo->xheight = 0;
     fo->linegap = 0;
     fo->auxiliary = pdc_false;
+    fo->dropcorewidths = pdc_false;
 
 }
 
@@ -918,6 +926,8 @@ pdf_get_font_options(PDF *p, pdf_font_options *fo, pdc_resopt *resopts)
     if (pdc_get_optvalues("linegap", resopts, &fo->linegap, NULL))
         fo->mask |= (1L << fo_linegap);
 
+
+        pdc_get_optvalues("dropcorewidths", resopts, &fo->dropcorewidths, NULL);
 }
 
 int
@@ -1004,6 +1014,7 @@ pdf_check_font_embedding(PDF *p, pdf_font *font, const char *fontname)
 int
 pdf_load_font_internal(PDF *p, pdf_font_options *fo)
 {
+    pdc_bool logg1 = pdc_logg_is_enabled(p->pdc, 1, trc_font);
     pdc_bool logg2 = pdc_logg_is_enabled(p->pdc, 2, trc_font);
     const char *fontname;
     const char *encoding;
@@ -1041,10 +1052,14 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
     else
     {
         fontname = pdc_utf8strprint(p->pdc, fontname_p);
-        pdc_free(p->pdc, fontname_p);
+        pdc_free_tmp(p->pdc, fontname_p);
     }
     fontname_p = NULL;
 
+    if (logg1)
+        pdc_logg(p->pdc, "\tCanonical font name: \"%s\"\n",
+                 fontname);
+
     /* font encoding */
     encoding = fo->encoding;
     encoding_aux = encoding;
@@ -1065,14 +1080,22 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
     /* UTF-8 font name with BOM */
     font->ft.utf8name = pdc_strdup(p->pdc, fo->fontname);
 
-    pdc_logg_cond(p->pdc, 1, trc_font, "\tFont UTF-8 name: \"%s\"\n",
-                  font->ft.utf8name);
+    if (logg1)
+        pdc_logg(p->pdc, "\tFont UTF-8 name: \"%s\"\n",
+                 font->ft.utf8name);
 
     /* specified encoding name */
     font->encapiname = pdc_strdup(p->pdc, encoding);
 
     /* search for a registered encoding */
     enc = pdc_find_encoding(p->pdc, encoding);
+
+    if (enc == pdc_unicode || enc == pdc_glyphid)
+    {
+        pdc_set_errmsg(p->pdc, PDF_E_UNSUPP_UNICODE, 0, 0, 0, 0);
+        goto PDF_PREMATURE_EXIT;
+    }
+
     if (enc == pdc_invalidenc || enc == pdc_unicode)
     {
         /* search for a predefined CMap and registered fonts */
@@ -1130,8 +1153,8 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
      * is found, make a copy in a new slot and attach the requested encoding.
      */
 
-    pdc_logg_cond(p->pdc, 1, trc_font,
-        "\tFont will be searched in the PDFlib font cache\n");
+    if (logg1)
+        pdc_logg(p->pdc, "\tFont will be searched in the PDFlib font cache\n");
     for (slot = 0; slot < p->fonts_number; slot++)
     {
         if (p->fonts[slot].obj_id == PDC_BAD_ID &&
@@ -1251,8 +1274,8 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
     }
 
     slot = -1;
-    pdc_logg_cond(p->pdc, 1, trc_font,
-        "\tFont not found in the PDFlib font cache\n");
+    if (logg1)
+        pdc_logg(p->pdc, "\tFont not found in the PDFlib font cache\n");
 
     /* embedding check */
     if (!pdf_check_font_embedding(p, font, fontname))
@@ -1290,8 +1313,8 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
     fontname_p = mastername;
 
     /* protocol */
-    pdc_logg_cond(p->pdc, 1, trc_font,
-        "\tPDFlib font name: \"%s\"\n", fontname_p);
+    if (logg1)
+        pdc_logg(p->pdc, "\tPDFlib font name: \"%s\"\n", fontname_p);
 
     /* Font file search hierarchy
      * - Check "FontOutline" resource entry and check TrueType font
@@ -1304,6 +1327,14 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
     retval = pdc_false;
     while (1)
     {
+        if (font->opt.auxiliary)
+        {
+            /* only in-core fonts are possible */
+            retval = pdf_get_metrics_core(p, font, fontname_p, "", enc,
+                                          pdc_false);
+            break;
+        }
+
 #ifdef PDF_TRUETYPE_SUPPORTED
         /* Check specified TrueType file */
         filename = pdc_find_resource(p->pdc, "FontOutline", fontname_p);
@@ -1379,8 +1410,8 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
         retval = pdc_false;
 
         /* Searching for a metric file */
-        pdc_logg_cond(p->pdc, 1, trc_font,
-            "\tSearching for font metrics data file:\n");
+        if (logg1)
+            pdc_logg(p->pdc, "\tSearching for font metrics data file:\n");
 
         filename = testfilename;
         for (i = 0; i < 100; i++)
@@ -1435,9 +1466,10 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
 	{
             retval = pdc_false;
 
-            pdc_logg_cond(p->pdc, 1, trc_font,
-                "\tMetric data file for font \"%s\" not available\n",
-                fontname_p);
+            if (logg1)
+                pdc_logg(p->pdc,
+                         "\tMetric data file for font \"%s\" not available\n",
+                         fontname_p);
             pdc_set_errmsg(p->pdc, PDF_E_FONT_NOMETRICS, 0, 0, 0, 0);
         }
 
@@ -1485,8 +1517,9 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
             else
             {
                 /* Searching font outline file */
-                pdc_logg_cond(p->pdc, 1, trc_font,
-                    "\tSearching for font outline data file:\n");
+                if (logg1)
+                    pdc_logg(p->pdc,
+                             "\tSearching for font outline data file:\n");
 
                 outfilename = testfilename;
                 for (i = 0; i < 100; i++)
@@ -1521,25 +1554,33 @@ pdf_load_font_internal(PDF *p, pdf_font_options *fo)
                 if (retval == pdc_undef)
 		{
                     retval = pdc_false;
-		    pdc_set_errmsg(p->pdc, PDF_E_FONT_NOOUTLINE, 0, 0, 0, 0);
+                    if (font->ft.m.type == fnt_Type1 ||
+                        font->ft.m.type == fnt_MMType1)
+                        pdc_set_errmsg(p->pdc, PDF_E_FONT_NOOUTLINE_PS,
+                                       0, 0, 0, 0);
+                    else
+                        pdc_set_errmsg(p->pdc, PDF_E_FONT_NOOUTLINE_TT,
+                                       0, 0, 0, 0);
                 }
             }
 
             if (retval == pdc_false)
             {
-                pdc_logg_cond(p->pdc, 1, trc_font,
-                    "\tOutline data file for font \"%s\" not found\n",
-                    fontname_p);
+                    if (logg1)
+                        pdc_logg(p->pdc,
+                            "\tOutline data file for font \"%s\" not found\n",
+                            fontname_p);
             }
             else
             {
                 if (!font->ft.img)
                     font->filename = font->ft.filename;
 
-                pdc_logg_cond(p->pdc, 1, trc_font,
-                    "\tFont outline data file \"%s\" available\n",
-                    font->filename ?
-                    font->filename : font->ft.imgname);
+                    if (logg1)
+                        pdc_logg(p->pdc,
+                                "\tFont outline data file \"%s\" available\n",
+                                font->filename ?
+                                font->filename : font->ft.imgname);
             }
         }
     }
@@ -1719,7 +1760,9 @@ pdf_write_fontdescriptor(
     pdc_printf(p->out, "/Ascent %d\n", font->ft.m.ascender);
     pdc_printf(p->out, "/CapHeight %d\n", font->ft.m.capHeight);
     pdc_printf(p->out, "/Descent %d\n", font->ft.m.descender);
-    pdc_printf(p->out, "/FontBBox[%d %d %d %d]\n",
+
+    if (font->ft.m.type != fnt_Type3)
+        pdc_printf(p->out, "/FontBBox[%d %d %d %d]\n",
                         (int) font->ft.m.llx, (int) font->ft.m.lly,
                         (int) font->ft.m.urx, (int) font->ft.m.ury);
 
@@ -1728,7 +1771,9 @@ pdf_write_fontdescriptor(
     pdc_puts(p->out, "\n");
 
     pdc_printf(p->out, "/ItalicAngle %d\n", (int) (font->ft.m.italicAngle));
-    pdc_printf(p->out, "/StemV %d\n", font->ft.m.StdVW);
+
+    if (font->ft.m.type != fnt_Type3)
+        pdc_printf(p->out, "/StemV %d\n", font->ft.m.StdVW);
 
     if (font->ft.m.StdHW > 0)
         pdc_printf(p->out, "/StemH %d\n", font->ft.m.StdHW);
@@ -1793,6 +1838,7 @@ pdf_put_font(PDF *p, pdf_font *font)
     /* save font struct members */
     pdc_encodingvector *ev = NULL;
     pdc_encoding       font_encoding = font->ft.enc;
+    pdc_encoding       font_towinansi = font->towinansi;
     int                font_numcodes = font->ft.numcodes;
     int                font_codesize = font->codesize;
 
@@ -1963,13 +2009,17 @@ pdf_put_font(PDF *p, pdf_font *font)
             {
                 fontdescriptor_id = pdc_alloc_id(p->out);
                 pdc_objref(p->out, "/FontDescriptor", fontdescriptor_id);
+
+                /* bug #1036 */
+                if (font->ft.isstdfont == pdc_true && font->opt.dropcorewidths)
+                    break;
             }
 
             /* determine missing width.
              * Only for embedded fonts because of a bug in Acrobat,
              * which arises if the font is not installed at host.
              */
-            if (font->opt.embedding)
+            if (font->opt.embedding && !font->used_in_formfield)
             {
                 if (fonttype != fnt_Type3)
                     defwidth = font->widths[0];
@@ -2426,6 +2476,7 @@ pdf_put_font(PDF *p, pdf_font *font)
 
     /* restore font struct members */
     font->ft.enc = font_encoding;
+    font->towinansi = font_towinansi;
     font->ft.numcodes = font_numcodes;
     font->codesize = font_codesize;
 
@@ -2501,7 +2552,6 @@ void
 pdf_write_page_fonts(PDF *p)
 {
     int i, total = 0;
-    int bias = p->curr_ppt->fn_bias;
 
     /* This doesn't really belong here, but all modules which write
      * font resources also need this, so we include it here.
@@ -2516,7 +2566,7 @@ pdf_write_page_fonts(PDF *p)
         if (p->fonts[i].used_on_current_page == pdc_true)
             total++;
 
-    if (total > 0 || bias)
+    if (total > 0)
     {
         pdc_puts(p->out, "/Font");
         pdc_begin_dict(p->out);         /* font resource dict */
@@ -2528,12 +2578,11 @@ pdf_write_page_fonts(PDF *p)
 	{
             if (p->fonts[i].used_on_current_page == pdc_true) {
                 p->fonts[i].used_on_current_page = pdc_false;   /* reset */
-                pdc_printf(p->out, "/F%d", bias + i);
+                pdc_printf(p->out, "/F%d", i);
                 pdc_objref(p->out, "", p->fonts[i].obj_id);
             }
 	}
 
-	if (!bias)
 	    pdc_end_dict(p->out);	/* font resource dict */
     }
 }
@@ -2558,6 +2607,3 @@ pdf_mark_page_font(PDF *p, int ft)
 {
     p->fonts[ft].used_on_current_page = pdc_true;
 }
-
-
-
