@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_generr.h,v 1.338.2.44 2008/02/27 19:48:11 kurt Exp $
+/* $Id: p_generr.h,v 1.338.2.61 2010/04/16 12:32:27 kurt Exp $
  *
  * PDFlib error messages
  *
@@ -222,6 +222,8 @@ gen(1, 2220, TEMPLATE_SELF,
 gen(1, 2222, TEXT_ALIGNCHARNOTFOUND,
     "Character U+$1 for option 'alignchar' not found in font or encoding")
 
+gen(1, 2223, TEXT_BADTEXTFORMAT, "Bad textformat '$1' (must be 'bytes')")
+
 /* UNUSED
 gen(1, 2230, TEXT_UNICODENOTSHOW,
     "Can't show character with Unicode value U+$1")
@@ -285,6 +287,8 @@ gen(0, 2352, COLOR_INVALSPEC,
 
 gen(2, 2400, IMAGE_CORRUPT, "Corrupt $1 image file '$2'")
 
+gen(4, 2401, IMAGE_TOO_LARGE, "$1 image '$2' too large ($3 x $4)")
+
 gen(3, 2402, IMAGE_NOPAGE, "Requested page $1 in $2 image '$3' not found")
 
 gen(2, 2404, IMAGE_BADDEPTH,
@@ -398,6 +402,10 @@ gen(2, 2483, TIFF_UNSUPP_COLORSPACE,
 gen(1, 2484, TIFF_UNSUPP_SEPARATE,
     "Couldn't open TIFF image '$1' (separate image planes not supported)")
 
+gen(1, 2485, TIFF_TILE_UNSUPP,
+    "Couldn't open TIFF image '$1' "
+    "(separate image planes with tiles not supported)")
+
 gen(2, 2486, TIFF_UNSUPP_SEP_NONCMYK,
     "Couldn't open TIFF image '$1' (unsupported inkset tag $2)")
 
@@ -426,6 +434,8 @@ gen(1, 2496, JPX_RAWDATAUNSUPP,
 gen(1, 2498, JPX_COMPOUNDUNSUPP,
     "Compound JPEG2000 (JPM) image file '$1' not supported")
 
+gen(1, 2499, TIFF_BPC_UNSUPP,
+"TIFF image '$1' contains bpc that not equal to 1,2, 4, 8, 16, not supported")
 
 /* -------------------------------------------------------------------- */
 /* Font							(25xx)		*/
@@ -463,9 +473,18 @@ gen(0, 2509, FONT_FORCECVTUNI,
 
 gen(0, 2514, FONT_EMBEDMM, "Multiple Master font cannot be embedded")
 
-gen(0, 2516, FONT_NOMETRICS, "Metrics data not found")
+gen(0, 2516, FONT_NOMETRICS,
+    "Font file (AFM, PFM, TTF, OTF etc.) or host font not found")
 
+gen(0, 2517, FONT_NOOUTLINE_PS,
+    "Font cannot be embedded (PFA or PFB font file not found)")
+
+/* Unused
 gen(0, 2518, FONT_NOOUTLINE, "File with outline data not found")
+*/
+
+gen(0, 2519, FONT_NOOUTLINE_TT,
+    "Font cannot be embedded (TTF or OTF font file not found)")
 
 /* Unused
 gen(0, 2520, FONT_NOGLYPHID, "Font doesn't contain any glyph IDs")
@@ -473,6 +492,9 @@ gen(0, 2520, FONT_NOGLYPHID, "Font doesn't contain any glyph IDs")
 
 gen(0, 2522, FONT_FORCEEMBED2, "Metadata requires embedding")
 
+gen(0, 2523, FONT_CORRUPT_PFA,
+    "Corrupt PFA font data (invalid hex code after 'currentfile eexec')")
+
 gen(0, 2530, CJK_UNSUPP_REGISTRY,
     "Font not supported (contains non-Adobe registry in CMap)")
 
@@ -631,6 +653,10 @@ gen(1, 2830, FF_NOEMBEDFONT,
 gen(1, 2832, FF_SUBSETTFONT,
     "Specified font '$1' not allowed for fields (must not be subset)")
 
+gen(2, 2833, FF_NOPDFDOCFONT,
+    "Specified encoding '$1' of font '$2' not recommended for fields "
+    "(please use encoding 'pdfdoc')")
+
 gen(1, 2834, FF_CAPTMISSING, "No caption or icon specified for field '$1'")
 
 gen(0, 2836, FF_DIFFSTRLISTS,
@@ -715,7 +741,7 @@ gen(1, 2912, INT_SSTACK_UNDER, "State stack underflow in function '$1'")
 gen(3, 2914, INT_WRAPPER, "Error in PDFlib $1 wrapper function $2 ($3)")
 
 gen(0, 2990, OT_UNSUPP_SID2CID,
-    "Accented characters not supported; contact support@pdflib.com")
+"Accented characters not supported; use autocidfont=false in PDF_load_font()")
 
 
 
