@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_gif.c,v 1.70.2.5 2007/11/08 17:50:15 kurt Exp $
+/* $Id: p_gif.c,v 1.70.2.6 2008/03/18 16:55:00 kurt Exp $
  *
  * GIF processing for PDFlib
  *
@@ -462,7 +462,7 @@ GetDataBlock(PDF *p, pdf_image *image, unsigned char *lbuf)
     if (!PDC_OK_FREAD(fp, &count, 1))
 	return -1;	/* Error in getting DataBlock size */
 
-    ZeroDataBlock = (count == 0);
+    ZeroDataBlock = (count == (unsigned char) 0);
 
     if ((count != (unsigned char) 0) && (!PDC_OK_FREAD(fp, lbuf, count)))
     {
