@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_hyper.c,v 1.191.2.24 2008/02/26 16:03:41 kurt Exp $
+/* $Id: p_hyper.c,v 1.191.2.31 2009/10/19 15:39:40 kurt Exp $
  *
  * PDFlib routines for hypertext stuff:
  * named destination, bookmarks, document info
@@ -1326,25 +1326,38 @@ pdf__set_info(PDF *p, const char *key, const char *value, int len)
     static const char fn[] = "pdf__set_info";
     char *key_buf, *val_buf;
     pdf_info *oldentry, *newentry;
+    const char **kp;
+    static const char *forbidden_keys[] =
+    {
+        "Producer",
+        "CreationDate",
+        "ModDate",
+        "GTS_PDFXVersion",
+        "GTS_PDFXConformance",
+        "ISO_PDFEVersion"
+    };
 
     if (key == NULL || !*key)
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "key", 0, 0, 0);
 
     len = pdc_check_text_length(p->pdc, &value, len, PDF_MAXSTRINGSIZE);
-    if (!len)
-        pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "value", 0, 0, 0);
 
-    if (!strcmp(key, "Producer") ||
-        !strcmp(key, "CreationDate") ||
-        !strcmp(key, "ModDate"))
-        pdc_error(p->pdc, PDC_E_ILLARG_STRING, "key", key, 0, 0);
+    for (kp = forbidden_keys;
+        kp != sizeof(forbidden_keys) / sizeof(char *) + forbidden_keys;
+        kp += 1)
+    {
+        if (!strcmp(*kp, key))
+        {
+            pdc_error(p->pdc, PDC_E_ILLARG_STRING, "key", key, 0, 0);
+        }
+    }
 
     /* converting key */
     key_buf = pdf_convert_name(p, key, 0, 0);
 
     /* convert text string */
     val_buf = pdf_convert_hypertext_depr(p, value, len);
-    if (!val_buf || !*val_buf)
+    if (val_buf == NULL)
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "value", 0, 0, 0);
 
     /* special handling required for "Trapped" */
@@ -1384,6 +1397,7 @@ pdf__set_info(PDF *p, const char *key, const char *value, int len)
 pdc_id
 pdf_write_info(PDF *p, pdc_bool moddate)
 {
+    pdc_bool logg3 = pdc_logg_is_enabled(p->pdc, 3, trc_xmp);
     char time_str[PDC_TIME_SBUF_SIZE];
     char producer[PDC_GEN_BUFSIZE];
     pdf_info    *info;
@@ -1391,7 +1405,10 @@ pdf_write_info(PDF *p, pdc_bool moddate)
 
 
 
-    const char  *product = "PDFlib Lite";
+    const char *product = "PDFlib Lite";
+    const char *security = "";
+
+    (void) logg3;
 
 
     if (!p->pdc->smokerun)
@@ -1446,11 +1463,11 @@ pdf_write_info(PDF *p, pdc_bool moddate)
      */
 
     if (p->pdc->binding)
-        pdc_sprintf(p->pdc, pdc_false, producer, "%s %s (%s/%s)", product,
-            PDFLIB_VERSIONSTRING, p->pdc->binding, PDF_PLATFORM);
+        pdc_sprintf(p->pdc, pdc_false, producer, "%s %s%s (%s/%s)", product,
+            PDFLIB_VERSIONSTRING, security, p->pdc->binding, PDF_PLATFORM);
     else
-        pdc_sprintf(p->pdc, pdc_false, producer, "%s %s (%s)", product,
-            PDFLIB_VERSIONSTRING, PDF_PLATFORM);
+        pdc_sprintf(p->pdc, pdc_false, producer, "%s %s%s (%s)", product,
+            PDFLIB_VERSIONSTRING, security, PDF_PLATFORM);
 
     pdc_puts(p->out, "/Producer ");
     pdf_put_hypertext(p, producer);
