@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_image.c,v 1.318.2.22 2008/02/21 15:56:42 kurt Exp $
+/* $Id: p_image.c,v 1.318.2.31 2009/03/03 21:25:20 kurt Exp $
  *
  * PDFlib image routines
  *
@@ -112,6 +112,8 @@ pdf_grow_images(PDF *p)
 
     for (im = p->images_capacity; im < 2 * p->images_capacity; im++)
 	pdf_init_image_struct(p, &(p->images[im]));
+    for (im = 0; im < p->images_capacity; im++)
+	p->images[im].src.private_data = &(p->images[im]);
 
     p->images_capacity *= 2;
 }
@@ -400,7 +402,10 @@ pdf_put_inline_image(PDF *p, int im)
 	    if ((int) image->width != 1728)	/* CCITT default width */
 		pdc_printf(p->out, "/Columns %d", (int) image->width);
 
-	    /* /Rows is not required */
+	    /* /Rows is optional, but it helps in certain cases with damaged
+	     * CCITT compressed image data
+	     */
+	    pdc_printf(p->out, "/Rows %d", (int) fabs(image->height));
 	}
 	pdc_puts(p->out, ">>]");		/* DecodeParms dict and array */
     }
@@ -618,9 +623,10 @@ pdf_put_image(PDF *p, int im, pdc_bool firststrip, pdc_bool checkcontentstream)
 	    if ((int) image->width != 1728)	/* CCITT default width */
 		pdc_printf(p->out, "/Columns %d", (int) image->width);
 
-	    /*
+	    /* /Rows is optional, but it helps in certain cases with damaged
+	     * CCITT compressed image data
+	     */
 	    pdc_printf(p->out, "/Rows %d", (int) fabs(image->height));
-	    */
 
             if (image->K != 0)
                 pdc_printf(p->out, "/K %d", image->K);
@@ -650,6 +656,9 @@ pdf_put_image(PDF *p, int im, pdc_bool firststrip, pdc_bool checkcontentstream)
 
 	/* We must avoid pdc_begin/end_pdfstream() here in order to
 	 * generate a really empty stream.
+	 * Note: this doesn't work with AES encryption, but we don't
+	 * bother to fix it since file references are no longer
+	 * supported anyway.
 	 */
 	pdc_puts(p->out, "stream\n");	/* dummy image stream */
 	pdc_puts(p->out, "endstream\n");
@@ -711,7 +720,10 @@ pdf_put_image(PDF *p, int im, pdc_bool firststrip, pdc_bool checkcontentstream)
 	    if ((int) image->width != 1728)	/* CCITT default width */
 		pdc_printf(p->out, "/Columns %d", (int) image->width);
 
-	    /* /Rows is not required */
+	    /* /Rows is optional, but it helps in certain cases with damaged
+	     * CCITT compressed image data
+	     */
+	    pdc_printf(p->out, "/Rows %d", (int) fabs(image->height));
 	}
 
 	pdc_puts(p->out, ">>\n");		/* DecodeParms dict */
@@ -1606,7 +1618,7 @@ static const pdc_defopt pdf_open_image_options[] =
      0.0, 0.0, NULL},
 
     {"clippingpathname", pdc_stringlist, PDF_CLIPPATH_FLAG, 1, 1,
-      1.0, PDC_INT_MAX, NULL},
+      1.0, 255.0, NULL},
 
     {"ignoremask", pdc_booleanlist, 0, 1, 1, 0.0, 0.0, NULL},
 
