@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_keyconn.h,v 1.117.2.17 2008/02/07 20:13:17 kurt Exp $
+/* $Id: p_keyconn.h,v 1.117.2.18 2009/04/15 12:06:35 york Exp $
  *
  * PDFlib shared keys connection lists
  *
@@ -422,6 +422,19 @@ static const pdc_keyconn pdf_mbox_keylist[] =
 #endif /* P_MBOX_C */
 
 
+#if defined(P_PAGE_C) || defined(P_TEMPLATE_C)
+
+static const pdc_keyconn pdf_tgroup_cs_pdfkeylist[] =
+{
+    {"DeviceGray",   color_gray},
+    {"DeviceRGB",    color_rgb},
+    {"DeviceCMYK",   color_cmyk},
+    {NULL, 0}
+};
+
+#endif /* P_PAGE_C || P_TEMPLATE_C */
+
+
 #if defined(P_DOCUMENT_C) || defined(P_PARAMS_C)
 
 static const pdc_keyconn pdf_compatibility_keylist[] =
