@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_mbox.c,v 1.57.2.18 2007/12/18 18:26:38 kurt Exp $
+/* $Id: p_mbox.c,v 1.57.2.22 2008/06/06 10:42:28 kurt Exp $
  *
  * PDFlib matchbox related routines
  *
@@ -452,9 +452,9 @@ pdf_set_mbox_rectangle(PDF *p, pdf_mbox *mbox, pdc_rectangle *rect, int flags)
     if (!(flags & mbox_statbottom))
     {
         if (mbox->percentbottom)
-            mbox->rect.lly += mbox->offsetbottom * height;
+            mbox->rect.lly += p->ydirection * mbox->offsetbottom * height;
         else
-            mbox->rect.lly += mbox->offsetbottom;
+            mbox->rect.lly += p->ydirection * mbox->offsetbottom;
     }
 
     if (!(flags & mbox_statright))
@@ -468,9 +468,9 @@ pdf_set_mbox_rectangle(PDF *p, pdf_mbox *mbox, pdc_rectangle *rect, int flags)
     if (!(flags & mbox_stattop))
     {
         if (mbox->percenttop)
-            mbox->rect.ury += mbox->offsettop * height;
+            mbox->rect.ury += p->ydirection * mbox->offsettop * height;
         else
-            mbox->rect.ury += mbox->offsettop;
+            mbox->rect.ury += p->ydirection * mbox->offsettop;
     }
 }
 
@@ -620,7 +620,7 @@ pdf_get_usematchbox(PDF *p, const char *option, const char *optval,
                               &strlist);
     if (ns)
     {
-        boxname = pdc_errprintf(p->pdc, "%.*s", PDC_ERR_MAXSTRLEN, strlist[0]);
+        boxname = pdc_strdup_tmp(p->pdc, strlist[0]);
 
         /* number of rectangles */
         pdf_get_mbox(p, NULL, boxname, 0, &nrect);
@@ -694,7 +694,6 @@ pdf__info_matchbox(PDF *p, const char *boxname, int len, int num,
                    const char *keyword)
 {
     pdf_mbox *mbox;
-    char *cname;
     double mbinfo = 0;
     int infokey, count;
 
@@ -705,11 +704,9 @@ pdf__info_matchbox(PDF *p, const char *boxname, int len, int num,
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "keyword", 0, 0, 0);
 
     /* Converting boxname */
-    cname = pdf_convert_name(p, boxname, len, 0);
-    if (cname == NULL || *cname == '\0')
+    boxname = pdf_convert_name(p, boxname, len, PDC_CONV_TMPALLOC);
+    if (boxname == NULL || *boxname == '\0')
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "boxname", 0, 0, 0);
-    boxname = pdc_errprintf(p->pdc, "%.*s", PDC_ERR_MAXSTRLEN, cname);
-    pdc_free(p->pdc, cname);
 
     infokey = pdc_get_keycode_ci(keyword, pdf_info_keylist);
     if (infokey == PDC_KEY_NOTFOUND)
