@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_page.c,v 1.116.2.18 2008/01/28 13:47:35 york Exp $
+/* $Id: p_page.c,v 1.116.2.25 2010/01/14 13:56:48 kurt Exp $
  *
  * PDFlib page related routines
  *
@@ -67,21 +67,6 @@ static const pdc_keyconn pdf_taborder_pdfkeylist[] =
     {NULL, 0}
 };
 
-static const pdc_keyconn pdf_colorspace_pdfkeylist[] =
-{
-    {"DeviceGray",   color_gray},
-    {"DeviceRGB",    color_rgb},
-    {"DeviceCMYK",   color_cmyk},
-    {NULL, 0}
-};
-
-typedef struct
-{
-    pdf_colortype       colorspace;     /* color space                  */
-    pdc_bool            isolated;       /* isolated flag I              */
-    pdc_bool            knockout;       /* knockout flag K              */
-} pg_transgroup;
-
 typedef struct
 {
     pdf_labelstyle      style;          /* page label style             */
@@ -160,7 +145,7 @@ typedef struct
     pdc_scalar          userunit;       /* page user unit */
     char *		action;		/* "action" option string */
 
-    pg_transgroup       tgroup;         /* transparency group definition */
+    pdf_transgroup	tgroup;         /* transparency group definition */
     pdc_bool            autotgroup;     /* create transp. group automatically */
 
 
@@ -219,31 +204,28 @@ static const pdc_rectangle pdf_null_rect =
 /*********************** initialization & cleanup ***********************/
 
 static void
-pdf_init_ppt(PDF *p, pdc_bool new_ppt)
+pdf_init_ppt(PDF *p)
 {
     pdf_ppt *ppt = p->curr_ppt;
 
-    if (new_ppt)
-    {
-	ppt->cstate = 0;
-	ppt->tstate = 0;
+    ppt->cstate = (pdf_cstate *) 0;
+    ppt->tstate = (pdf_tstate *) 0;
 
-        ppt->mboxes = (pdc_vtr *) 0;
+    ppt->mboxes = (pdc_vtr *) 0;
 
-	ppt->cs_bias = 0;
-	ppt->eg_bias = 0;
-	ppt->fn_bias = 0;
-	ppt->pt_bias = 0;
-	ppt->sh_bias = 0;
-	ppt->xo_bias = 0;
-    }
+} /* pdf_init_ppt */
+
+static void
+pdf_init_ppt_states(PDF *p)
+{
+    pdf_ppt *ppt = p->curr_ppt;
 
     ppt->sl = 0;
 
     pdf_init_tstate(p);
     pdf_init_gstate(p);
     pdf_init_cstate(p);
-} /* pdf_init_ppt */
+} /* pdf_init_ppt_states */
 
 static void
 pdf_reset_ppt(pdf_ppt *ppt)
@@ -365,6 +347,9 @@ pdf_init_pages(PDF *p, const char **groups, int n_groups)
     dp->pages = (page_obj *) 0;
     dp->pnodes = (pdc_id *) 0;
 
+    p->curr_ppt = &dp->default_ppt;
+    pdf_init_ppt(p);
+
     dp->pages_capacity = PAGES_CHUNKSIZE;
     dp->pages = (page_obj *)
 	    pdc_malloc(p->pdc, sizeof (page_obj) * dp->pages_capacity, fn);
@@ -385,13 +370,6 @@ pdf_init_pages(PDF *p, const char **groups, int n_groups)
     dp->current_pnode	= 0;
     dp->current_pnode_kids = 0;
 
-    /* clients may set char/word spacing and horizontal scaling outside pages
-    ** for PDF_stringwidth() calculations, and they may set a color for use
-    ** in PDF_makespotcolor(). that's what default_ppt is good for.
-    */
-    p->curr_ppt = &dp->default_ppt;
-    pdf_init_ppt(p, pdc_true);
-
     for (i = 0; i < n_groups - 1; ++i)
 	for (k = i + 1; k < n_groups; ++k)
 	    if (strcmp(groups[i], groups[k]) == 0)
@@ -412,6 +390,13 @@ pdf_init_pages(PDF *p, const char **groups, int n_groups)
 	dp->groups[i].label.prefix = (char *) 0;
 	dp->groups[i].label.start = 0;
     }
+
+    /* clients may set char/word spacing and horizontal scaling outside pages
+    ** for PDF_stringwidth() calculations, and they may set a color for use
+    ** in PDF_makespotcolor(). that's what default_ppt is good for.
+    */
+    pdf_init_ppt_states(p);
+
 } /* pdf_init_pages */
 
 
@@ -808,22 +793,20 @@ pdf_write_pagelabels(PDF *p)
 static const pdc_defopt pdf_transgroup_options[] =
 {
     {"CS", pdc_keywordlist, PDC_OPT_NONE, 1, 1,
-      1.0, PDC_INT_MAX, pdf_colorspace_pdfkeylist},
+      1.0, PDC_INT_MAX, pdf_tgroup_cs_pdfkeylist},
 
     {"I", pdc_booleanlist, PDC_OPT_NONE, 1, 1,
       0.0, 0.0, NULL},
 
-    {"K", pdc_booleanlist, PDC_OPT_NONE, 1, 1, \
-      0.0, 0.0, NULL}, \
+    {"K", pdc_booleanlist, PDC_OPT_NONE, 1, 1,
+      0.0, 0.0, NULL},
 
     PDC_OPT_TERMINATE
 };
 
-static void
-pdf_set_transgroup(PDF *p, const char *optlist, int pageno)
+void
+pdf_set_transgroup(PDF *p, const char *optlist, pdf_transgroup *tgroup)
 {
-    pdf_pages * dp = p->doc_pages;
-    page_obj *  po = &dp->pages[pageno];
     pdc_resopt *resopts = NULL;
     int         inum;
 
@@ -831,38 +814,28 @@ pdf_set_transgroup(PDF *p, const char *optlist, int pageno)
                                    NULL, pdc_true);
 
     if (pdc_get_optvalues("CS", resopts, &inum, NULL))
-        po->tgroup.colorspace = (pdf_colortype) inum;
+        tgroup->colorspace = (pdf_colortype) inum;
 
-    pdc_get_optvalues("I", resopts, &po->tgroup.isolated, NULL);
-    pdc_get_optvalues("K", resopts, &po->tgroup.knockout, NULL);
+    pdc_get_optvalues("I", resopts, &tgroup->isolated, NULL);
+    pdc_get_optvalues("K", resopts, &tgroup->knockout, NULL);
 }
 
-static void
-write_transgroup(PDF *p, int pageno)
+void
+pdf_write_transgroup(PDF *p, pdf_transgroup *tgroup)
 {
-    pdf_pages * dp = p->doc_pages;
-    page_obj *po = &dp->pages[pageno];
-
     pdc_puts(p->out, "/Group");
     pdc_begin_dict(p->out);
 
     pdc_puts(p->out, "/S/Transparency/CS/");
 
-    if (po->tgroup.colorspace != color_none)
-    {
-        pdc_printf(p->out, "%s", pdc_get_keyword(po->tgroup.colorspace,
-                                                 pdf_colorspace_pdfkeylist));
+    pdc_printf(p->out, "%s", pdc_get_keyword(tgroup->colorspace,
+					     pdf_tgroup_cs_pdfkeylist));
 
-        if (po->tgroup.isolated)
-            pdc_puts(p->out, "/I true");
+    if (tgroup->isolated)
+	pdc_puts(p->out, "/I true");
 
-        if (po->tgroup.knockout)
-            pdc_puts(p->out, "/K true");
-    }
-    else
-    {
-        pdc_puts(p->out, "DeviceRGB");
-    }
+    if (tgroup->knockout)
+	pdc_puts(p->out, "/K true");
 
     pdc_end_dict(p->out);
 }
@@ -960,7 +933,7 @@ pdf_search_page_bwd(PDF *p, int start_page, pdc_id id)
 } /* pdf_search_page_bwd */
 
 
-double
+pdc_scalar
 pdf_get_pageheight(PDF *p)
 {
     pdf_pages *	dp = p->doc_pages;
@@ -1365,8 +1338,11 @@ pdf_write_pages_tree(PDF *p)
 	    pdc_end_dict(p->out);
 	}
 
-        if (po->tgroup.colorspace != color_none || po->autotgroup)
-            write_transgroup(p, i);
+        if (po->tgroup.colorspace == color_none && po->autotgroup)
+	    po->tgroup.colorspace = color_rgb;
+
+        if (po->tgroup.colorspace != color_none)
+            pdf_write_transgroup(p, &po->tgroup);
 
 	pdf_write_box(p, po->boxes[pdf_artbox],		"ArtBox");
 	pdf_write_box(p, po->boxes[pdf_bleedbox],	"BleedBox");
@@ -1674,7 +1650,9 @@ get_page_options3(PDF *p, pdc_resopt *resopts, pdc_bool end_page)
     }
 
     if (pdc_get_optvalues("transparencygroup", resopts, NULL, &slist))
-        pdf_set_transgroup(p, slist[0], pageno);
+    {
+        pdf_set_transgroup(p, slist[0], &po->tgroup);
+    }
 
     /* the "width" and "height" options must be processed BEFORE the
     ** "mediabox" option, since the latter dominates over the formers.
@@ -1829,6 +1807,7 @@ pdf__begin_page_ext(
 
     pg = dp->curr_pg = (pdf_page *) pdc_malloc(p->pdc, sizeof (pdf_page), fn);
     p->curr_ppt = &pg->ppt;
+    pdf_init_ppt(p);
 
     pg->contents_ids = (pdc_id *) 0;
     pg->annots = (pdc_vtr *) 0;
@@ -1867,7 +1846,7 @@ pdf__begin_page_ext(
     /* initialize the current ppt descriptor. p->ydirection
     ** must be set before pdf_init_ppt()!
     */
-    pdf_init_ppt(p, pdc_true);
+    pdf_init_ppt_states(p);
 
     pg->next_content = 0;
 
@@ -1950,7 +1929,7 @@ pdf_pg_suspend(PDF *p)
 	p->curr_ppt = &dp->default_ppt;
     }
 
-    pdf_init_ppt(p, pdc_false);
+    pdf_init_ppt_states(p);
 } /* pdf_pg_suspend */
 
 
@@ -2237,7 +2216,7 @@ pdf__end_page_ext(PDF *p, const char *optlist)
     /* restore the default ppt for out-of-page usage.
     */
     p->curr_ppt = &dp->default_ppt;
-    pdf_init_ppt(p, pdc_false);
+    pdf_init_ppt_states(p);
     PDF_SET_STATE(p, pdf_state_document);
     pdf_delete_page(p, pg);
     dp->curr_pg = (pdf_page *) 0;
