@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_params.c,v 1.252.2.17 2008/02/07 19:49:20 kurt Exp $
+/* $Id: p_params.c,v 1.252.2.20 2008/11/24 13:54:51 kurt Exp $
  *
  * PDFlib parameter handling
  *
@@ -26,6 +26,7 @@
 #include "p_tagged.h"
 
 #include "pc_scope.h"
+#include "pc_file.h"
 
 /*
  * PDF_get_parameter() and PDF_set_parameter() deal with strings,
@@ -167,7 +168,7 @@ pdf__set_parameter(PDF *p, const char *key, const char *value)
         case PDF_PARAMETER_ICCPROFILE:
         case PDF_PARAMETER_STANDARDOUTPUTINTENT:
 	{
-            pdf_add_resource(p, key, value);
+            pdf_add_pdflib_resource(p, key, value);
             break;
         }
 
@@ -577,6 +578,15 @@ pdf__set_value(PDF *p, const char *key, double value)
 
     switch (i)
     {
+#if defined(WIN32) && !defined(__BORLANDC__)
+        case PDF_PARAMETER_MAXFILEHANDLES:
+            ivalue = pdc_set_maxfilehandles(p->pdc, ivalue);
+            if (ivalue == -1)
+                pdc_error(p->pdc, PDC_E_PAR_ILLVALUE,
+                    pdc_errprintf(p->pdc, "%f", value), key, 0, 0);
+        break;
+#endif
+
         case PDF_PARAMETER_COMPRESS:
 	    if (ivalue < 0 || ivalue > 9)
 		pdc_error(p->pdc, PDC_E_PAR_ILLVALUE,
@@ -837,6 +847,12 @@ pdf__get_value(PDF *p, const char *key, double mod)
 
     switch (i)
     {
+#if defined(WIN32) && !defined(__BORLANDC__)
+        case PDF_PARAMETER_MAXFILEHANDLES:
+            result = (double) pdc_get_maxfilehandles();
+        break;
+#endif
+
         case PDF_PARAMETER_COMPRESS:
             result = (double) pdc_get_compresslevel(p->out);
             break;
