@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_tiff.c,v 1.134.2.23 2008/02/27 17:48:22 kurt Exp $
+/* $Id: p_tiff.c,v 1.134.2.41 2010/01/14 11:04:03 tm Exp $
  *
  * TIFF processing for PDFlib
  *
@@ -240,6 +240,7 @@ pdf_data_source_TIFF_fill(PDF *p, PDF_data_source *src)
         }
         else
         {
+            {
             if (image->info.tiff.cur_line++ == image->height)
             {
                 PDC_EXIT_TRY(p->pdc);
@@ -308,6 +309,7 @@ pdf_data_source_TIFF_fill(PDF *p, PDF_data_source *src)
                 pdc_error(p->pdc, PDF_E_IMAGE_BADCOMP,
                       pdc_errprintf(p->pdc, "%d", image->components),
                       pdf_get_image_filename(p, image), 0, 0);
+                }
             }
         }
     }
@@ -548,6 +550,18 @@ pdf_process_TIFF_data(
 	image->use_raw = pdc_false;
     }
 
+    /* Can't pass through grayscale or RGB with additional alpha channel.
+     * Some TIFF images do not count the alpha channel as extra channel.
+     */
+    if (extra == 0 && 
+	((photometric == PHOTOMETRIC_RGB && components == 4) ||
+	 ((photometric == PHOTOMETRIC_MINISBLACK || 
+	   photometric == PHOTOMETRIC_MINISWHITE) && components == 2)))
+    {
+	image->components -= 1;	/* ignore the alpha channel */
+	image->use_raw = pdc_false;
+    }
+
     /* PDF doesn't support other values of the color depth */
     if (bpc != 1 && bpc != 2 && bpc != 4 && bpc != 8 && bpc != 16)
 	image->use_raw = pdc_false;
@@ -575,7 +589,7 @@ pdf_process_TIFF_data(
     if (strips > 25 &&
 	compression != COMPRESSION_OJPEG && compression != COMPRESSION_JPEG &&
 	photometric != PHOTOMETRIC_PALETTE &&
-	image->width * image->width < PDF_TIFF_THRESHOLD)
+	image->width * image->height < PDF_TIFF_THRESHOLD)
     {
 	image->use_raw = pdc_false;
     }
@@ -835,6 +849,7 @@ pdf_process_TIFF_data(
 	    TIFFGetField(MYTIFF, TIFFTAG_COLORMAP, &rmap, &gmap, &bmap))
 	{
 	    image->components = 3;
+	    image->bpc = 8;
 	}
 	pdc_logg_cond(p->pdc, 1, trc_image, "\tno passthrough mode...\n");
     }
@@ -912,7 +927,6 @@ pdf_process_TIFF_data(
                     /* if it's not separated it must be RGB with alpha */
                     image->components = 3;
                     image->colorspace = DeviceRGB;
-                    image->compression = pdf_comp_none;
                 }
                 break;
 
@@ -928,6 +942,8 @@ pdf_process_TIFF_data(
     image->src.init		= pdf_data_source_TIFF_init;
     image->src.fill		= pdf_data_source_TIFF_fill;
     image->src.terminate	= pdf_data_source_TIFF_terminate;
+    image->in_use = pdc_true;	/* mark slot as used */
+    image->src.next_byte        = NULL;
 
     if (image->use_raw) {
 	uint32 row, rowsperstrip;
@@ -945,6 +961,25 @@ pdf_process_TIFF_data(
                 goto PDF_TIFF_ERROR;
 	    }
 
+	    /* CCITT compression implicitly carries 
+	     * photometric==PHOTOMETRIC_MINISWHITE. Although the combination
+	     * of CCITT compression and palette probably doesn't conform
+	     * to the TIFF spec, we accept it, but must invert the color
+	     * palette in order to match Acrobat's behavior.
+	     * Note that most TIFF viewers either ignore the palette in
+	     * this case or display the image with wrong colors.
+	     */
+	    if (compression == COMPRESSION_CCITTRLE ||
+		compression == COMPRESSION_CCITTRLEW ||
+		compression == COMPRESSION_CCITTFAX3 ||
+		compression == COMPRESSION_CCITTFAX4)
+	    {
+		image->invert = !image->invert;
+		pdc_logg_cond(p->pdc, 1, trc_image,
+		    "\tinverting colors for CCITT-compressed "
+		    "image with palette...\n");
+	    }
+
 	    cs.type = Indexed;
 	    cs.val.indexed.palette_size = 1 << bpc;
 	    cs.val.indexed.colormap = &colormap;
@@ -952,7 +987,7 @@ pdf_process_TIFF_data(
 
 	    cs.val.indexed.base = DeviceRGB;
 
-#define CVT(x) (uint16) (((x) * 255) / ((1L<<16)-1))
+#define CVT(x) (uint16) ((x)>>8)
 	    /* TODO: properly deal with 16-bit palette entries in PDF 1.5 */
 	    if (pdf_check_colormap(cs.val.indexed.palette_size,
 		rmap, gmap, bmap) == 16)
@@ -1024,6 +1059,8 @@ pdf_process_TIFF_data(
 	     */
 	    image->info.tiff.cur_line = strip;
 	    pdf_put_image(p, imageslot, pdc_false, pdc_false);
+            /* Refresh, in case p->images reallocate */
+            image = &p->images[imageslot];
 	}
 
         image->height = (pdc_scalar) height;
@@ -1059,7 +1096,12 @@ pdf_process_TIFF_data(
 	    image->pixelmode = pdc_true;
 	}
 
-	if (image->pixelmode)
+        if (planarconfig == PLANARCONFIG_SEPARATE)
+	{
+            errcode = PDF_E_TIFF_UNSUPP_SEPARATE;
+            goto PDF_TIFF_ERROR;
+        }
+	else if (image->pixelmode)
 	{
 	    npixels = (size_t) (width * height);
 
@@ -1077,18 +1119,10 @@ pdf_process_TIFF_data(
         {
 	    int linecounter = 0, sclsize = TIFFScanlineSize(MYTIFF);
 
-            if (planarconfig == PLANARCONFIG_SEPARATE)
-            {
-                errcode = PDF_E_TIFF_UNSUPP_SEPARATE;
-                goto PDF_TIFF_ERROR;
-            }
-            else
-            {
-                npixels = (size_t) (sclsize * height);
-                image->info.tiff.raster =
-                             (uint32 *)pdc_malloc(p->pdc, npixels, fn);
+            npixels = (size_t) (sclsize * height);
+	    image->info.tiff.raster = (uint32 *)pdc_malloc(p->pdc, npixels, fn);
 
-                while (linecounter < height)
+	    while (linecounter < (int) height)
                 {
                     size_t ndots = (size_t)((height - linecounter - 1) * width);
 
@@ -1108,15 +1142,16 @@ pdf_process_TIFF_data(
                     linecounter++;
                 }
             }
-        }
 
 	pdf_put_image(p, imageslot, pdc_true, pdc_true);
 
-	if (image->info.tiff.raster != NULL)
+        if (image->info.tiff.raster != NULL){
 	    pdc_free(p->pdc, (void *) image->info.tiff.raster);
+            image->info.tiff.raster = NULL;
+        }
+
     }
 
-    image->in_use = pdc_true;			/* mark slot as used */
 
     if (!image->corrupt)
     {
