@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_type3.c,v 1.120.2.9 2008/02/07 20:13:17 kurt Exp $
+/* $Id: p_type3.c,v 1.120.2.14 2009/07/30 16:52:33 kurt Exp $
  *
  * Routines for Type 3 (user-defined) fonts
  *
@@ -147,6 +147,7 @@ pdf_handle_t3font(PDF *p, const char *fontname, pdc_encoding enc,
     /* copy data from available font (see pdf__begin_font()) */
     font->ft.m.type = fnt_Type3;
     font->ft.matrix = deffont->ft.matrix;
+    font->ft.bbox = deffont->ft.bbox;
     font->t3font = deffont->t3font;
     font->ft.numglyphs = deffont->t3font->next_glyph;
     nalloc = (size_t) font->ft.numglyphs;
@@ -321,7 +322,6 @@ pdf__begin_font(
     const char *optlist)
 {
     static const char fn[] = "pdf__begin_font";
-    char *fname;
     pdc_resopt *results;
     pdf_font tmpfont, *font;
     pdf_font_options fo;
@@ -335,11 +335,10 @@ pdf__begin_font(
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "fontname", 0, 0, 0);
 
     /* Converting fontname */
-    fname = pdf_convert_name(p, fontname, len, PDC_CONV_WITHBOM);
-    if (fname == NULL || *fname == '\0')
+    fontname = pdf_convert_name(p, fontname, len,
+                                PDC_CONV_WITHBOM | PDC_CONV_TMPALLOC);
+    if (fontname == NULL || *fontname == '\0')
         pdc_error(p->pdc, PDC_E_ILLARG_EMPTY, "fontname", 0, 0, 0);
-    fontname = pdc_errprintf(p->pdc, "%.*s", PDC_ERR_MAXSTRLEN,  fname);
-    pdc_free(p->pdc, fname);
 
     pdc_logg_cond(p->pdc, 1, trc_font,
         "\tBegin of Type3 font \"%s\"\n", fontname);
@@ -574,6 +573,7 @@ pdf__begin_glyph(
     pdf_font *font;
     pdf_t3font *t3font;
     pdf_t3glyph *glyph = NULL;
+    pdc_scalar tbc;
     int ig;
 
     if (glyphname == NULL || *glyphname == '\0')
@@ -614,6 +614,21 @@ pdf__begin_glyph(
              urx != 0 || ury != 0))
             pdc_error(p->pdc, PDF_E_T3_BADBBOX, 0, 0, 0, 0);
 
+
+        if (urx < llx)
+        {
+            tbc = llx;
+            llx = urx;
+            urx = tbc;
+        }
+
+        if (ury < lly)
+        {
+            tbc = lly;
+            lly = ury;
+            ury = tbc;
+        }
+
         if (ig == t3font->capacity)
         {
             t3font->capacity *= 2;
