@@ -10,7 +10,7 @@
  |                                                                           |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_util.c,v 1.169.2.19 2007/12/06 12:53:45 kurt Exp $
+/* $Id: p_util.c,v 1.169.2.34 2009/10/09 09:35:27 kurt Exp $
  *
  * PDFlib utility functions
  *
@@ -320,6 +320,9 @@ pdf_convert_hypertext(PDF *p, const char *text, int len,
 
     /* conversion to UTF-16-BE or PDFDocEncoding / EBCDIC */
     pdf_set_convertflags(p, &convflags);
+    if (pdc_logg_is_enabled(p->pdc, 3, trc_text))
+        convflags |= PDC_CONV_LOGGING;
+
     pdc_convert_string(p->pdc, hypertextformat, codepage, inev,
                        intext, len,
                        &textformat, outev, &outtext, outlen,
@@ -332,9 +335,13 @@ pdf_convert_hypertext(PDF *p, const char *text, int len,
         pdc_text_format outtextformat = PDC_UTF8;
         pdc_byte *newtext = NULL;
 
+        convflags = PDC_CONV_WITHBOM;
+        if (pdc_logg_is_enabled(p->pdc, 3, trc_text))
+            convflags |= PDC_CONV_LOGGING;
+
         pdc_convert_string(p->pdc, textformat, 0, NULL, outtext, *outlen,
                            &outtextformat, NULL, &newtext, outlen,
-                           PDC_CONV_WITHBOM, verbose);
+                           convflags, verbose);
         pdc_free(p->pdc, outtext);
         outtext = newtext;
     }
@@ -348,18 +355,19 @@ pdf_convert_hypertext(PDF *p, const char *text, int len,
  */
 
 char *
-pdf_convert_pdfstring(PDF *p, const char *text, int inlen, int *outlen)
+pdf_convert_pdfstring(PDF *p, const char *text, int inlen, int convflags,
+                      int *outlen)
 {
-    pdc_byte *newtext;
-    pdc_encodingvector *outev = pdc_get_encoding_vector(p->pdc, pdc_pdfdoc);
-    int convflags = PDC_CONV_WITHBOM | PDC_CONV_TRYBYTES;
+    pdc_byte *newtext = NULL;
 
     if (pdc_is_utf8_bytecode(text))
     {
         pdc_text_format textformat = PDC_UTF8;
         pdc_text_format outtextformat = pdc_utf16be;
+        pdc_encodingvector *outev = pdc_get_encoding_vector(p->pdc, pdc_pdfdoc);
 
         pdf_set_convertflags(p, &convflags);
+
         pdc_convert_string(p->pdc, textformat, 0, NULL,
                            (pdc_byte *) text, inlen,
                            &outtextformat, outev, &newtext, outlen,
@@ -374,39 +382,49 @@ pdf_convert_pdfstring(PDF *p, const char *text, int inlen, int *outlen)
     return (char *) newtext;
 }
 
-static void
-pdf_put_hypertext_ext(PDF *p, const char *text, pdc_bool isfilename)
+void
+pdf_put_hypertext(PDF *p, const char *text)
 {
-    char *newtext;
-    int inlen = (int) pdc_strlen(text), outlen;
+    int convflags = PDC_CONV_WITHBOM | PDC_CONV_TRYBYTES;
+    int inlen = (int) pdc_strlen(text);
+    int outlen;
 
-    newtext = pdf_convert_pdfstring(p, text, inlen, &outlen);
+    char *newtext = pdf_convert_pdfstring(p, text, inlen, convflags, &outlen);
 
-    if (isfilename)
-    {
-        if (pdc_is_utf16be_unicode(newtext) && p->compatibility < PDC_1_7)
-            pdc_error(p->pdc, PDC_E_IO_UNSUPP_PDFUNINAME, 0, 0, 0, 0);
-        pdc_put_pdffilename(p->out, newtext, outlen);
-    }
-    else
-    {
-        pdc_put_pdfstring(p->out, newtext, outlen);
-    }
+    pdc_put_pdfstring(p->out, newtext, outlen);
 
     if (newtext != text)
         pdc_free(p->pdc, newtext);
 }
 
 void
-pdf_put_hypertext(PDF *p, const char *text)
+pdf_put_pdffilename(PDF *p, const char *text)
 {
-    pdf_put_hypertext_ext(p, text, pdc_false);
+    int convflags = PDC_CONV_FILENAME | PDC_CONV_TRYBYTES;
+    int inlen = (int) pdc_strlen(text);
+    int outlen;
+
+    char *newtext = pdf_convert_pdfstring(p, text, inlen, convflags, &outlen);
+
+    pdc_put_pdffilename(p->out, newtext, outlen);
+
+    if (newtext != text)
+        pdc_free(p->pdc, newtext);
 }
 
 void
-pdf_put_pdffilename(PDF *p, const char *text)
+pdf_put_pdfunifilename(PDF *p, const char *text)
 {
-    pdf_put_hypertext_ext(p, text, pdc_true);
+    int convflags = PDC_CONV_WITHBOM | PDC_CONV_TRYBYTES;
+    int inlen = (int) pdc_strlen(text);
+    int outlen;
+
+    char *newtext = pdf_convert_pdfstring(p, text, inlen, convflags, &outlen);
+
+    pdc_put_pdffilename(p->out, newtext, outlen);
+
+    if (newtext != text)
+        pdc_free(p->pdc, newtext);
 }
 
 
@@ -455,6 +473,9 @@ pdf_convert_name(PDF *p, const char *name, int len, int flags)
                             &newname, &newlen, &htenc, &htcp);
 
     flags |= PDC_CONV_EBCDIC;
+    if (pdc_logg_is_enabled(p->pdc, 3, trc_text))
+        flags |= PDC_CONV_LOGGING;
+
     resname = pdc_convert_name_ext(p->pdc, newname, newlen, htenc, htcp, flags);
     if (newname != name)
         pdc_free(p->pdc, newname);
@@ -476,6 +497,9 @@ pdf_convert_filename(PDF *p, const char *filename, int len,
                             &newfilename, &newlen, &htenc, &htcp);
 
     flags |= PDC_CONV_EBCDIC;
+    if (pdc_logg_is_enabled(p->pdc, 3, trc_filesearch))
+        flags |= PDC_CONV_LOGGING;
+
     resfilename = pdc_convert_filename_ext(p->pdc, newfilename, len,
                                            paramname, htenc, htcp, flags);
     if (newfilename != filename)
@@ -485,7 +509,7 @@ pdf_convert_filename(PDF *p, const char *filename, int len,
 }
 
 void
-pdf_add_resource(PDF *p, const char *category, const char *resname)
+pdf_add_pdflib_resource(PDF *p, const char *category, const char *resname)
 {
     char *newresname;
     int newlen;
@@ -526,8 +550,10 @@ pdf_get_opt_textlist(PDF *p, const char *keyword, pdc_resopt *resopts,
     {
         pdc_byte *string = NULL;
         pdc_encodingvector *inev = NULL, *outev = NULL;
-        pdc_text_format intextformat = pdc_bytes, outtextformat = pdc_utf16be;
-        int convflags = PDC_CONV_WITHBOM | PDC_CONV_TRYBYTES;
+        pdc_text_format intextformat = pdc_bytes;
+        pdc_text_format outtextformat = pdc_utf16be;
+        pdc_text_format textformat;
+        int convflags = PDC_CONV_WITHBOM;
         pdc_bool isutf8;
         int i, outlen;
 
@@ -547,8 +573,20 @@ pdf_get_opt_textlist(PDF *p, const char *keyword, pdc_resopt *resopts,
                     inev = pdc_get_encoding_vector(p->pdc, enc);
             }
 
-            /* PDFDocEncoding */
-            outev = pdc_get_encoding_vector(p->pdc, pdc_pdfdoc);
+            /* ugly solution of bug #2344 */
+            if (ishypertext == pdc_true)
+            {
+                outev = pdc_get_encoding_vector(p->pdc, pdc_pdfdoc);
+
+                /* conversion to PDFDocEncoding if possible */
+                convflags |= PDC_CONV_TRYBYTES;
+            }
+            else if (ishypertext == pdc_undef)
+            {
+                /* filename as hypertext */
+                outtextformat = PDC_UTF8;
+                convflags |= PDC_CONV_TRY7BYTES;
+            }
         }
         else
         {
@@ -561,9 +599,10 @@ pdf_get_opt_textlist(PDF *p, const char *keyword, pdc_resopt *resopts,
                 }
                 return 0;
             }
-            else if (enc >= 0)
+            else if (enc >= 0 && !isutf8)
             {
-                outev = pdc_get_encoding_vector(p->pdc, enc);
+                /* bug #2069: always conversion to UTF-16BE */
+                inev = pdc_get_encoding_vector(p->pdc, enc);
             }
         }
 
@@ -586,16 +625,17 @@ pdf_get_opt_textlist(PDF *p, const char *keyword, pdc_resopt *resopts,
             string = (pdc_byte *) strlist[i];
 
             {
-                if (ishypertext || isutf8)
+                if (ishypertext || isutf8 || inev != NULL)
                 {
                     intextformat = isutf8 ?  PDC_UTF8 : pdc_bytes;
 
-                    if (pdc_logg_is_enabled(p->pdc, 2, trc_optlist))
+                    if (pdc_logg_is_enabled(p->pdc, 3, trc_text))
                         convflags |= PDC_CONV_LOGGING;
                     pdf_set_convertflags(p, &convflags);
+                    textformat = outtextformat;
                     pdc_convert_string(p->pdc, intextformat, codepage, inev,
                                 string, (int) strlen((char *) string),
-                                &outtextformat, outev, &string, &outlen,
+                                &textformat, outev, &string, &outlen,
                                 convflags, pdc_true);
                     pdc_free(p->pdc, strlist[i]);
                     strlist[i] = (char *) string;
@@ -618,6 +658,98 @@ pdf_get_opt_textlist(PDF *p, const char *keyword, pdc_resopt *resopts,
     return ns;
 }
 
+char *
+pdf_get_opt_filename(PDF *p, const char *keyword, pdc_resopt *resopts,
+                     pdc_encoding enc, int codepage)
+{
+    pdc_bool logg1 = pdc_logg_is_enabled(p->pdc, 1, trc_optlist);
+    pdc_bool logg3 = pdc_logg_is_enabled(p->pdc, 3, trc_text);
+    pdc_byte *filename = NULL;
+    char **strlist;
+
+    if (pdc_get_optvalues(keyword, resopts, NULL, &strlist))
+    {
+        pdc_encodingvector *inev = NULL, *outev = NULL;
+        pdc_text_format intextformat = pdc_bytes;
+        pdc_text_format outtextformat = pdc_utf16; /* sic! */
+        int convflags = PDC_CONV_NOBOM | PDC_CONV_TRYBYTES | PDC_CONV_NEWALLOC;
+        pdc_bool isutf8;
+        int ic, outlen;
+
+        /* whole option list or string list is in UTF-8 */
+        isutf8 = pdc_is_lastopt_utf8(resopts);
+
+        if (!isutf8)
+        {
+            if (enc < 0 && enc != pdc_unicode && enc != pdc_cid)
+                enc = pdf_get_hypertextencoding(p, "auto", &codepage,
+                                                pdc_true);
+            if (enc >= 0)
+                inev = pdc_get_encoding_vector(p->pdc, enc);
+        }
+        else
+        {
+            intextformat = PDC_UTF8;
+        }
+
+        if (logg1)
+        {
+            if (isutf8)
+            {
+                pdc_logg(p->pdc, "\tOption \"%s\" is "PDC_UTF8_STRG" encoded\n",
+                         keyword);
+            }
+            else
+            {
+                pdc_logg(p->pdc, "\tOption \"%s\" is %s encoded\n",
+                         keyword, pdc_get_user_encoding(p->pdc, enc));
+            }
+        }
+
+        outev = pdc_get_encoding_vector(p->pdc, pdc_winansi);
+
+        if (logg3)
+            convflags |= PDC_CONV_LOGGING;
+        pdf_set_convertflags(p, &convflags);
+
+        pdc_convert_string(p->pdc, intextformat, codepage, inev,
+                    (pdc_byte *) strlist[0], (int) strlen(strlist[0]),
+                    &outtextformat, outev, &filename, &outlen,
+                    convflags, pdc_true);
+
+        if (outtextformat == pdc_utf16)
+        {
+            pdc_ushort uv, *unifilename = (pdc_ushort *) filename;
+            int code;
+
+            if (p->compatibility < PDC_1_7)
+                pdc_error(p->pdc, PDC_E_IO_UNSUPP_PDFUNINAME, 0, 0, 0, 0);
+
+            /* we must replace non-WinAnsi characters by period
+             * and omit the BOM to get a WinAnsi string.
+             */
+            outlen /= 2;
+            for (ic = 0; ic < outlen; ic++)
+            {
+                uv = unifilename[ic];
+
+                code = pdc_get_encoding_bytecode(p->pdc, outev, uv);
+                if (code <= 0)
+                    uv = PDC_UNICODE_PERIOD;
+
+                filename[ic] = (char) uv;
+            }
+            filename[ic] = 0;
+        }
+
+        if (logg3)
+            pdc_logg_hexdump(p->pdc, "output filename", "\t\t",
+                             (char *) filename, strlen((char *) filename));
+    }
+
+    return (char *) filename;
+}
+
 char *
 pdf_get_opt_utf8name(PDF *p, const char *keyword, pdc_resopt *resopts)
 {
@@ -656,7 +788,7 @@ pdf_get_errorpolicy(PDF *p, pdc_resopt *resopts, pdc_bool verbose)
 int
 pdf_check_opt_handle(void *opaque, int handle, pdc_opttype type)
 {
-    PDF *p = opaque;
+    PDF *p = (PDF*)opaque;
     int minval = 0, maxval = 0;
     pdc_bool empty = pdc_false;
 
@@ -772,5 +904,3 @@ pdf_set_clientdata(PDF *p, pdc_clientdata *cdata)
 
     cdata->compatibility = p->compatibility;
 }
-
-
