@@ -1,5 +1,5 @@
 /* PDFlib GmbH cvsid:
- * $Id: tif_dir.c,v 1.20 2005/12/21 14:12:52 rjs Exp $ */
+ * $Id: tif_dir.c,v 1.20.2.1 2010/04/22 13:59:59 tm Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -75,6 +75,10 @@ void _TIFFsetDoubleArray(TIFF* tif, double** dpp, double* dp, uint32 n)
 static int
 setExtraSamples(TIFF* tif, TIFFDirectory* td, va_list ap, uint32* v)
 {
+/* PDFlib GmbH: bug #2424, fix backported from libtiff 4.0.0alpha */
+/* XXX: Unassociated alpha data == 999 is a known Corel Draw bug, see below */
+#define EXTRASAMPLE_COREL_UNASSALPHA 999
+
 	uint16* va;
 	uint32 i;
 
@@ -84,12 +88,25 @@ setExtraSamples(TIFF* tif, TIFFDirectory* td, va_list ap, uint32* v)
 	va = va_arg(ap, uint16*);
 	if (*v > 0 && va == NULL)		/* typically missing param */
 		return (0);
-	for (i = 0; i < *v; i++)
-		if (va[i] > EXTRASAMPLE_UNASSALPHA)
-			return (0);
+        for (i = 0; i < *v; i++)
+        {
+                if (va[i] > EXTRASAMPLE_UNASSALPHA) {
+                        /*
+                         * XXX: Corel Draw is known to produce incorrect
+                         * ExtraSamples tags which must be patched here if we
+                         * want to be able to open some of the damaged TIFF
+                         * files:
+                         */
+                        if (va[i] == EXTRASAMPLE_COREL_UNASSALPHA)
+                                va[i] = EXTRASAMPLE_UNASSALPHA;
+                        else
+                                return 0;
+                }
+        }
 	td->td_extrasamples = (uint16) *v;
 	_TIFFsetShortArray(tif, &td->td_sampleinfo, va, td->td_extrasamples);
 	return (1);
+#undef EXTRASAMPLE_COREL_UNASSALPHA
 }
 
 static uint32
