@@ -1,5 +1,5 @@
 /* PDFlib GmbH cvsid:
- * $Id: tif_fax3.c,v 1.27 2006/02/23 12:04:32 kurt Exp $ */
+ * $Id: tif_fax3.c,v 1.27.2.1 2008/03/10 13:22:11 rjs Exp $ */
 
 /*
  * Copyright (c) 1990-1997 Sam Leffler
@@ -311,7 +311,8 @@ Fax3Decode2D(TIFF* tif, tidata_t buf, tsize_t occ, tsample_t s)
  * machine characteristics.
  */
 #if defined(__alpha) || _MIPS_SZLONG == 64 || \
-        defined(__sparcv9) || defined(__LP64__) || defined(__arch64__)
+        defined(__sparcv9) || defined(__LP64__) || defined(__arch64__) \
+	|| (defined(__x86_64__) && defined(__SunOS))
 /* #if SIZEOF_LONG == 8 */
 # define FILL(n, cp)							    \
     switch (n) {							    \
