@@ -1,5 +1,5 @@
 /* PDFlib GmbH cvsid:
- * $Id: tif_lzw.c,v 1.9 2005/12/21 14:12:52 rjs Exp $ */
+ * $Id: tif_lzw.c,v 1.9.2.1 2009/08/07 10:02:59 tm Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -651,6 +651,7 @@ LZWDecodeCompat(TIFF* tif, tidata_t op0, tsize_t occ0, tsample_t s)
 		}
 		oldcodep = codep;
 		if (code >= 256) {
+			char *op_orig = op;
 			/*
 		 	 * Code maps to a string, copy string
 			 * value to output (written in reverse).
@@ -685,7 +686,7 @@ LZWDecodeCompat(TIFF* tif, tidata_t op0, tsize_t occ0, tsample_t s)
 			tp = op;
 			do {
 				*--tp = codep->value;
-			} while( (codep = codep->next) != NULL);
+			} while( (codep = codep->next) != NULL && tp > op_orig);
 		} else
 			*op++ = code, occ--;
 	}
