@@ -1,4 +1,4 @@
-/* PDFlib GmbH cvsid: $Id: tiff.h,v 1.12.2.2 2007/11/04 22:48:45 rjs Exp $ */
+/* PDFlib GmbH cvsid: $Id: tiff.h,v 1.12.2.3 2008/03/10 13:22:11 rjs Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -74,7 +74,8 @@
     typedef	short int16;
     typedef	unsigned short uint16;	/* sizeof (uint16) must == 2 */
     #if defined(__alpha) || (defined(_MIPS_SZLONG) && _MIPS_SZLONG == 64) \
-    	|| defined(__LP64__) || defined(__arch64__) || defined(__sparcv9)
+    	|| defined(__LP64__) || defined(__arch64__) || defined(__sparcv9) \
+	|| (defined(__x86_64__) && defined(__SunOS))
     typedef	int int32;
 	#ifndef _UINT32
     typedef	unsigned int uint32;	/* sizeof (uint32) must == 4 */
